      SUBROUTINE BACKWARD_PROPAGATION
C
C***
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C  MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C
C
C  Centre National de Recherches Meteorologiques
C
C***
C
C Purpose   : Propagate x* backward along the assimilation window
C             It is a sum for the 3DFGAT
C             and the adjoint of the linear tangent model for the 4DINC
C
C Original  : A. Piacentini, CERFACS, 07/2006
C
C**
C
C
      USE palmlib
      USE palm_user_param
C
      IMPLICIT NONE
C
C Local Declaration
C
      INTEGER :: il_starttime, il_endtime, il_restime
      INTEGER :: il_dimx
      INTEGER :: ib_time
      INTEGER :: il_dt0

      REAL, DIMENSION(:), ALLOCATABLE :: rda_xstar
      REAL, DIMENSION(:), ALLOCATABLE :: rda_xstar0
C
C PALM Declaration
C
      CHARACTER(LEN=PL_LNAME) cl_name, cl_space
      INTEGER il_time, il_usrtag, il_err, il_err2
      INTEGER il_day, il_month, il_year, il_hour, il_min, il_sec
C
C***
C
C**  ALLOCATE MEMORY
C
      il_dimx = ip_lono1*ip_lato1*ip_nivm1*ip_nassim
      ALLOCATE( rda_xstar(il_dimx),STAT=il_err)
      IF (il_err .ne. 0) THEN
        WRITE(*,*) 'Bkwd prop: Error allocating rda_xstar'
        WRITE(PL_OUT,*) 'Bkwd prop: Error allocating rda_xstar'
        CALL PALM_Abort(il_err)
      ENDIF
      ALLOCATE( rda_xstar0(il_dimx),STAT=il_err)
      IF (il_err .ne. 0) THEN
        WRITE(*,*) 'Bkwd prop: Error allocating rda_xstar0'
        WRITE(PL_OUT,*) 'Bkwd prop: Error allocating rda_xstar0'
        CALL PALM_Abort(il_err)
      ENDIF
C
C**  Get Loop informations
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_starttime'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $    il_starttime,il_err)
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_endtime'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $    il_endtime,il_err)
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_step'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $    il_dt0,il_err)
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_resulttime'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $    il_restime,il_err)
C
C**  INITIALIZE THE RESULT 
C
      rda_xstar0(:) = 0.
C
      DO ib_time = il_endtime-il_dt0, il_starttime, -il_dt0
C
C**  GET THE TWO VECTORs xstar for each time window and propagate them
C
        cl_space = 'increment_space'
        il_time = ib_time
        il_usrtag = PL_NO_TAG
        cl_name = 'rda_xstar_e'
        CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $      rda_xstar,il_err)
C 
#ifdef USE_BLAS
        CALL DAXPY(il_dimx,1.,rda_xstar,1,rda_xstar0,1)
#else
        rda_xstar0(:) = rda_xstar0(:) + rda_xstar(:)
#endif
C
C*   HERE in the 4DINC there would be a backward propagation
C*   of rda_xstar0 from ib_time+rl_dt0 to ib_time
C*   in the 3DFGAT this is the identity
C
        cl_space = 'increment_space'
        il_time = ib_time
        il_usrtag = PL_NO_TAG
        cl_name = 'rda_xstar_b'
        CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $      rda_xstar,il_err) 
C 
#ifdef USE_BLAS
        CALL DAXPY(il_dimx,1.,rda_xstar,1,rda_xstar0,1)
#else
        rda_xstar0(:) = rda_xstar0(:) + rda_xstar(:)
#endif
C
      END DO
C  
C**  SEND THE RESULT
C
      cl_space = 'increment_space'
      il_time = il_restime
      il_usrtag = PL_NO_TAG
      cl_name = 'rda_xstar'
      CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,
     $    rda_xstar0,il_err) 
C
C**  DEALLOCATE MEMORY
C
      DEALLOCATE(rda_xstar,STAT=il_err)
      IF (il_err .ne. 0) THEN
        WRITE(*,*) 'Bkwd prop: Error deallocating rda_xstar'
        WRITE(PL_OUT,*) 'Bkwd prop: Error deallocating rda_xstar'
        CALL PALM_Abort(il_err)
      ENDIF
      DEALLOCATE(rda_xstar0,STAT=il_err)
      IF (il_err .ne. 0) THEN
        WRITE(*,*) 'Bkwd prop: Error deallocating rda_xstar'
        WRITE(PL_OUT,*) 'Bkwd prop: Error deallocating rda_xstar'
        CALL PALM_Abort(il_err)
      ENDIF
C
      END SUBROUTINE BACKWARD_PROPAGATION
