      MODULE DATDAT_MOD
C
C------------------------------------------------------------------------
C 	DATA ASSIMILATION
C
C	AUTHOR	: Andrea Piacentini	- CERFACS -
C	DATE		: 02/05/2007
C	VERSION	: 2_0_0
C------------------------------------------------------------------------
C
C  PrePALM constants
C
      USE palm_user_param
C
      IMPLICIT NONE
C
C** Observational data description
C
C ig_nobs : total number of observation locations
      INTEGER,                        SAVE :: ig_nobs
C
C + FOR EACH LOCATION
C . rga_obslat   : location latitude
C . rga_obslon   : location longitude
C . iga_inst     : the index of the corresponding instrument in cga_inst
C . cga_str_id   : the identifier string
C . iga_int_id   : the identifier integer
C . iga_dateobs  : the absolute date on 5 integers
C . iga_secobs   : the relative date in seconds
C . iga_npreslev : the number of the computed vertical levels for the location
C . iga_nobslev  : the number of the observed vertical levels for the location
C . iga_iq_vertlev : the number of the interm iq. vert levels for the location
      REAL(KIND=8),      ALLOCATABLE, SAVE :: rga_obslat(:)
      REAL(KIND=8),      ALLOCATABLE, SAVE :: rga_obslon(:)
      INTEGER,           ALLOCATABLE, SAVE :: iga_inst(:)
      CHARACTER(LEN=6),  ALLOCATABLE, SAVE :: cga_str_id(:)
      INTEGER,           ALLOCATABLE, SAVE :: iga_int_id(:)
      INTEGER,           ALLOCATABLE, SAVE :: iga_dateobs(:,:)
      INTEGER,           ALLOCATABLE, SAVE :: iga_secobs(:)
      INTEGER,           ALLOCATABLE, SAVE :: iga_npreslev(:)
      INTEGER,           ALLOCATABLE, SAVE :: iga_zerolev(:)
      INTEGER,           ALLOCATABLE, SAVE :: iga_nobslev(:)
      INTEGER,           ALLOCATABLE, SAVE :: iga_iq_vertlev(:)
C
C ++ FOR EACH LEVEL OF EACH LOCATION
C .. rga_preslev : the pressure level (2 bounds for integrated quantities)
C .. iga_nbconst : number of observed species at this position
      REAL(KIND=8),      ALLOCATABLE, SAVE :: rga_preslev(:,:,:)
      INTEGER,           ALLOCATABLE, SAVE :: iga_nbconst(:,:)
C
C +++ FOR EACH SPECIES AT EACH LEVEL OF EACH POSITION
C ... cga_constname : the species identifier
C ... iga_tshold : the dynamic rejection flag
C ... rga_val : the observed value	
C ... rga_Hx : model equivalent to observation
C ... rga_misfit : misfit on computed levels
      CHARACTER(LEN=5),  ALLOCATABLE, SAVE :: cga_constname(:,:,:)
      INTEGER,           ALLOCATABLE, SAVE :: iga_tshold(:,:,:)
      REAL(KIND=8),      ALLOCATABLE, SAVE :: rga_val(:,:,:)
      REAL(KIND=8),      ALLOCATABLE, SAVE :: rga_Hx(:,:,:)	
      REAL(KIND=8),      ALLOCATABLE, SAVE :: rga_misfit(:)	
      REAL(KIND=8),      ALLOCATABLE, SAVE :: rga_invR(:,:,:,:)
C
C** Instruments description
C        
C ig_ninst : number of different instruments
      INTEGER,                        SAVE :: ig_ninst
C + FOR EACH INSTRUMENT
C . cga_inst : instrument description
C . lga_avg_kernel : the instrument uses an averaging kernel
C . iga_avk_idx : the index of the corresp av. kern. in the avk structures
C . lga_tot_col : the instrument measures integrated quantities
      CHARACTER(LEN=68), ALLOCATABLE, SAVE :: cga_inst(:)
      LOGICAL,           ALLOCATABLE, SAVE :: lga_avg_kernel(:)
      INTEGER,           ALLOCATABLE, SAVE :: iga_avk_idx(:)
      LOGICAL,           ALLOCATABLE, SAVE :: lga_tot_col(:)
      LOGICAL,           ALLOCATABLE, SAVE :: lga_int_qty(:)
C
C** Averaging kernels
C
C rga_avk : the avk matrix for each profile requiring an av. kern.
      REAL(KIND=8),      ALLOCATABLE, SAVE :: rga_avk(:,:,:,:)
C
C*** Allocation routines
C
      CONTAINS
C
      SUBROUTINE ALLOC_DATDAT
C
      ALLOCATE (rga_obslat(ip_nobsmax))
      ALLOCATE (rga_obslon(ip_nobsmax))
      ALLOCATE (iga_inst(ip_nobsmax))
      ALLOCATE (cga_str_id(ip_nobsmax))
      ALLOCATE (iga_int_id(ip_nobsmax))
      ALLOCATE (iga_dateobs(ip_nobsmax,5))
      ALLOCATE (iga_secobs(ip_nobsmax))
      ALLOCATE (iga_npreslev(ip_nobsmax))
      ALLOCATE (iga_zerolev(ip_nobsmax))
      ALLOCATE (iga_nobslev(ip_nobsmax))
      ALLOCATE (iga_iq_vertlev(ip_nobsmax))
      ALLOCATE (rga_preslev(ip_nobsmax,ip_nbnivpresmax,2))
      ALLOCATE (iga_nbconst(ip_nobsmax,ip_nbnivpresmax))
      ALLOCATE (cga_constname(ip_nobsmax,ip_nbnivpresmax,ip_nbconstmax))
      ALLOCATE (iga_tshold(ip_nobsmax,ip_nbnivpresmax,ip_nbconstmax))
      ALLOCATE (rga_val(ip_nobsmax,ip_nbnivpresmax,ip_nbconstmax))
      ALLOCATE (rga_Hx(ip_nobsmax,ip_nbnivpresmax,ip_nbconstmax))
      ALLOCATE (rga_misfit(ip_nobstspmax*ip_nbnivpresmax*ip_nbconstmax))
      ALLOCATE
     $  (rga_invR(ip_nobstspmax,ip_nbnivpresmax,ip_nbnivpresmax,
     $    ip_nbconstmax))
      rga_Hx(:,:,:) = 0.
      rga_misfit(:) = 0.
      rga_invR(:,:,:,:) = 0.
      iga_tshold(:,:,:) = 3
      iga_npreslev(:)=0
      iga_zerolev(:)=0
      iga_nobslev(:)=0
C
      ALLOCATE (cga_inst(ip_ninstmax)) 
      ALLOCATE (lga_avg_kernel(ip_ninstmax)) 
      ALLOCATE (iga_avk_idx(ip_nobsmax))
      ALLOCATE (lga_tot_col(ip_ninstmax))
      ALLOCATE (lga_int_qty(ip_ninstmax))
      iga_avk_idx(:) = 0
C
      END SUBROUTINE ALLOC_DATDAT
C
      SUBROUTINE DEALLOC_DATDAT
C
      DEALLOCATE (rga_obslat)
      DEALLOCATE (rga_obslon)
      DEALLOCATE (iga_inst)
      DEALLOCATE (cga_str_id)
      DEALLOCATE (iga_int_id)
      DEALLOCATE (iga_dateobs)
      DEALLOCATE (iga_secobs)
      DEALLOCATE (iga_npreslev)
      DEALLOCATE (iga_zerolev)
      DEALLOCATE (iga_nobslev)
      DEALLOCATE (iga_iq_vertlev)
      DEALLOCATE (rga_preslev)
      DEALLOCATE (iga_nbconst)
      DEALLOCATE (cga_constname)
      DEALLOCATE (iga_tshold)
      DEALLOCATE (rga_val)
      DEALLOCATE (rga_Hx)
      DEALLOCATE (rga_misfit)
      DEALLOCATE (rga_invR)
C
      DEALLOCATE (cga_inst)
      DEALLOCATE (lga_avg_kernel)
      DEALLOCATE (iga_avk_idx)
      DEALLOCATE (lga_tot_col) 
      DEALLOCATE (lga_int_qty) 
C
      END SUBROUTINE DEALLOC_DATDAT
C
      SUBROUTINE ALLOC_DATAVK
C
      ALLOCATE (rga_avk(ip_nobsmax,ip_nbniv_avk_out,ip_nbniv_avk_in,
     $    ip_nbconstmax))
      rga_avk(:,:,:,:) = 0.
C
      END SUBROUTINE ALLOC_DATAVK
C
      SUBROUTINE ALLOC_DATAVK_IQ
C
      ALLOCATE (rga_avk(ip_nobsmax,1,ip_nbniv_avk_in,
     $    ip_nbconstmax))
      rga_avk(:,:,:,:) = 0.
C
      END SUBROUTINE ALLOC_DATAVK_IQ
C
      SUBROUTINE DEALLOC_DATAVK
C
      INTEGER :: il_err
C
      DEALLOCATE (rga_avk,STAT=il_err)
      IF (il_err .ne. 0) THEN
	  WRITE(*,*) 'Datavk: Error deallocating avk'
      ENDIF
C
      END SUBROUTINE DEALLOC_DATAVK
C
      END MODULE DATDAT_MOD
