SUBROUTINE HOR_COREL_DIR(rda_v,id_nprocs,id_myid)
!
! 0. DECLARATION
!------------------
!
! 0.1. Modules declaration
!
  USE palm_user_param
  USE palmlib
  USE sqrtB_mod
!
! 0.2. Implicit
!
  IMPLICIT NONE
!
#include "mpif.h"
!
! 0.3. Dummy variables
!
  REAL, DIMENSION(ip_lonc1,ip_latc1,ip_nivm1),TARGET :: rda_v
  INTEGER :: id_nprocs,id_myid
!
! 0.4. Local variables
!    
  REAL, DIMENSION(:,:,:),POINTER :: rla_w
  INTEGER :: il_nivstart,il_nivend,il_err
  INTEGER :: ib_lat, ib_niv
!
!-------------------------
!
!
!  1.1. Compute the loop boundaries
!
  IF (id_nprocs.GT.1) THEN
      il_nivstart=id_myid*(ip_nivm1/id_nprocs)+1
      IF (id_myid .NE. id_nprocs-1) THEN
	  il_nivend=(id_myid+1)*(ip_nivm1/id_nprocs)
      ELSE
	  il_nivend=ip_nivm1
      ENDIF
!
      ALLOCATE (rla_w(ip_lonc1,ip_latc1,ip_nivm1))
  ELSE
      il_nivstart = 1
      il_nivend = ip_nivm1
!
      rla_w => rda_v
  ENDIF
!
!  1.2. Apply W^(-1/2)
! 
  DO ib_niv = 1,il_nivstart-1
    rla_w(:,:,ib_niv) = 0.
  END DO
  DO ib_niv = il_nivstart,il_nivend
      rla_w(:,:,ib_niv) = rda_v(:,:,ib_niv) 
  END DO
  DO ib_niv = il_nivend+1,ip_nivm1
    rla_w(:,:,ib_niv) = 0.
  END DO
!
!  1.3. Apply L^(1/2)
!  
  CALL DIF_STATE(rla_w,il_nivstart,il_nivend)
!
!  1.4. Apply normalisation
!  
  DO ib_niv = il_nivstart,il_nivend
    rla_w(:,:,ib_niv) = rla_w(:,:,ib_niv) * rg_filno
  END DO
!
!  1.4. Collect the result
!
  IF (id_nprocs.GT.1) THEN
      CALL MPI_AllReduce (rla_w,rda_v, ip_lonc1*ip_latc1*ip_nivm1, &
	 MPI_DOUBLE_PRECISION,MPI_SUM,PL_COMM_EXEC,il_err)
!
      DEALLOCATE(rla_w)
  END IF
!
END SUBROUTINE HOR_COREL_DIR
!
!---------------------------------------------------------------
!
SUBROUTINE HOR_COREL_ADJ(rda_v,id_nprocs,id_myid)
!
! 0. DECLARATION
!------------------
!
! 0.1. Modules declaration
!
  USE palm_user_param
  USE palmlib
  USE sqrtB_mod
!
! 0.2. Implicit
!
  IMPLICIT NONE
!
#include "mpif.h"
!
! 0.3. Dummy variables
!
  REAL, DIMENSION(ip_lonc1,ip_latc1,ip_nivm1),TARGET :: rda_v
  INTEGER :: id_nprocs,id_myid
!
! 0.4. Local variables
!    
  REAL, DIMENSION(:,:,:),POINTER :: rla_w
  INTEGER :: il_nivstart,il_nivend,il_err
  INTEGER :: ib_lat, ib_niv
!
!-------------------------
!
!  1.1. Compute the loop boundaries
!
  IF (id_nprocs.GT.1) THEN
      il_nivstart=id_myid*(ip_nivm1/id_nprocs)+1
      IF (id_myid .NE. id_nprocs-1) THEN
	  il_nivend=(id_myid+1)*(ip_nivm1/id_nprocs)
      ELSE
	  il_nivend=ip_nivm1
      ENDIF
!
      ALLOCATE (rla_w(ip_lonc1,ip_latc1,ip_nivm1))
  ELSE
      il_nivstart = 1
      il_nivend = ip_nivm1
!
      rla_w => rda_v
  ENDIF
!
!  1.2. Apply normalisation 
!  
  DO ib_niv = 1,il_nivstart-1
    rla_w(:,:,ib_niv) = 0.
  END DO
  DO ib_niv = il_nivstart,il_nivend
    rla_w(:,:,ib_niv) = rda_v(:,:,ib_niv) * rg_filno
  END DO
  DO ib_niv = il_nivend+1,ip_nivm1
    rla_w(:,:,ib_niv) = 0.
  END DO
!
!  1.3. Apply W_{s} L^(1/2) W_{s}^{-1}
!  
  DO ib_niv = il_nivstart,il_nivend
    DO ib_lat = 1, ip_latc1
      rla_w(:,ib_lat,ib_niv) = rla_w(:,ib_lat,ib_niv) &
                             / dga_wts(ib_lat) 
    ENDDO
  END DO
!
  CALL DIF_STATE(rla_w,il_nivstart,il_nivend)
!
!  1.4. Apply W^(-1/2)
!  
  DO ib_niv = il_nivstart,il_nivend
    DO ib_lat = 1, ip_latc1
      rla_w(:,ib_lat,ib_niv) = rla_w(:,ib_lat,ib_niv) &
                             * dga_wts(ib_lat)
!                             * SQRT(dga_wts(ib_lat))
    ENDDO
  END DO
!
!  1.4. Collect the result
!
  IF (id_nprocs.GT.1) THEN
      CALL MPI_AllReduce (rla_w,rda_v, ip_lonc1*ip_latc1*ip_nivm1, &
	 MPI_DOUBLE_PRECISION,MPI_SUM,PL_COMM_EXEC,il_err)
!
      DEALLOCATE(rla_w)
  END IF
!
END SUBROUTINE HOR_COREL_ADJ
!
!---------------------------------------------------------------
!
SUBROUTINE DIF_STATE(rda_v,id_nivstart,id_nivend)
!
!***
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!  MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!
!
!  Centre National de Recherches Meteorologiques
!
!***
!
! Purpose   : Compute dx=U.v where B=U.U^T
!
! Original  : S. Massart, CERFACS/GLOBC, 01/2006
!
!**
!
! 0. DECLARATION
!------------------
!
! 0.1. Modules declaration
!
    USE palm_user_param
    USE sqrtB_mod
!
! 0.2. Implicit
!
    IMPLICIT NONE
!
! 0.4. Dummy variables
!
    REAL, DIMENSION(ip_lonc1,ip_latc1,ip_nivm1) :: rda_v
!
! 0.5. Local variables
!    
    INTEGER :: ib_do
    INTEGER :: id_nivstart,id_nivend
    INTEGER :: il_nt, il_idx
!
    LOGICAL, SAVE :: firsttime = .TRUE.
!
    REAL :: rl_beta
!
    REAL, DIMENSION(:,:,:), ALLOCATABLE :: work
    REAL, DIMENSION(:), ALLOCATABLE, SAVE :: rla_hmu
!
! 1. COEF COMPUTATION
!------------------
!
  IF (firsttime) THEN 
!
    ALLOCATE( rla_hmu(ip_latc1+1) )
!
    rl_beta = sin(REAL(ig_flsh)*rg_pi/360.)**2
!    rl_beta = 1.
!
    rla_hmu = 1.
    DO ib_do = 1, (ip_latc1 + 1)
      rla_hmu(ib_do) = exp(-REAL(ib_do*(ib_do-1))*rl_beta)
    ENDDO
!    
    firsttime = .FALSE.
!    
  ENDIF
!
! 1. MEMORY ALLOCATION
!------------------
!
    il_nt = id_nivend-id_nivstart+1
    ALLOCATE( work(ip_lonc1,ip_latc1,il_nt) )
!
! 2. DIFFUSION COMPUTATION
!-------------------------
!
! 2.1. Transposition des niveaux courants et insertion dans 
!      le tableau de travail
!
    DO ib_do = 1,il_nt
      il_idx = id_nivstart - 1 + ib_do
      work(:,:,ib_do) = rda_v(:,:,il_idx)
    ENDDO
!
    CALL DIFSPHERE(ip_latc1, ip_lonc1, il_nt, work, rla_hmu)
!    
! 2.5. Extraction et transposition des niveaux courants du
!      tableau de travail
!
    DO ib_do = 1,il_nt
      il_idx = id_nivstart - 1 + ib_do
      rda_v(:,:,il_idx) = work(:,:,ib_do)
    ENDDO	
!
! 3. MEMORY DEALLOCATION
!------------------
!
    DEALLOCATE( work )
!
!      DEALLOCATE( rla_hmu )
!
END SUBROUTINE DIF_STATE
!
!
!---------------------------------------------------------------
!
SUBROUTINE TOSPHERE(nlat, nlon, nt, field, ndimab, a, b, ierror)
!
    USE SQRTB_MOD
!
    IMPLICIT NONE
!
! 1. Parameters
!
	INTEGER, PARAMETER :: isym=0
!
! 2. Dummy 
!
	INTEGER, INTENT(IN) :: nlat
	INTEGER, INTENT(IN) :: nlon
	INTEGER, INTENT(IN) :: ndimab
	INTEGER, INTENT(IN) :: nt
	REAL, DIMENSION(nlat,nlon,nt), INTENT(IN) :: field
!
	INTEGER, INTENT(OUT) :: ierror
	REAL, DIMENSION(ndimab,nlat,nt), INTENT(OUT) :: a, b
!
! 3. Local
!
	INTEGER :: ldim,l1,l2
	INTEGER :: lwork 
!
!	REAL, DIMENSION(:), ALLOCATABLE :: shwork 
	REAL, DIMENSION(:), ALLOCATABLE :: work 
!	DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: dwork
!
! 1. Array initialization 
!       l1 = min0(nlat,(nlon+2)/2) if nlon is even or
!       l1 = min0(nlat,(nlon+1)/2) if nlon is odd
        l1=min0(nlat,(nlon+2-mod(nlon,2))/2)
!       l2 = nlat/2        if nlat is even or
!       l2 = (nlat+1)/2    if nlat is odd
        l2= (nlat+mod(nlat,2))/2
!* Dimensions for spherepack analysis on a Gaussian grid
#ifndef B_COMPU
!*     Spherepack storing Legendre
        ldim = nlat*(3*(l1+l2)-2)+(l1-1)*(l2*(2*nlat-l1)-3*l1)/2+nlon+15
#else
!*     Spherepack recomputing Legendre
        ldim = nlat*(2*l2+3*l1-2)+3*l1*(1-l1)/2+nlon+15
#endif
#ifndef B_COMPU
!*     Spherepack storing Legendre
        lwork=(nt+1)*nlat*nlon
#else
!*     Spherepack recomputing Legendre
        lwork=nlat*(nlon*nt+max0(3*l2,nlon))
#endif
!
	ALLOCATE(work (lwork) ) 
!
! 2. subroutine shaec performs the spherical harmonic analysis
!
        a = 0. 
        b = 0.
        work(:)=0.
!* Perform spherepack analysis on a Gaussian grid
#ifndef B_COMPU
!*     Spherepack storing Legendre
	CALL shags(nlat,nlon,isym,nt,field,nlat,nlon,a,b,ndimab,nlat, &
             rga_shaw,ldim,work,lwork,ierror)
#else
!*     Spherepack recomputing Legendre
	CALL shagc(nlat,nlon,isym,nt,field,nlat,nlon,a,b,ndimab,nlat, &
             rga_shaw,ldim,work,lwork,ierror)
#endif
!
! 3. Array deallocation
!
	DEALLOCATE(work) 
!
END SUBROUTINE TOSPHERE

!
!---------------------------------------------------------------
!

SUBROUTINE TOGRID(nlat, nlon, nt, field, ndimab, a, b, ierror)
!
    USE SQRTB_MOD
!
    IMPLICIT NONE
!
! 1. Parameters
!
	INTEGER, PARAMETER :: isym=0
!
! 2. Dummy 
!
	INTEGER, INTENT(IN) :: nlat
	INTEGER, INTENT(IN) :: nlon
	INTEGER, INTENT(IN) :: ndimab
	INTEGER, INTENT(IN) :: nt
	REAL, DIMENSION(ndimab,nlat,nt), INTENT(IN) :: a, b
!
	INTEGER, INTENT(OUT) :: ierror
	REAL, DIMENSION(nlat,nlon,nt), INTENT(OUT) :: field
!
! 3. Local
!
	INTEGER :: ldim,l1,l2
	INTEGER :: lwork 
!
	REAL, DIMENSION(:), ALLOCATABLE :: shwork 
	REAL, DIMENSION(:), ALLOCATABLE :: work 
	DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: dwork
!
! 1. Array initialization 
!
!       l1 = min0(nlat,(nlon+2)/2) if nlon is even or
!       l1 = min0(nlat,(nlon+1)/2) if nlon is odd
        l1=min0(nlat,(nlon+2-mod(nlon,2))/2)
!       l2 = nlat/2        if nlat is even or
!       l2 = (nlat+1)/2    if nlat is odd
        l2= (nlat+mod(nlat,2))/2
!* Dimensions for spherepack synthesis on a Gaussian grid
#ifndef B_COMPU
!*     Spherepack storing Legendre
        ldim = nlat*(3*(l1+l2)-2)+(l1-1)*(l2*(2*nlat-l1)-3*l1)/2+nlon+15
#else
!*     Spherepack recomputing Legendre
        ldim = nlat*(2*l2+3*l1-2)+3*l1*(1-l1)/2+nlon+15
#endif
#ifndef B_COMPU
!*     Spherepack storing Legendre
        lwork=(nt+1)*nlat*nlon
#else
!*     Spherepack recomputing Legendre
        lwork=nlat*(nlon*nt+max0(3*l2,nlon))
#endif
!
	ALLOCATE(work (lwork) ) 
!
! 2. subroutine shsec performs the spherical harmonic synthesis
!
        work(:)=0.
!* Perform spherepack synthesis on a Gaussian grid
#ifndef B_COMPU
!*     Spherepack storing Legendre
	CALL shsgs(nlat,nlon,isym,nt,field,nlat,nlon,a,b,ndimab,nlat, &
             rga_shsw,ldim,work,lwork,ierror)
#else
!*     Spherepack recomputing Legendre
	CALL shsgc(nlat,nlon,isym,nt,field,nlat,nlon,a,b,ndimab,nlat, &
             rga_shsw,ldim,work,lwork,ierror)
#endif
!
! 3. Array deallocation
!
	DEALLOCATE(work) 
!
END SUBROUTINE TOGRID
!
!---------------------------------------------------------------
!
SUBROUTINE DIFSPHERE(nlat, nlon, nt, v, mu)
!
! Routine pour diffuser de facon homogene et isotrope 
! un scalaire sur la sphere en utilisant SPHEREPACK
!
! 0. DECLARATION
!------------------
!
! 0.1. Implcit
!
      USE sqrtB_mod
	IMPLICIT NONE
!
! 0.2. Dummy declarations
!
	INTEGER, INTENT(IN) :: nlat
	INTEGER, INTENT(IN) :: nlon
	INTEGER, INTENT(IN) :: nt
	REAL, DIMENSION(nlat), INTENT(IN) :: mu
!
	REAL, DIMENSION(nlon,nlat,nt), INTENT(INOUT) :: v 
!
! 0.3. Local declaration
!
	INTEGER :: n, m
	INTEGER :: ilat, ilon, iniv, ierror, ndimab
	REAL :: coef
	REAL, DIMENSION(:,:,:), ALLOCATABLE :: a, b
	REAL, DIMENSION(:,:,:), ALLOCATABLE :: alap, blap
	REAL, DIMENSION(:,:,:), ALLOCATABLE :: work
!
	REAL :: pi
	REAL :: dp
	REAL, DIMENSION(nlat) :: wsav
!
! 1. MEMORY ALLOCATION
! ---------------------
!
	ndimab=min0(nlat,(nlon+2)/2)
	ALLOCATE ( a(ndimab,nlat,nt) )
	ALLOCATE ( b(ndimab,nlat,nt) )
	ALLOCATE ( alap(ndimab,nlat,nt) )
	ALLOCATE ( blap(ndimab,nlat,nt) )
	ALLOCATE ( work(nlat,nlon,nt) )
!
! 2. GO TO SPHERIC HARMONIC SPACE
! --------------------------------
!
	CALL geo2maths(0, nlon, nlat, nt, v, work)
	CALL TOSPHERE(nlat, nlon, nt, work, ndimab, a, b, ierror)
!
!
! 3. DIFFUSION INTO THE SPHERIC HARMONIC SPACE
! --------------------------------
!
!
      alap = 0.
      blap = 0.
!      
      alap(:,1,:) = a(:,1,:) 
      blap(:,1,:) = b(:,1,:) 
      DO n=2,nlat
        alap(1,n,:) = mu(n)*a(1,n,:)
        blap(1,n,:) = mu(n)*b(1,n,:)
      ENDDO
!
      DO  m=2,(ip_trunc + 1)
        DO  n=m,nlat
          alap(m,n,:) = mu(n)*a(m,n,:)
          blap(m,n,:) = mu(n)*b(m,n,:)
        ENDDO
      ENDDO
!
!
! 4. COME BACK TO GRID POINT SPACE
! --------------------------------
!
      work = 0.
      CALL TOGRID(nlat, nlon, nt, work, ndimab, alap, blap, ierror)
      v = 0.
      CALL math2geos(0, nlat, nlon, nt, work, v)
!
! 5. MEMORY DEALLOCATION
! ---------------------
!
	DEALLOCATE ( a )
	DEALLOCATE ( b )
	DEALLOCATE ( alap )
	DEALLOCATE ( blap )
	DEALLOCATE ( work )
!
END SUBROUTINE DIFSPHERE
!
!---------------------------------------------------------------
