SUBROUTINE INIVCOR(id_nprocs,id_myid,id_assimtime)
    !
    ! 0. DECLARATION
    !------------------
    !
    ! 0.1. Modules declaration
    !
    USE palm_user_param
    USE SQRTB_MOD
    USE palmlib
    !
    ! 0.2. Implicit
    !
    IMPLICIT NONE
    !
#include "mpif.h"
    !
    ! 0.3. Parameter variables
    !
    REAL, PARAMETER :: rp_dt_vcor = 1./ip_niter_vcor 
    !
    ! 0.4. Dummy variables
    !
    INTEGER :: id_nprocs
    INTEGER :: id_myid
    INTEGER :: id_assimtime
    !
    ! 0.5 PALM Declaration
    !  
    CHARACTER(LEN=PL_LNAME) :: cl_name, cl_space
    INTEGER :: il_time, il_usrtag
    !
    ! 0.5. Local variables
    !
    INTEGER :: il_info
    INTEGER :: il_loopstart
    INTEGER :: il_loopend
    INTEGER :: il_err
    !
    REAL :: rl_pi
    !
    REAL, DIMENSION(:), ALLOCATABLE :: rla_cfl_max
    REAL, DIMENSION(:), ALLOCATABLE :: rla_lpz
    REAL, DIMENSION(:,:,:), ALLOCATABLE :: rla_nu
    !
    REAL, DIMENSION(:,:,:), ALLOCATABLE :: rla_dz_moy
    REAL, DIMENSION(:,:,:), ALLOCATABLE :: rla_dz_sup
    REAL, DIMENSION(:,:,:), ALLOCATABLE :: rla_dz_inf
    REAL, DIMENSION(:,:,:), ALLOCATABLE :: rla_nu_sup
    REAL, DIMENSION(:,:,:), ALLOCATABLE :: rla_nu_inf
    !
    REAL, DIMENSION(:,:), ALLOCATABLE :: rla_cksup
    REAL, DIMENSION(:,:), ALLOCATABLE :: rla_ckinf
    !
    REAL, DIMENSION(:,:,:,:), ALLOCATABLE :: rla_mat_sup
    REAL, DIMENSION(:,:,:,:), ALLOCATABLE :: rla_mat_inf
    REAL, DIMENSION(:,:), ALLOCATABLE :: rla_mat_sup_inv
    REAL, DIMENSION(:,:), ALLOCATABLE :: rla_mat_inf_inv
    REAL, ALLOCATABLE, DIMENSION(:,:) :: rla_mat_vcor_sqrt
    REAL, ALLOCATABLE, DIMENSION(:,:) :: rla_mat_vcor_sup
    REAL, ALLOCATABLE, DIMENSION(:,:) :: rla_mat_vcor_inf
    !
    ! 0.4. Loop variables
    !
    INTEGER :: ib_lat
    INTEGER :: ib_lon
    INTEGER :: ib_niv
    INTEGER :: ib_iter
    !
    IF (id_myid.eq.0) THEN
	WRITE(*,*)'+ SUBROUTINE INIVCOR'
	IF (lg_vcor_1D) THEN
	    WRITE(*,*) '+ USES AN AVERAGE CONSTANT PRESSURE PROFILE'
	ELSE
	    WRITE(*,*) '+ USES A FULL 3D PRESSURE FIELD'
	ENDIF
	WRITE(PL_OUT,*)'+ SUBROUTINE INIVCOR'
	IF (lg_vcor_1D) THEN
	    WRITE(PL_OUT,*) '+ USES AN AVERAGE CONSTANT PRESSURE PROFILE'
	ELSE
	    WRITE(PL_OUT,*) '+ USES A FULL 3D PRESSURE FIELD'
	ENDIF
    ENDIF
    !
    ! 5. DOMAIN BOUNDS
    !------------------
    !
     IF (id_nprocs.GT.1) THEN
       il_loopstart=id_myid*(ip_lono1/id_nprocs)+1
       IF (id_myid .NE. id_nprocs-1) THEN
 	  il_loopend=(id_myid+1)*(ip_lono1/id_nprocs)
       ELSE
 	  il_loopend=ip_lono1
       ENDIF
     ELSE
       il_loopstart = 1
       il_loopend = ip_lono1
     ENDIF
    !
    ! 1. ALLOCATION MEMOIRES
    !----------------------------------
    !
    IF (lg_vcor_1D) THEN
      ALLOCATE(rla_dz_moy(1,1,1:ip_nivm1))
      ALLOCATE(rla_dz_sup(1,1,1:ip_nivm1))
      ALLOCATE(rla_dz_inf(1,1,1:ip_nivm1))
      ALLOCATE(rla_nu_sup(1,1,1:ip_nivm1))
      ALLOCATE(rla_nu_inf(1,1,1:ip_nivm1))
      !
      ALLOCATE(rla_cksup(1,1))
      ALLOCATE(rla_ckinf(1,1))
    ELSE
      ALLOCATE(rla_dz_moy(il_loopstart:il_loopend,ip_lato1,1:ip_nivm1))
      ALLOCATE(rla_dz_sup(il_loopstart:il_loopend,ip_lato1,1:ip_nivm1))
      ALLOCATE(rla_dz_inf(il_loopstart:il_loopend,ip_lato1,1:ip_nivm1))
      ALLOCATE(rla_nu_sup(il_loopstart:il_loopend,ip_lato1,1:ip_nivm1))
      ALLOCATE(rla_nu_inf(il_loopstart:il_loopend,ip_lato1,1:ip_nivm1))
      !
      ALLOCATE(rla_cksup(il_loopstart:il_loopend,ip_lato1))
      ALLOCATE(rla_ckinf(il_loopstart:il_loopend,ip_lato1))
    ENDIF 
    rla_dz_moy(:,:,:)=0.
    rla_dz_sup(:,:,:)=0.
    rla_dz_inf(:,:,:)=0.
    rla_nu_sup(:,:,:)=0.
    rla_nu_inf(:,:,:)=0.
    rla_cksup(:,:)=0.
    rla_ckinf(:,:)=0.
    !
    ! 2. INITIALISATION DES LONGUEURS DE PORTES
    !----------------------------------
    !
    ALLOCATE( rla_lpz(ip_nivm1))
    rla_lpz(:)=0.
    IF (lg_vcor_1D) THEN
	IF (.NOT.ALLOCATED(rga_filnoz)) &
	   ALLOCATE( rga_filnoz(1,1,ip_nivm1))
	ALLOCATE( rla_nu(1,1,ip_nivm1))
    ELSE
	IF (.NOT.ALLOCATED(rga_filnoz)) &
	   ALLOCATE( rga_filnoz(il_loopstart:il_loopend,ip_lato1,ip_nivm1))
	ALLOCATE( rla_nu(il_loopstart:il_loopend,ip_lato1,ip_nivm1))
    ENDIF 
    rga_filnoz(:,:,:)=0.
    rla_nu(:,:,:)=0.
    !
    cl_space = 'vert_vector'
    il_time = id_assimtime
    il_usrtag = PL_NO_TAG
    cl_name = 'vert_corr_length'
    CALL PALM_Get(cl_space, cl_name, il_time, il_usrtag, rla_lpz, il_err) 
    !
    rl_pi = 4. * ATAN(1.)
    !
    DO ib_niv = 1, ip_nivm1
      rla_nu(:,:,ib_niv) = rla_lpz(ib_niv)**2  / 2.
      rga_filnoz(:,:,ib_niv) = SQRT( SQRT(2. * rl_pi) * rla_lpz(ib_niv))
    ENDDO
    DEALLOCATE( rla_lpz)
    !
    ! 1. CALCUL DES DELTA Z & COEF nu
    !----------------------------------
    !
    DO ib_niv = 2, ip_nivm1-1
      rla_dz_sup(:,:,ib_niv) &
              = (rga_logp(:,:,ib_niv+1) - rga_logp(:,:,ib_niv))
      rla_dz_inf(:,:,ib_niv) &
              = (rga_logp(:,:,ib_niv) - rga_logp(:,:,ib_niv-1))
      rla_nu_sup(:,:,ib_niv) &
              = .5 * (rla_nu(:,:,ib_niv+1) + rla_nu(:,:,ib_niv))
      rla_nu_inf(:,:,ib_niv) &
              = .5 * (rla_nu(:,:,ib_niv) + rla_nu(:,:,ib_niv-1))
    ENDDO
    !
    ! Conditions limites en 1 
    !
    rla_dz_sup(:,:,1) = (rga_logp(:,:,2) - rga_logp(:,:,1))
    rla_nu_sup(:,:,1) = .5 * (rla_nu(:,:,2) + rla_nu(:,:,1))
    rla_dz_inf(:,:,1) = rla_dz_inf(:,:,2)
    rla_nu_inf(:,:,1) = rla_nu_inf(:,:,2)
    !
    ! Conditions limites en ip_nivm1 
    !
    rla_dz_sup(:,:,ip_nivm1) = rla_dz_sup(:,:,ip_nivm1-1)
    rla_nu_sup(:,:,ip_nivm1) = rla_nu_sup(:,:,ip_nivm1-1)
    rla_dz_inf(:,:,ip_nivm1) &
            = (rga_logp(:,:,ip_nivm1) - rga_logp(:,:,ip_nivm1-1))
    rla_nu_inf(:,:,ip_nivm1) &
            = .5 * (rla_nu(:,:,ip_nivm1) + rla_nu(:,:,ip_nivm1-1))
    !
    ! 2. CALCUL DES DELTA Z MOYEN ET DE LA METRIQUE
    !-----------------------------------------------
    !
    rla_dz_moy = .5 * (rla_dz_sup + rla_dz_inf)
    !
    IF (lg_vcor_1D) THEN
	IF (.NOT.ALLOCATED(rga_Wz)) &
	   ALLOCATE( rga_Wz(1,1,ip_nivm1))
    ELSE
	IF (.NOT.ALLOCATED(rga_Wz)) &
	   ALLOCATE( rga_Wz(il_loopstart:il_loopend,ip_lato1,ip_nivm1))
    ENDIF
    rga_Wz(:,:,:)=0.
    rga_Wz = rla_dz_moy(:,:,1:ip_nivm1) 
    !
    ! 3. CONSTRUCTION DE LA MATRICE A INVERSER
    !------------------------------------------
    !
    IF (lg_vcor_1D) THEN
      ALLOCATE (rla_mat_sup(1,1,&
                          1:ip_nivm1,1:ip_nivm1))
      ALLOCATE (rla_mat_inf(1,1,&
                          1:ip_nivm1,1:ip_nivm1))
    ELSE
      ALLOCATE (rla_mat_sup(il_loopstart:il_loopend,ip_lato1,&
                          1:ip_nivm1,1:ip_nivm1))
      ALLOCATE (rla_mat_inf(il_loopstart:il_loopend,ip_lato1,&
                          1:ip_nivm1,1:ip_nivm1))
    ENDIF
    ALLOCATE (rla_cfl_max(1:ip_nivm1))
    rla_mat_sup(:,:,:,:) = 0.
    rla_mat_inf(:,:,:,:) = 0.
    rla_cfl_max(:) = 0.
    !
    DO ib_niv = 2, ip_nivm1-1 
      rla_cksup = rla_nu_sup(:,:,ib_niv) * rp_dt_vcor &
               / (rla_dz_moy(:,:,ib_niv) * rla_dz_sup(:,:,ib_niv))
      rla_ckinf = rla_nu_inf(:,:,ib_niv) * rp_dt_vcor &
               / (rla_dz_moy(:,:,ib_niv) * rla_dz_inf(:,:,ib_niv))
      rla_cfl_max(ib_niv) = MAX(MAXVAL(rla_cksup),MAXVAL(rla_ckinf))
      rla_mat_sup(:,:,ib_niv,ib_niv) = 1. + rla_cksup + rla_ckinf
      rla_mat_sup(:,:,ib_niv,ib_niv-1) = -rla_ckinf
      rla_mat_sup(:,:,ib_niv,ib_niv+1) = -rla_cksup
      rla_mat_inf(:,:,ib_niv,ib_niv) = 1. + rla_cksup + rla_ckinf
      rla_mat_inf(:,:,ib_niv,ib_niv-1) = -rla_ckinf
      rla_mat_inf(:,:,ib_niv,ib_niv+1) = -rla_cksup
    ENDDO
    !
    ! Conditions limites en 1 
    !
    rla_cksup = rla_nu_sup(:,:,1) * rp_dt_vcor &
             / (rla_dz_moy(:,:,1) * rla_dz_sup(:,:,1))
    rla_ckinf = rla_nu_inf(:,:,1) * rp_dt_vcor &
             / (rla_dz_moy(:,:,1) * rla_dz_inf(:,:,1))
    rla_cfl_max(1) = MAX(MAXVAL(rla_cksup),MAXVAL(rla_ckinf))
    rla_mat_sup(:,:,1,1)   = 1. + rla_cksup 
    rla_mat_inf(:,:,1,1)   = 1. + rla_cksup  + 2. * rla_ckinf
    rla_mat_sup(:,:,1,2) = -rla_cksup
    rla_mat_inf(:,:,1,2) = -rla_cksup
    !
    ! Conditions limites en ip_nivm1 
    !
    rla_cksup = rla_nu_sup(:,:,ip_nivm1) * rp_dt_vcor &
             / (rla_dz_moy(:,:,ip_nivm1) * rla_dz_sup(:,:,ip_nivm1))
    rla_ckinf = rla_nu_inf(:,:,ip_nivm1) * rp_dt_vcor &
             / (rla_dz_moy(:,:,ip_nivm1) * rla_dz_inf(:,:,ip_nivm1))
    rla_cfl_max(ip_nivm1) = MAX(MAXVAL(rla_cksup),MAXVAL(rla_ckinf))
    rla_mat_sup(:,:,ip_nivm1,ip_nivm1) = 1. + rla_ckinf
    rla_mat_inf(:,:,ip_nivm1,ip_nivm1) = 1. + 2. * rla_cksup  + rla_ckinf
    rla_mat_sup(:,:,ip_nivm1,ip_nivm1-1) = -rla_ckinf
    rla_mat_inf(:,:,ip_nivm1,ip_nivm1-1) = -rla_ckinf

    !
    ! Condition CFL
    !
    IF (MAXVAL(rla_cfl_max) .GE. 5.) THEN 
	IF (id_myid.eq.0) THEN
	    WRITE(*,*)'+'
	    WRITE(*,*)'+  WARNING: SUSPICIOUS CFL STABILITY CHECK'
	    WRITE(*,*)'+  LEVEL ', MAXLOC(rla_cfl_max), ' CFL=', MAXVAL(rla_cfl_max)
	    WRITE(*,*)'+'
	ENDIF
	WRITE(PL_OUT,*)'+'
	WRITE(PL_OUT,*)'+  WARNING: SUSPICIOUS CFL STABILITY CHECK'
	WRITE(PL_OUT,*)'+  LEVEL ', MAXLOC(rla_cfl_max), ' CFL=', MAXVAL(rla_cfl_max)
	WRITE(PL_OUT,*)'+'
    ELSE  
	IF (id_myid.eq.0) THEN	
	    WRITE(*,*)'+'
	    WRITE(*,*)'+  CFL STABILTY CHECK PASSED:'
	    WRITE(*,*)'+  LEVEL ', MAXLOC(rla_cfl_max),' CFL_MAX=', MAXVAL(rla_cfl_max)
	    WRITE(*,*)'+'
	ENDIF
	WRITE(PL_OUT,*)'+'
	WRITE(PL_OUT,*)'+  CFL STABILTY CHECK PASSED:'
	WRITE(PL_OUT,*)'+  LEVEL ', MAXLOC(rla_cfl_max),' CFL_MAX=', MAXVAL(rla_cfl_max)
	WRITE(PL_OUT,*)'+'
    ENDIF  
    !
    ! 4. Liberation memoire
    !----------------------------------
    !
    DEALLOCATE(rla_dz_moy)
    DEALLOCATE(rla_dz_sup)
    DEALLOCATE(rla_dz_inf)
    DEALLOCATE(rla_nu_sup)
    DEALLOCATE(rla_nu_inf)
    DEALLOCATE(rla_cksup)
    DEALLOCATE(rla_ckinf)
    DEALLOCATE(rla_nu)
    DEALLOCATE(rla_cfl_max)
    !
    !
    ! 6. INVERSION DE LA MATRICE 
    !------------------------------------------
    !
    ! 6.1. Allocation memoire suivant que l'on prenne la moyenne des pressions ou non
    !
    IF (lg_vcor_1D) THEN
	IF (.NOT.ALLOCATED(rga_mat_vcor)) &
	   ALLOCATE (rga_mat_vcor(1,1,1:ip_nivm1,1:ip_nivm1))
    ELSE
	IF (.NOT.ALLOCATED(rga_mat_vcor)) &
	   ALLOCATE (rga_mat_vcor(il_loopstart:il_loopend,ip_lato1,&
	   1:ip_nivm1,1:ip_nivm1))
    ENDIF
    !
    ALLOCATE (rla_mat_vcor_sqrt(1:ip_nivm1,1:ip_nivm1))
    ALLOCATE (rla_mat_vcor_sup(1:ip_nivm1,1:ip_nivm1))
    ALLOCATE (rla_mat_vcor_inf(1:ip_nivm1,1:ip_nivm1))
    rla_mat_vcor_sqrt(:,:) = 0.
    !
    ALLOCATE(rla_mat_sup_inv(1:ip_nivm1,1:ip_nivm1))
    ALLOCATE(rla_mat_inf_inv(1:ip_nivm1,1:ip_nivm1))
    rla_mat_sup_inv(:,:) = 0.
    rla_mat_inf_inv(:,:) = 0.
    !
    ! 6.2. Calcul de la matrice en implicit 
    !
    !    
    IF (lg_vcor_1D) THEN
      !
      CALL M_INVERSION(ip_nivm1, ip_nivm1, &
                       rla_mat_sup(1,1,:,:),&
                       rla_mat_sup_inv(:,:),&
                       il_info)
      IF (il_info .NE. 0) THEN
        WRITE(*,*) il_info, ib_lon, ib_lat
        CALL PALM_Abort(il_err) 
      ENDIF
      !
      CALL M_INVERSION(ip_nivm1, ip_nivm1, &
                       rla_mat_inf(1,1,:,:),&
                       rla_mat_inf_inv(:,:),&
                       il_info)
      IF (il_info .NE. 0) THEN
        WRITE(*,*) il_info, ib_lon, ib_lat
        CALL PALM_Abort(il_err) 
      ENDIF
      !
      rla_mat_vcor_sup(:,:) = rla_mat_sup_inv
      rla_mat_vcor_inf(:,:) = rla_mat_inf_inv
      DO ib_iter = 1, (ip_niter_vcor - 1)
        rla_mat_vcor_sup(:,:) = MATMUL(rla_mat_sup_inv,&
                                          rla_mat_vcor_sup(:,:))
        rla_mat_vcor_inf(:,:) = MATMUL(rla_mat_inf_inv,&
                                          rla_mat_vcor_inf(:,:))
      ENDDO 
      !
      rga_mat_vcor(1,1,:,:) &
          = (rla_mat_vcor_sup(:,:) + rla_mat_vcor_inf(:,:)) / 2.
      CALL ROOT_SQUARE(ip_nivm1, rga_mat_vcor(1,1,:,:),&
                                rla_mat_vcor_sqrt(:,:) )
      rga_mat_vcor(1,1,:,:) = rla_mat_vcor_sqrt(:,:)
!
    ELSE
!
      DO ib_lon=il_loopstart, il_loopend
      DO ib_lat=1, ip_lato1
        !
        CALL M_INVERSION(ip_nivm1, ip_nivm1, &
                         rla_mat_sup(ib_lon,ib_lat,:,:),&
                         rla_mat_sup_inv(:,:),&
                         il_info)
        IF (il_info .NE. 0) THEN
          WRITE(*,*) il_info, ib_lon, ib_lat
          CALL PALM_Abort(il_err) 
        ENDIF
        !
        CALL M_INVERSION(ip_nivm1, ip_nivm1, &
                         rla_mat_inf(ib_lon,ib_lat,:,:),&
                         rla_mat_inf_inv(:,:),&
                         il_info)
        IF (il_info .NE. 0) THEN
          WRITE(*,*) il_info, ib_lon, ib_lat
          CALL PALM_Abort(il_err) 
        ENDIF
        !
        rla_mat_vcor_sup = rla_mat_sup_inv
        rla_mat_vcor_inf = rla_mat_inf_inv
        DO ib_iter = 1, (ip_niter_vcor - 1)
          rla_mat_vcor_sup = MATMUL(rla_mat_sup_inv,&
                                            rla_mat_vcor_sup)
          rla_mat_vcor_inf = MATMUL(rla_mat_inf_inv,&
                                            rla_mat_vcor_inf)
        ENDDO 
        !
        rga_mat_vcor(ib_lon,ib_lat,:,:) &
            = (rla_mat_vcor_sup(:,:) + rla_mat_vcor_inf(:,:)) / 2.
        CALL ROOT_SQUARE(ip_nivm1, rga_mat_vcor(ib_lon,ib_lat,:,:),&
                                  rla_mat_vcor_sqrt(:,:) )
        rga_mat_vcor(ib_lon,ib_lat,:,:) = rla_mat_vcor_sqrt(:,:)
      ENDDO
      ENDDO
    ENDIF
    !
    DEALLOCATE (rla_mat_vcor_sqrt)
    DEALLOCATE (rla_mat_vcor_sup)
    DEALLOCATE (rla_mat_vcor_inf)
    DEALLOCATE (rla_mat_sup_inv)
    DEALLOCATE (rla_mat_inf_inv)
    DEALLOCATE (rla_mat_sup)
    DEALLOCATE (rla_mat_inf)
    !
    !
    ! 4. Fin de la routine
    !------------------------------------------
    !
END SUBROUTINE INIVCOR


  !
  !--------------------------------------------------------------------------------
  ! 2.2. RACINE CARRE
  !--------------------------------------------------------------------------------
  !
  !
  SUBROUTINE ROOT_SQUARE(id_n, dda_A, dda_Ademi )
    !
    ! 0. DECLARATION
    !------------------
    !
    USE palmlib
    IMPLICIT NONE 
    !
    ! Dummy Variable 
    !
    INTEGER :: id_n
    REAL, DIMENSION(id_n,id_n) :: dda_A
    REAL, DIMENSION(id_n,id_n) :: dda_Ademi
    !
    ! Local Variable 
    !
    INTEGER :: il_ilo
    INTEGER :: il_ihi
    INTEGER :: il_info
    INTEGER :: il_m
    !
    REAL, DIMENSION(:), ALLOCATABLE :: rla_scale
    REAL, DIMENSION(:), ALLOCATABLE :: rla_tau
    REAL, DIMENSION(:), ALLOCATABLE :: rla_wr
    REAL, DIMENSION(:), ALLOCATABLE :: rla_wi
    !
    REAL, DIMENSION(:,:), ALLOCATABLE :: rla_Z
    REAL, DIMENSION(:,:), ALLOCATABLE :: dda_Q
    REAL, DIMENSION(:,:), ALLOCATABLE :: dda_T
    REAL, DIMENSION(:,:), ALLOCATABLE :: dda_Vr
    REAL, DIMENSION(:,:), ALLOCATABLE :: dda_Vl
    REAL, DIMENSION(:,:), ALLOCATABLE :: dda_L
    REAL, DIMENSION(:,:), ALLOCATABLE :: dda_H
    !
    INTEGER :: il_lwork
    REAL, DIMENSION(:), ALLOCATABLE :: rla_work
    !
    ! Loop Variable 
    !
    INTEGER :: ib_i
    !
    ! BALANCES A GENERAL REAL MATRIX A.
    !----------------------------------
    !
    ! Initialisation
    !
    ALLOCATE ( dda_H(id_n,id_n)) 
    ALLOCATE( rla_scale(id_n))
    dda_H = dda_A
    !
    ! Calcul Blas
    !
    CALL dgebal('N', id_n, dda_H, id_n, il_ilo, il_ihi, rla_scale, il_info )
    IF (il_info.ne.0) THEN
      WRITE(*,*) 'SUBROUTINE ROOT_SQUARE : Erreur sgebal.'
    ENDIF
    !
    ! Liberation memoire 
    !
    DEALLOCATE( rla_scale)
    !
    ! Reduces A to upper Hessenberg form Q' * A * Q = H
    !---------------------------------------------------
    !
    ! Initialisation
    !
    ALLOCATE (rla_tau(id_n-1))
    !
    ! Calcul Blas pour rechercher la dimension lwork
    !
    ALLOCATE (rla_work(id_n))
    CALL dgehrd(id_n, il_ilo, il_ihi, dda_H, id_n, rla_tau, &
              rla_work, -1, il_info)
    il_lwork = rla_work(1)
    DEALLOCATE (rla_work)
    !
    ! Calcul Blas
    !
    ALLOCATE (rla_work(il_lwork))
    CALL dgehrd(id_n, il_ilo, il_ihi, dda_H, id_n, rla_tau, &
            rla_work, il_lwork, il_info)
    DEALLOCATE (rla_work)
    IF (il_info.ne.0) THEN
      WRITE(*,*) 'SUBROUTINE ROOT_SQUARE : Erreur sgehrd.'
    ENDIF
    !
    ! GENERATES A REAL ORTHOGONAL MATRIX Q 
    !---------------------------------------------------
    !
    ! Initialisation
    !
    ALLOCATE ( dda_Q(id_n,id_n)) 
    dda_Q = dda_H
    !
    ! Calcul Blas pour rechercher la dimension lwork
    !
    ALLOCATE (rla_work(id_n))
    CALL dorghr( id_n, il_ilo, il_ihi, dda_Q, id_n, rla_tau, &
                rla_work, -1, il_info )
    il_lwork = rla_work(1)
    DEALLOCATE (rla_work)
    !
    ! Calcul Blas
    !
    ALLOCATE (rla_work(il_lwork))
    CALL dorghr( id_n, il_ilo, il_ihi, dda_Q, id_n, rla_tau, &
                rla_work, il_lwork, il_info )
    DEALLOCATE (rla_work)
    IF (il_info.ne.0) THEN
      WRITE(*,*) 'SUBROUTINE ROOT_SQUARE : Erreur sorghr.'
    ENDIF
    !
    ! Liberation memoire 
    !
    DEALLOCATE (rla_tau)
    !
    ! COMPUTE  THE EIGENVALUES OF A HESSENBERG MATRIX H
    !---------------------------------------------------
    !
    ! Initialisation
    !
    ALLOCATE (rla_Z(id_n,id_n))
    ALLOCATE (dda_T(id_n,id_n))
    ALLOCATE (rla_wr(id_n))
    ALLOCATE (rla_wi(id_n))
    !
    dda_T = dda_H
    rla_Z = dda_Q
    !
    ! Calcul Blas pour rechercher la dimension lwork
    !
    ALLOCATE (rla_work(id_n))
    CALL dhseqr( 'S', 'V', id_n, il_ilo, il_ihi, dda_T, id_n,  &
                rla_wr,  rla_wi,  rla_Z,  id_n, rla_work, -1, il_info )
    il_lwork = rla_work(1)
    DEALLOCATE (rla_work)
    !
    ! Calcul Blas
    !
    ALLOCATE (rla_work(il_lwork))
    CALL dhseqr( 'S', 'V', id_n, il_ilo, il_ihi, dda_T, id_n,  &
                rla_wr,  rla_wi,  rla_Z,  id_n, rla_work, il_lwork, il_info )
    DEALLOCATE (rla_work)
    IF (il_info.ne.0) THEN
      WRITE(*,*) 'SUBROUTINE ROOT_SQUARE : Erreur shseqr.'
    ENDIF
    !
    ! Calcul de la matrice diagonale 
    !
    ALLOCATE (dda_L(id_n,id_n))
    dda_L(:,:) = 0.d0
    DO ib_i = 1, id_n
      IF (rla_wr(ib_i) < 0. ) THEN
        dda_L(ib_i,ib_i) = 0.
      ELSE
        dda_L(ib_i,ib_i) = SQRT(rla_wr(ib_i))
      ENDIF
    ENDDO
    !
    ! Liberation memoire 
    !
    DEALLOCATE (rla_wr)
    DEALLOCATE (rla_wi)
    DEALLOCATE (dda_Q) 
    DEALLOCATE (dda_H) 
    !
    ! COMPUTE  THE EIGENVECTOR OF THE REAL  UPPER QUASI-TRIANGULAR MATRIX T
    !---------------------------------------------------
    !
    ! Initialisation
    !
    ALLOCATE (dda_Vr(id_n,id_n))
    ALLOCATE (dda_Vl(id_n,id_n))
    !
    ! Calcul Blas
    !
    ALLOCATE (rla_work(id_n*3))
    CALL dtrevc( 'B', 'A', .TRUE., id_n, dda_T, id_n, &
                dda_Vl,  id_n,  dda_Vr,  id_n, & 
                            id_n, il_m, rla_work, il_info )
    DEALLOCATE (rla_work)
    IF (il_info.ne.0) THEN
      WRITE(*,*) 'SUBROUTINE ROOT_SQUARE : Erreur strevc.'
    ENDIF
    !
    CALL M_INVERSION(id_n, id_n, dda_Vr, dda_Vl, il_info)
    !
    dda_Ademi = MATMUL(dda_Vr,MATMUL(dda_L, dda_Vl))
    dda_Ademi = MATMUL(rla_Z,MATMUL(dda_Ademi, TRANSPOSE(rla_Z)))
    !
    ! DEALLOCATIONS GLOBALE
    !----------------------------------
    !
    DEALLOCATE (rla_Z)
    DEALLOCATE (dda_T)
    DEALLOCATE (dda_Vr)
    DEALLOCATE (dda_Vl)
    DEALLOCATE (dda_L)
    !
    ! END SUBROUTINE
    !------------------
    !
  END SUBROUTINE ROOT_SQUARE
!
! --------------------------------------------
!
! INVERSION DE MATRICES GENERALE
!
! --------------------------------------------
!
SUBROUTINE M_INVERSION(id_size, id_ldmat, rda_mat, rda_invmat, id_info)
    !
    ! 0. DECLARATION
    !------------------
    !
    ! 0.1. Modules declaration
    !
    USE palmlib
    !
    ! 0.2. Implicit
    !
    IMPLICIT NONE
    !
    ! 0.3. Dummy variables
    !
    INTEGER, INTENT(IN) :: id_size
    INTEGER, INTENT(IN) :: id_ldmat
    REAL, DIMENSION(id_ldmat,id_ldmat), INTENT(IN) :: rda_mat
    REAL, DIMENSION(id_size,id_size), INTENT(OUT) :: rda_invmat
    !
    ! 0.4. Local variables
    !
    INTEGER :: il_err, id_info
    !
    INTEGER, DIMENSION(:), ALLOCATABLE :: ila_pivot
    !
    DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: rla_work
    DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE :: rla_invmattmp
    !
    ! 1. ALLOCATION MEMOIRE DES TABLEAUX DE TRAVAIL 
    !-----------------------------------------------
    !
    ALLOCATE( ila_pivot(id_size), stat=il_err )
    IF (il_err.ne.0) THEN
      WRITE(*,*) 'SUBROUTINE M_INVERSION : Erreur allocation de ila_pivot.'
      
    ENDIF
    ila_pivot(:) = 0.
    !    
    ALLOCATE( rla_work(id_size), stat=il_err )  
    IF (il_err.ne.0) THEN
      WRITE(*,*) 'SUBROUTINE M_INVERSION : Erreur allocation de rla_work.'
      CALL PALM_Abort(il_err) 
    ENDIF
    rla_work(:) = 0.
    !    
    ALLOCATE( rla_invmattmp(id_size,id_size), stat=il_err )  
    IF (il_err.ne.0) THEN
      WRITE(*,*) 'SUBROUTINE M_INVERSION : Erreur allocation de rla_invmattmp.'
      CALL PALM_Abort(il_err) 
    ENDIF
    rla_invmattmp(:,:) = 0.
    ! 
    ! 2. EXTRACTION DES N COLONNES ET LIGNES DE LA MATRICE B
    !-------------------------------------------------------
    !   
    rla_invmattmp(1:id_size,1:id_size) = DBLE(rda_mat(1:id_size,1:id_size))
    !  
    ! 3. FACTORISATION L.U. AVANT INVERSION
    !-------------------------------------------------------
    !
    CALL DGETRF(id_size, id_size, rla_invmattmp, id_size, ila_pivot, id_info)
    IF (id_info.ne.0) THEN
        WRITE(*,*) 'SUBROUTINE M_INVERSION : Erreur lors de l''appel de DGETRF.'
        WRITE(*,*) '                           id_info = ',id_info
        RETURN
    ENDIF
    !  
    ! 4. INVERSION DE LA MATRICE 
    !-------------------------------------------------------
    !
    CALL DGETRI(id_size, rla_invmattmp, id_size, ila_pivot, rla_work, &
                id_size , id_info)
    IF (id_info.ne.0) THEN
        WRITE(*,*) 'SUBROUTINE M_INVERSION : Erreur lors de l''appel de DGETRI.'
        WRITE(*,*) '                           id_info = ',id_info
        RETURN
    ENDIF
    !  
    ! 5. COPY DE LA MATRICE INVERSE VERS LA VARIABLE DE SORTIE
    !-------------------------------------------------------
    !
    rda_invmat(:,:) = REAL(rla_invmattmp(:,:))
    !
    ! 6. LIBERATION MEMOIRE DES TABLEAUX DE TRAVAIL
    !-------------------------------------------------------
    !
    DEALLOCATE(ila_pivot, stat=il_err)
    IF (il_err.ne.0) THEN
      WRITE(*,*) 'SUBROUTINE M_INVERSION : Erreur lors de la deallocation de pivot.'
      CALL PALM_Abort(il_err) 
    ENDIF
    !
    DEALLOCATE(rla_work, stat=il_err)
    IF (il_err.ne.0) THEN
      WRITE(*,*) 'SUBROUTINE M_INVERSION : Erreur lors de la deallocation de work.'
      CALL PALM_Abort(il_err) 
    ENDIF
    !
    DEALLOCATE(rla_invmattmp, stat=il_err)
    IF (il_err.ne.0) THEN
      WRITE(*,*) 'SUBROUTINE M_INVERSION : Erreur lors de la deallocation de rla_invmattmp.'
      CALL PALM_Abort(il_err) 
    ENDIF
    !
    ! 7. FIN DE LA SUBROUTINE
    !-------------------------------------------------------
    !    
END SUBROUTINE M_INVERSION
