      SUBROUTINE INV_R()
C
      USE palmlib
C
C
C***
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C  MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C
C
C  Centre National de Recherches Meteorologiques
C
C***
C
C Purpose   : Compute inv_R * d
C
C Original  : S. Massart, CERFACS/GLOBC, 07/2003
C
C**
C
	USE palm_user_param
	USE DATDAT_MOD
C
        IMPLICIT NONE
C
C Local Declaration
C
      INTEGER :: i,lobs,ipres1,iyo1,ipres2,iyo2
      INTEGER icount
      INTEGER ILOBS(ip_nobstspmax)
C
      INTEGER, DIMENSION(5) :: idate0
C
      REAL :: daynum
      REAL, DIMENSION(:), ALLOCATABLE :: MISFIT
      REAL, DIMENSION(:,:,:,:), ALLOCATABLE :: invR  
      REAL, DIMENSION(:), ALLOCATABLE :: invRd 
C
      REAL, DIMENSION(ip_nbnivpresmax) :: rla_tshold
C
      LOGICAL LEND
C
C PALM Declaration
C
      CHARACTER(LEN=PL_LNAME) cl_name, cl_space
      INTEGER il_time, il_usrtag, il_err, il_err2
      INTEGER il_day, il_month, il_year, il_hour, il_min, il_sec
C
C***
C
C PALM GET OF INPUTS
C
      cl_space = 'one_real'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'rd_daynum'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,daynum,il_err)
C 
      cl_space = 'date'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'ida_date0'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,idate0,il_err)
C
      cl_space = 'one_logical'
      il_time = int(daynum)
      il_usrtag = PL_NO_TAG
      cl_name = 'ld_lend'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,LEND,il_err)
C
      icount=0
      DO i=1,ig_nobs
        IF (icount.GT.ip_nobstspmax) THEN
            WRITE(*,*) '+ TOO MANY OBS. FOR CURRENT TIMESTEP: ABORT!'
            CALL PALM_Abort(il_err)
        ENDIF
        IF ((int(daynum).LE.iga_secobs(i)).AND.
     +      (int(daynum)+ip_obs_slot.GT.iga_secobs(i))) THEN
            icount=icount+1
            ILOBS(icount)=i
        ENDIF
      ENDDO
C
      IF (icount. gt. 0) THEN 
C
          ALLOCATE( MISFIT(ip_nobstspmax*ip_nbnivpresmax*ip_nbconstmax))
          MISFIT(:) = 0.
          ALLOCATE(invR(ip_nobstspmax,
     $        ip_nbnivpresmax,ip_nbnivpresmax,ip_nbconstmax))
          invR(:,:,:,:) = 0.
          ALLOCATE( invRd(ip_nobstspmax*ip_nbnivpresmax*ip_nbconstmax))
          invRd(:) = 0.
C
          cl_space = 'vectobs'
          il_time = daynum
          il_usrtag = PL_NO_TAG
          cl_name = 'rda_misfit'
          CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,MISFIT,
     $        il_err)
C
          cl_space = 'matcovobs'
          il_time = daynum
          il_usrtag = PL_NO_TAG
          cl_name = 'rda_invR'
          CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,invR,il_err)
C
C**  COMPUTE invR * d
C
          invRd(:) = 0.
C
          DO i=1,icount
            lobs=ILOBS(i)
            DO ipres1=1,iga_npreslev(lobs)
              IF (ABS(iga_tshold(lobs,ipres1,1)).EQ.1) THEN
                  rla_tshold(ipres1)=1.
              ELSE
                  rla_tshold(ipres1)=0.
              ENDIF
            END DO
            DO ipres1=1,iga_npreslev(lobs)
              iyo1=(ipres1-1)*ip_nobstspmax+i
              invRd(iyo1) = 0.
              IF (ABS(iga_tshold(lobs,ipres1,1)).eq.1) THEN
                  DO ipres2=1,iga_npreslev(lobs)
                    iyo2=(ipres2-1)*ip_nobstspmax+i
                    invRd(iyo1) = invRd(iyo1)
     $                  + invR(i,ipres1,ipres2,1)
     $                  * rla_tshold(ipres2)
     $                  * MISFIT(iyo2)
                 END DO
              ENDIF
            ENDDO
          ENDDO
C
C**  PALM PUT MISFIT invR * d
C
          cl_space = 'vectobs'
          il_time = daynum
          il_usrtag = PL_NO_TAG
          cl_name = 'rda_invR_misfit'
          CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,invRd,il_err)
C
          DEALLOCATE( MISFIT )
          DEALLOCATE( invR )
          DEALLOCATE( invRd )
C
      END IF
C
C** END OF COMPUTE (invR * d) EVERY DAY OR AT THE END
C
C  ENDIF
C
C***
C
      RETURN 
      END
