C
      SUBROUTINE LECTDAT() 
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Observations 
C   Purpose   : Read-in observational data files HDAT, HCOV, HAVK
C   Called by : 
C
CCC
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 03/1999
C
C Modified  : A. Piacentini - CERFACS, 05/2007
C
CCC
C
      USE palm_user_param
      USE DATDAT_MOD
      USE GRIDS_OBS_MOD
      USE palmlib
C
      IMPLICIT NONE	
C
C** Run mode
C
C* ll_initobs : toggle first time mode for file reading
C* ll_infct : switch between balancing and forecast mode
C* il_loader_verb : verbosity level
C
      LOGICAL :: ll_initobs
      LOGICAL :: ll_infct
      INTEGER :: il_loader_verb
C
C** Time handling
C
C* daynum : slot time = begin of the slot, in seconds since idate0
C* idate0 : the reference date
C* idate1 : current date - used in coversions
C* nbday, nbhour : day and hour in daynum secs. to date conversion
C* nbjour : days per month
C
      REAL(KIND=8)          :: daynum
      INTEGER, DIMENSION(5) :: idate0
      INTEGER, ALLOCATABLE  :: idate1(:)
      INTEGER               :: nbday,nbhour
      INTEGER, ALLOCATABLE  :: nbjour(:)	
C
C** File handling
C
C* ila_unit : the unit nr. for HDAT, HCOV, HAVK are stored in an array
C* il_dat, il_cov, il_avk : indexes of HDAT, HCOV, HAVK in ila_UNIT
C* clfdat,clfcov, clfavk : the HDAT, HCOV, HAVK file names
C* ll_obsfile : inquiry flag for file existence
C
      INTEGER, DIMENSION(3) :: ila_unit
      INTEGER               :: il_dat, il_cov, il_avk
      CHARACTER(LEN=13)     :: clfdat,clfcov,clfavk
      LOGICAL, SAVE         :: ll_obsfile
C
C** File reading : observations
C
C* il_nobs : total observation locations number in file
C* ila_nobs_inst : observation locations number in file per instrument
C* il_dp, il_ep : string positions for instrument description parsing
C* ila_dateobs : obs. date on yyyy mm dd hh mm fields
C* cl_str_id : string obs identifier
C* il_int_id : integer obs identifier
C* il_ninst_avk : counter of the instruments using an averagin kernel
C* rl_obslat, rl_obslon : location coordinates
C* il_npreslev : number of vertical levels for the location
C* rla_preslev : pressure level bounds
C* ila_nbconst : number of observed species per level
C* cla_constname : label of the observed species
C* rla_val : measurement value
C
      INTEGER                                       :: il_nobs
      INTEGER, DIMENSION(ip_ninstmax)               :: ila_nobs_inst
      INTEGER                                       :: il_dp, il_ep
      INTEGER, DIMENSION(5)                         :: ila_dateobs
      CHARACTER(LEN=6)                              :: cl_str_id
      INTEGER                                       :: il_int_id
      INTEGER                                       :: il_ninst_avk
      REAL(KIND=8)                                  :: rl_obslat
      REAL(KIND=8)                                  :: rl_obslon
      INTEGER                                       :: il_npreslev
      REAL,DIMENSION(ip_nbnivpresmax,2)             :: rla_preslev
      INTEGER, DIMENSION(ip_nbnivpresmax)           :: ila_nbconst
      CHARACTER(LEN=5), DIMENSION(ip_nbnivpresmax,ip_nbconstmax)
     $                                              :: cla_constname
      REAL,DIMENSION(ip_nbnivpresmax,ip_nbconstmax) :: rla_val
C
C** File reading : covariances and averaging kernels
C
C* il_est_ncov : estimated number of covariance matrices
C* il_ncov_rd : number of actually read covariance matrices 
C* il_noscov : number of covariance matrices sets
C* il_est_navk : estimated number of av. kern. matrices
C* il_navk_rd : number of actually read av. kern. matrices 
C* il_nosavk : number of averaging kernels sets
C* ila_date : record date on yyyy mm dd hh mm fields
C* cl_str_id2 : record string identifier
C* il_int_id2 : record integer identifier
C* rl_xlat, rl_xlon : location coordinates
C* il_xnpreslev : number of vertical levels for the location
C* rl_dummy1, rl_dummy2 : pressure level bounds for fake reading
C* ila_nbcov : number of elements per row in cov. matrix
C* ila_nbavk : number of elements per row in av. kern. matrix
C* cla_label : label of the record
C* rla_cov_rd : element of the covariance matrix
C* rla_avk : element of the av. kern. matrix
C
      INTEGER                            :: il_est_ncov, il_ncov_rd
      INTEGER                            :: il_est_noscov
      INTEGER                            :: il_est_navk ,il_navk_rd 
      INTEGER                            :: il_est_nosavk, il_est_nosiq
      INTEGER, DIMENSION(5)              :: ila_date(5)
      CHARACTER(LEN=6)                   :: cl_str_id2
      INTEGER                            :: il_int_id2 
      REAL(KIND=8)                       :: rl_xlat,rl_xlon
      INTEGER                            :: il_xnpreslev
      REAL(KIND=8)                       :: rl_dummy1, rl_dummy2
      INTEGER,DIMENSION(ip_nbnivpresmax) :: ila_nbcov
      INTEGER,DIMENSION(ip_nbnivpresmax) :: ila_nbavk
      CHARACTER(LEN=7)                   ::  cl_label
      REAL, DIMENSION(ip_nbnivpresmax,ip_nbnivpresmax,ip_nbconstmax)
     $                                   :: rla_cov_rd
      REAL, ALLOCATABLE, DIMENSION(:,:)  :: rla_avk
C
C** Data selection
C
C* rl_latmin, rl_latmax : latitude domain bounds
C* rl_lonmin, rl_lonmax : longitude domain bounds
C* rl_presmin, rl_presmax : vertical domain bounds
C* lla_take : vertical selection flag for levels on a single location
C* il_countniv : number of selected levels on a single location
C
      REAL(KIND=8)                       :: rl_latmin,rl_latmax
      REAL(KIND=8)                       :: rl_lonmin,rl_lonmax
      REAL(KIND=8)                       :: rl_presmin,rl_presmax
      LOGICAL,DIMENSION(ip_nbnivpresmax) :: lla_take
      INTEGER                            :: il_countniv
C
C** Observation storages (most of them are in datdat_mod)
C
C
C** Covariance storages
C
C* il_ncov : number of accepted covariances locations
C* il_noscov : number of covariances sets
C* cla_defcov : description of covariances sets
C* lla_hcov : toggle covariance reading in HCOV for each instrument
C* rla_percent : % constant std error if given for the instrument
C* il_sigobs : sigma scaling for obs. error matrices
C* ila_covnpreslev : number of selected levels for each location
C* rla_cov : selected and scaled triangular inverse var/cov matrices
C* invR : full symetric storage of inverse var/cov matrices
C* ila_nbcovconst : number of elements per row in full sym storage
C
      INTEGER                                        :: il_ncov
      INTEGER                                        :: il_noscov
      CHARACTER(LEN=68), DIMENSION(:), ALLOCATABLE   :: cla_defcov
      LOGICAL, DIMENSION(ip_ninstmax)                :: lla_hcov
      REAL,    DIMENSION(ip_ninstmax)                :: rla_percent      
      INTEGER, DIMENSION(ip_ninstmax)                :: ila_iq_vertlev    
      INTEGER                                        :: il_sigobs
      INTEGER, DIMENSION(ip_nobsmax), SAVE           :: ila_covnpreslev
      REAL, DIMENSION(ip_nobsmax,ip_nbnivpresmax,ip_nbnivpresmax,
     & 	          ip_nbconstmax), SAVE               :: rla_cov
      INTEGER, DIMENSION(ip_nobsmax,ip_nbnivpresmax), SAVE
     $                                               ::  ila_nbcovconst 
C
C** Averaging kernels storages
C
C* il_navk : number of accepted av. kern. locations
C* il_nosavk : number of av. kern. sets
C* cla_defavk : description of av. kern. sets
C
      INTEGER                                      :: il_navk
      INTEGER                                      :: il_nosavk
      CHARACTER(LEN=68), DIMENSION(:), ALLOCATABLE :: cla_defavk
C
C** Mapping pointers
C
C* icount : number of locations in a time slot
C* ilobs : maps obs location numbers from slot to day
C* lobs, lcov : obs or cov location indexes mapped to global numbering
C
      INTEGER                           :: icount 
      INTEGER, DIMENSION(ip_nobstspmax) :: ilobs
      INTEGER                           :: lobs, lcov
C
C** Loop indexes and other counters
C
      INTEGER :: ib_inst, ib_j, ib_k
      INTEGER :: i, j, k
      INTEGER :: iobs, icov
      INTEGER :: ipres
C
C** Temporary variables and arrays
C
C* rl_switch : exchange storage for pressure level bounds sorting
C
      REAL :: rl_switch
C
C** PALM related variables
C
      CHARACTER(LEN=PL_LNAME) :: cl_name, cl_space
      INTEGER :: il_time, il_usrtag, il_err, il_err2
      INTEGER :: il_day, il_month, il_year, il_hour, il_min, il_sec
C
CCC 1. PALM GET OF INPUTS
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'verbosity'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_loader_verb,
     $    il_err)
c
      cl_space = 'one_real'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'rd_daynum'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,daynum,il_err)
C
      cl_space = 'one_logical'
      il_time = int(daynum)
      il_usrtag = PL_NO_TAG
      cl_name = 'ld_init'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $    ll_initobs,il_err)
C
      cl_space = 'one_logical'
      il_time = int(daynum)
      il_usrtag = PL_NO_TAG
      cl_name = 'ld_infct'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,ll_infct,il_err)
C     
      cl_space = 'date'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'ida_date0'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,idate0,il_err)
C     
CCC 2. OPEN NEW OBSERVATIONAL FILES ONCE A DAY
C
      IF ((INT(daynum).EQ.0).AND.ll_initobs) THEN
C
CCC 2.1. SET THE OBSERVATIONAL DOMAIN BOUNDS
C
        rl_latmin = minval(rga_rlatm1)
        rl_latmin = max(rl_latmin,REAL(ip_latobsmin))
        rl_latmax = maxval(rga_rlatm1)
        rl_latmax = min(rl_latmax,REAL(ip_latobsmax))
C
        IF (lg_periodic_lon) THEN
            rl_lonmin = REAL(ip_lonobsmin)
            rl_lonmax = REAL(ip_lonobsmax)
        ELSE
            rl_lonmin = minval(rga_rlonm1)
            rl_lonmin = max(rl_lonmin,REAL(ip_lonobsmin))
            rl_lonmax = maxval(rga_rlonm1)
            rl_lonmax = min(rl_lonmax,REAL(ip_lonobsmax))
        END IF
C
        IF (lg_hybrid_sigma) THEN
            rl_presmin = REAL(ip_presobsmin)
            rl_presmax = REAL(ip_presobsmax)
        ELSE
            IF (rga_presgri(1).lt.rga_presgri(ip_nivm1)) THEN
                rl_presmin = 2*rga_presgri(1) - rga_presint(1)
            ELSE
                rl_presmin = 2*rga_presgri(ip_nivm1) -
     $              rga_presint(ip_nivm1)
            ENDIF
            rl_presmin = max(rl_presmin, 0.)
            rl_presmin = max(rl_presmin, REAL(ip_presobsmin))
            rl_presmax = maxval(rga_presint)
            rl_presmax = min(rl_presmax, REAL(ip_presobsmax))
        ENDIF
C
CCC 2.2. DETERMINE FILES' DATE
C
        ALLOCATE (idate1(5))
        ALLOCATE (nbjour(12))
C     
        nbjour=(/31,28,31,30,31,30,31,31,30,31,30,31/)
C
        il_dat=1
        il_cov=2
        il_avk=3
C
        NBDAY=int(daynum/86400.)
        NBHOUR=int((daynum-real(NBDAY)*86400.)/3600.)
C
        idate1(5)=0
        idate1(4)=idate0(4)+NBHOUR
        idate1(3)=idate0(3)+NBDAY
        idate1(2)=idate0(2)
        idate1(1)=idate0(1)
C
        IF (idate1(4).GE.24) THEN
          idate1(4)=idate1(4)-24
          idate1(3)=idate1(3)+1
        ENDIF
C
        IF (mod(idate0(1),4).EQ.0) THEN
          nbjour(2)=29
        ENDIF 
C
        IF (idate1(3).GT.nbjour(idate1(2))) THEN
          idate1(3)=idate1(3)-nbjour(idate1(2))
          idate1(2)=idate1(2)+1
        ENDIF
C
        IF (idate1(2).GT.12) THEN
          idate1(1)=idate1(1)+1
          idate1(2)=1
        ENDIF
C
C
CCC 2.3. OPEN OBSERVATION DATA FILE
C
        WRITE(clfdat,fmt='(''HDAT+'',I4.4,I2.2,I2.2)')
     +      idate1(1),idate1(2),idate1(3)
C
        INQUIRE(FILE=clfdat, EXIST=ll_obsfile)
C
        IF (.NOT.ll_obsfile) THEN
            WRITE(*,*) '+ NO VALID OBSERVATION FILE ',trim(clfdat)
            WRITE(*,*) '+ ASSUMING NO OBSERVATIONS '
            WRITE(PL_OUT,*) '+ NO VALID OBSERVATION FILE ',trim(clfdat)
            WRITE(PL_OUT,*) '+ ASSUMING NO OBSERVATIONS '
            ig_nobs = 0
            il_ncov = 0
            il_navk = 0
            ig_ninst = 0
            RETURN
        ENDIF
C
        ila_unit(il_dat)=50
C
        OPEN(unit=ila_unit(il_dat),file=clfdat,form='formatted',err=99)
        GOTO 100
 99     WRITE(*,*) '+ ERROR OPENING OBS. FILE ',clfdat,' : ABORT!'
        CALL PALM_Abort(il_err)
 100    CONTINUE
            
            
C
CCC 2.4. READ OBSERVATION DESCRIPTION AND DETERMINE WHAT ELSE TO READ
C
        ig_nobs = 0
        ig_ninst = 0
        il_ncov = 0
        il_navk = 0
        ila_nobs_inst(:) = 0
        lga_avg_kernel(:) = .false.
        lga_tot_col(:) = .false.
        lga_int_qty(:) = .false.
        lla_hcov(:) = .false.
        rla_percent(:) = 0.
        ila_iq_vertlev(:) = 0
        rla_cov(:,:,:,:) = 0.
        il_ninst_avk = 0
C
        READ(ila_unit(il_dat),*) il_nobs,ig_ninst
C
        IF (ig_ninst .gt. ip_ninstmax) THEN
            WRITE(*,*) '+ !! TOO MANY INSTRUMENTS IN FILE ',
     $          trim(clfdat)
            WRITE(*,*) '+ !! Check the ip_ninstmax PrePALM constant'
            WRITE(*,*) '+ !! Shutting down the simulation'
            WRITE(PL_OUT,*) '+ !! TOO MANY INSTRUMENTS IN FILE ',
     $          trim(clfdat)
            WRITE(PL_OUT,*) '+ !! Check the ip_ninstmax ',
     $          'PrePALM constant'
            WRITE(PL_OUT,*) '+ !! Shutting down the simulation'
            CALL PALM_Abort(il_err)
        END IF
            
C
CCC 2.4.1. For each instrument : 
C
        DO ib_inst=1,ig_ninst
C
CCC 2.4.1.1 Read the identifier : 
C
          READ(ila_unit(il_dat),101) cga_inst(ib_inst)
C
CCC 2.4.1.2. Look for the number of profiles
C
          IF (INDEX(cga_inst(ib_inst),'#P=').ne.0) THEN
              il_dp=INDEX(cga_inst(ib_inst),'#P=')
              il_ep=INDEX(cga_inst(ib_inst)(il_dp:),' ')
              IF(il_ep.eq.0) THEN
                  READ(cga_inst(ib_inst)(il_dp+3:),*)
     $                ila_nobs_inst(ib_inst) 
              ELSE
                  READ(cga_inst(ib_inst)(il_dp+3:il_dp+il_ep-1),*)
     $                ila_nobs_inst(ib_inst)  
              END IF
          ELSE
              ila_nobs_inst(ib_inst) = il_nobs
          END IF
C
CCC 2.4.1.3. Look for the averaging kernel switch
C
          IF (INDEX(cga_inst(ib_inst),'AVKERN').ne.0) THEN
              lga_avg_kernel(ib_inst) = .true.
              il_ninst_avk = il_ninst_avk + 1
          ENDIF
C
CCC 2.4.1.4. Look for the total column switch
C
          IF (INDEX(cga_inst(ib_inst),'TOTCOL').ne.0) THEN
              lga_tot_col(ib_inst) = .true.
              IF (ip_tc_as_prof.eq.1.and.
     +            ip_nivm1.gt.ip_nbnivpresmax) THEN
                  WRITE(*,*) '+ !! Number of intermediate levels ',
     +                ip_nivm1,
     +                'for instrument ',trim(cga_inst(ib_inst)),
     +                'exceeds ip_nbnivpresmax = ',
     +                ip_nbnivpresmax
                  WRITE(PL_OUT,*)'+ !! Number of intermediate levels ',
     +                ip_nivm1,
     +                'for instrument ',trim(cga_inst(ib_inst)),
     +                'exceeds ip_nbnivpresmax = ',
     +                ip_nbnivpresmax
                  CALL PALM_Abort(il_err)
              END IF              
          ENDIF
C
CCC 2.4.1.4. Look for the integrated quantity switch
C
          IF (INDEX(cga_inst(ib_inst),'INTQTY').ne.0) THEN
              lga_int_qty(ib_inst) = .true.
              lga_avg_kernel(ib_inst) = .true.
              il_dp=INDEX(cga_inst(ib_inst),'INTQTY=')
              il_ep=INDEX(cga_inst(ib_inst)(il_dp:),' ')
              IF(il_ep.eq.0) THEN
                  READ(cga_inst(ib_inst)(il_dp+7:),*)
     $                ila_iq_vertlev(ib_inst)
              ELSE
                  READ(cga_inst(ib_inst)(il_dp+7:il_dp+il_ep-1),*)
     $                ila_iq_vertlev(ib_inst)
              END IF 
              IF (ila_iq_vertlev(ib_inst) .gt. ip_nbnivpresmax) THEN
                  WRITE(*,*) '+ !! Number of interpolation levels ',
     +                ila_iq_vertlev(ib_inst),
     +                'for instrument ',trim(cga_inst(ib_inst)),
     +                'exceeds ip_nbnivpresmax = ',
     +                ip_nbnivpresmax
                  WRITE(PL_OUT,*)'+ !! Number of interpolation levels ',
     +                ila_iq_vertlev(ib_inst),
     +                'for instrument ',trim(cga_inst(ib_inst)),
     +                'exceeds ip_nbnivpresmax = ',
     +                ip_nbnivpresmax
                  CALL PALM_Abort(il_err)
              END IF
          END IF
C
CCC 2.4.1.5. Look for the percent obs error covariance switch
C            and for the std error value (as a % of the obs value)
C
          IF (INDEX(cga_inst(ib_inst),'%R=').ne.0) THEN
              il_dp=INDEX(cga_inst(ib_inst),'%R=')
              il_ep=INDEX(cga_inst(ib_inst)(il_dp:),' ')
              IF(il_ep.eq.0) THEN
                  READ(cga_inst(ib_inst)(il_dp+3:),*)
     $                rla_percent(ib_inst)
              ELSE
                  READ(cga_inst(ib_inst)(il_dp+3:il_dp+il_ep-1),*)
     $                rla_percent(ib_inst)
              END IF
              IF (rla_percent(ib_inst).lt.EPSILON(1.))
     $            lla_hcov(ib_inst)=.true.
          ELSE
              lla_hcov(ib_inst)=.true.
          END IF
C          
        END DO
C
        IF (il_ninst_avk .gt. ip_ninstmax_avk) THEN
            WRITE(*,*) '+ !! TOO MANY AVKERN INSTRUMENTS IN FILE ',
     $          trim(clfdat)
            WRITE(*,*) '+ !! Check the ip_ninstmax_avk PrePALM constant'
            WRITE(*,*) '+ !! Shutting down the simulation'
            WRITE(PL_OUT,*) '+ !! TOO MANY AVKERN INSTRUMENTS IN FILE ',
     $          trim(clfdat)
            WRITE(PL_OUT,*) '+ !! Check the ip_ninstmax_avk ',
     $          'PrePALM constant'
            WRITE(PL_OUT,*) '+ !! Shutting down the simulation'
            CALL PALM_Abort(il_err)
        END IF
C
        IF (ig_ninst-il_ninst_avk.gt.ip_ninstmax-ip_ninstmax_avk) THEN
            WRITE(*,*) '+ !! TOO MANY NOT AVKERN INSTRUMENTS IN FILE ',
     $          trim(clfdat)
            WRITE(*,*) '+ !! Check the ip_ninstmax_avk PrePALM constant'
            WRITE(*,*) '+ !! Shutting down the simulation'
            WRITE(PL_OUT,*) '+ !! TOO MANY NOT AVKERN INSTRUMENTS',
     $          ' IN FILE ',trim(clfdat)
            WRITE(PL_OUT,*) '+ !! Check the ip_ninstmax_avk ',
     $          'PrePALM constant'
            WRITE(PL_OUT,*) '+ !! Shutting down the simulation'
            CALL PALM_Abort(il_err)
        END IF
C
CCC 2.4.2. Some diagnostic prints
C
        IF (il_loader_verb .GT. 0) THEN
            WRITE(*,*) '+ Opening DATA file ',trim(clfdat)
            WRITE(*,*) '  Number of datasets ',ig_ninst
            WRITE(*,*) '  Total number of profiles ',il_nobs
            DO i=1,ig_ninst
              WRITE(*,*) '    Data set : ',trim(cga_inst(i))
              WRITE(*,*) '    nr. of profiles  : ',ila_nobs_inst(i)
            END DO
            WRITE(*,*)
            WRITE(*,*) '  Longitude bounds ',rl_lonmin, rl_lonmax
            WRITE(*,*) '  Latitude bounds  ',rl_latmin, rl_latmax
            WRITE(*,*) '  Pressure bounds  ',rl_presmin, rl_presmax
            WRITE(*,*) '+'
C
            WRITE(PL_OUT,*) '+ Opening DATA file ',trim(clfdat)
            WRITE(PL_OUT,*) '  Number of datasets ',ig_ninst
            WRITE(PL_OUT,*) '  Total number of profiles ',il_nobs
            DO i=1,ig_ninst
              WRITE(PL_OUT,*) '    Data set : ',trim(cga_inst(i))
              WRITE(PL_OUT,*) '    nr. of profiles  : ',ila_nobs_inst(i)
            END DO
            WRITE(PL_OUT,*)
            WRITE(PL_OUT,*) '  Longitude bounds ',rl_lonmin,rl_lonmax
            WRITE(PL_OUT,*) '  Latitude bounds  ',rl_latmin,rl_latmax
            WRITE(PL_OUT,*) '  Pressure bounds  ',
     $          rl_presmin, rl_presmax
            WRITE(PL_OUT,*) '+'
C
        END IF
C
CCC 2.4.3. Sanity checks
C
        IF (sum(ila_nobs_inst(1:ig_ninst)).ne.il_nobs) THEN
            WRITE(*,*) '+ !! Total and partial numbers of profiles',
     $          ' do not agree'
            WRITE(*,*) '+ !! Check file ',trim(clfdat)
            DO i=1,ig_ninst
              WRITE(*,*) '+ !! with data set ',trim(cga_inst(i))
            END DO
            WRITE(*,*) '+ !! Shutting down the simulation'
            WRITE(PL_OUT,*) '+ !! Total and partial numbers of '
     $          ,'profiles do not agree'
            DO i=1,ig_ninst
              WRITE(PL_OUT,*) '+ !! with data set ',trim(cga_inst(i))
            END DO
            WRITE(PL_OUT,*) '+ !! Check file ',trim(clfdat)
            WRITE(PL_OUT,*) '+ !! Shutting down the simulation'
            CALL PALM_Abort(il_err)
        END IF
C
        DO i = 1,ig_ninst
          IF (lga_tot_col(i) .and .lga_avg_kernel(i)) THEN
              WRITE(*,*) '+ !! TOTCOL and AVKERN attributes cannot be',
     $            'set simultaneously'
              WRITE(*,*) '+ !! Error in data set ',trim(cga_inst(i))
              WRITE(*,*) '+ !! From file ',trim(clfdat)
              WRITE(*,*) '+ !! Shutting down the simulation'
              WRITE(PL_OUT,*)'+ !! TOTCOL and AVKERN attributes',
     $            'cannot be set simultaneously'
              WRITE(PL_OUT,*) '+ !! Error in data set ',
     $            trim(cga_inst(i))
              WRITE(PL_OUT,*) '+ !! From file ',trim(clfdat)
              WRITE(PL_OUT,*) '+ !! Shutting down the simulation'
              CALL PALM_Abort(il_err)
          END IF
        END DO
C
CCC 2.5 OPEN COVARIANCE FILE
C
        IF (ANY(lla_hcov)) THEN
C
CCC 2.5.1 Estimate of how many covariances matrices have to be read
C
            il_est_ncov = SUM(ila_nobs_inst,mask=lla_hcov)
            il_est_noscov = COUNT(lla_hcov)
C
            ila_unit(il_cov)=51
            WRITE(clfcov,fmt='(''HCOV+'',I4.4,I2.2,I2.2)')
     +          idate1(1),idate1(2),idate1(3)
C
            INQUIRE(FILE=TRIM(clfcov), EXIST=ll_obsfile)
            IF (ll_obsfile) THEN
                OPEN(unit=ila_unit(il_cov),file=clfcov,form='formatted',
     $              err=199)
                GOTO 200
 199            WRITE(*,*) '+ ERROR OPENING COV. FILE ',TRIM(clfcov),
     $              ' : ABORT!'
                CALL PALM_Abort(il_err)
 200            CONTINUE
C	
                ALLOCATE (cla_defcov(ip_ninstmax)) 
C
                READ(ila_unit(il_cov),*) il_ncov_rd,il_noscov
                IF (il_ncov_rd .ne. il_est_ncov
     $              .or. il_noscov .ne. il_est_noscov) THEN
                    WRITE(*,*) '+ !! Covariance dimensions not ',
     $                  'coherent with HDAT file'
                    WRITE(PL_OUT,*) '+ !! Covariance dimensions',
     $                  'not coherent with HDAT file'
                    CALL PALM_Abort(il_err)
                ENDIF
C
                DO i=1,ig_ninst
                  IF(lla_hcov(i)) READ(ila_unit(il_cov),101)
     $                cla_defcov(i)
                END DO
            ELSE
                WRITE(*,*) '+ !! Missing covariance file',TRIM(clfcov)
                WRITE(PL_OUT,*)'+ !! Missing covariance file',
     $              TRIM(clfcov)
                CALL PALM_Abort(il_err)
            ENDIF
C            
        ENDIF
C
CCC 2.6 OPEN AVERAGING KERNEL FILE
C
        IF (ANY(lga_avg_kernel)) THEN
C
CCC 2.6.1 Estimate of how many av. kernel matrices have to be read
C
            il_est_navk = SUM(ila_nobs_inst,mask=lga_avg_kernel)
            il_est_nosavk = COUNT(lga_avg_kernel)
            il_est_nosiq = COUNT(lga_int_qty)
C
            ila_unit(il_avk)=52
            WRITE(clfavk,fmt='(''HAVK+'',I4.4,I2.2,I2.2)')
     +          idate1(1),idate1(2),idate1(3)
C
            INQUIRE(FILE=TRIM(clfavk), EXIST=ll_obsfile)
            IF (ll_obsfile) THEN
                OPEN(unit=ila_unit(il_avk),file=clfavk,form='formatted',
     $              err=299)
                GOTO 300
 299            WRITE(*,*) '+ !! ERROR OPENING AVK. FILE ',TRIM(clfavk),
     $              ' : ABORT!'
                CALL PALM_Abort(il_err)
 300            CONTINUE
C
                ALLOCATE (cla_defavk(ip_ninstmax))
                IF (il_est_nosavk.gt.il_est_nosiq) THEN
                    CALL ALLOC_DATAVK
                    ALLOCATE (rla_avk(ip_nbniv_avk_out,ip_nbniv_avk_in))
                ELSE 
                    CALL ALLOC_DATAVK_IQ
                    ALLOCATE (rla_avk(1,ip_nbniv_avk_in))
                END IF
                rla_avk(:,:)=0.
C
                READ(ila_unit(il_avk),*) il_navk_rd,il_nosavk
                IF (il_navk_rd .ne. il_est_navk .or.
     $              il_nosavk .ne. il_est_nosavk) THEN
                    WRITE(*,*) '+ !! Averaging kernel dimensions not',
     $                  'coherent with HDAT file'
                    WRITE(PL_OUT,*) '+ !! Averaging kernel dimensions',
     $                  'not coherent with HDAT file'
                    CALL PALM_Abort(il_err)
                ENDIF
C
                DO i=1,ig_ninst
                  IF(lga_avg_kernel(i)) READ(ila_unit(il_avk),101)
     $                cla_defavk(i)
                ENDDO
            ELSE
                WRITE(*,*) '+ !! Missing avg. kern. file',TRIM(clfavk)
                WRITE(PL_OUT,*) '+ !! Missing avg. kern file',
     $              TRIM(clfavk)
                CALL PALM_Abort(il_err)
            ENDIF
        ENDIF
C
CCC 3. READ OBSERVATIONAL DATA
C
C
CCC 3.1 LOOP OVER PROFILES
C
        DO iobs=1,il_nobs
          DO i=1,ig_ninst
            IF(iobs.le.SUM(ila_nobs_inst(1:i))) THEN
                ib_inst = i
                EXIT
            END IF
          END DO
C
CCC 3.2 READ DATA COORDS
C
          READ(ila_unit(il_dat),102)
     +      ila_dateobs(1),ila_dateobs(2),
     +      ila_dateobs(3),ila_dateobs(4),ila_dateobs(5),
     +      cl_str_id,il_int_id,
     +      rl_obslat,rl_obslon,
     +      il_npreslev
C
CCC 3.3 READ COV COORDS AND CHECK COHERENCY
C
          IF (lla_hcov(ib_inst)) THEN
              READ(ila_unit(il_cov),102)
     +            ila_date(1),ila_date(2),
     +            ila_date(3),ila_date(4),ila_date(5),
     +            cl_str_id2,il_int_id2,
     +            rl_xlat,rl_xlon,
     +            il_xnpreslev
              IF (ANY(ila_date.NE.ila_dateobs).OR.
     $            ABS(rl_xlat-rl_obslat).GT.1e-10.OR.
     $            ABS(rl_xlon-rl_obslon).GT.1e-10.OR.
     $            il_xnpreslev.NE.il_npreslev) THEN
                  WRITE(*,*) '+ !! Covariance profiles not',
     $                'coherent with HDAT file'
                  WRITE(PL_OUT,*) '+ !! Covariance profiles',
     $                'not coherent with HDAT file'
                  CALL PALM_Abort(il_err)
              ENDIF
          ENDIF
C
CCC 3.4 READ AVG KERN COORDS AND CHECK COHERENCY
C
          IF (lga_avg_kernel(ib_inst)) THEN
              READ(ila_unit(il_avk),102)
     +            ila_date(1),ila_date(2),
     +            ila_date(3),ila_date(4),ila_date(5),
     +            cl_str_id2,il_int_id2,
     +            rl_xlat,rl_xlon,
     +            il_xnpreslev
     
              IF (ANY(ila_date.NE.ila_dateobs).OR.
     $            ABS(rl_xlat-rl_obslat).GT.1e-10.OR.
     $            ABS(rl_xlon-rl_obslon).GT.1e-10.OR.
     $            il_xnpreslev.NE.il_npreslev) THEN
                  WRITE(*,*) '+ !! Avg. kern. profiles not',
     $                'coherent with HDAT file'
                  WRITE(PL_OUT,*) '+ !! Avg. kern. profiles',
     $                'not coherent with HDAT file'
                  CALL PALM_Abort(il_err)
              ENDIF
          ENDIF
C
CCC 3.5 SELECT OBSERVATIONS IN THE GEOGRAPHICAL DOMAIN
C
CCC 3.5.1 Fit longitudes in [-180,180) period
C
          IF (rl_obslon.lt.-180) THEN
              rl_obslon =  rl_obslon+360.
          ELSE IF (rl_obslon.ge.180) THEN
              rl_obslon =  rl_obslon-360.
          ENDIF
C
CCC 3.5.2 Check the horizontal observation domain
C
          IF (((rl_obslat.LT.rl_latmin).OR.
     +        (rl_obslat.GT.rl_latmax).OR.
     +        (rl_obslon.LT.rl_lonmin).OR.
     +        (rl_obslon.GT.rl_lonmax)).AND.
     +        il_loader_verb .GT. 1) THEN
C
              WRITE(*,*) '+ DISCARDED Profile number ',iobs
              WRITE(*,*) '  from DATA set ',trim(cga_inst(1))
              WRITE(*,*) '  Longitude ',rl_obslon
              WRITE(*,*) '  Latitude ',rl_obslat
C
              WRITE(PL_OUT,*) '+ DISCARDED Profile number ',iobs
              WRITE(PL_OUT,*) '  from DATA set ',trim(cga_inst(1))
              WRITE(PL_OUT,*) '  Longitude ',rl_obslon
              WRITE(PL_OUT,*) '  Latitude ',rl_obslat
          ENDIF
C
CCC 3.6 LOOP OVER PRESSURE LEVELS
C
          il_countniv = 0
          lla_take(:) = .FALSE.
          rla_cov_rd(:,:,:) = 0
          ila_nbavk(:) = 0
C
          DO j=1,max(il_npreslev,ila_iq_vertlev(ib_inst))
C
            IF (lga_tot_col(ib_inst)) THEN
                IF (lga_avg_kernel(ib_inst))
     +              READ(ila_unit(il_avk),106)
     +              rl_dummy1,rl_dummy2,ila_nbavk(j)
C
                READ(ila_unit(il_dat),106)
     +              rla_preslev(j,1),rla_preslev(j,2),ila_nbconst(j)
C
                IF (lla_hcov(ib_inst)) THEN
                    READ(ila_unit(il_cov),106)
     +                  rl_dummy1,rl_dummy2,ila_nbcov(j)
                ELSE
                    ila_nbcov(j) = ila_nbconst(j)
                ENDIF
C
CCC 3.6.1 Sort the integrated quantity vertical bounds
C
                IF (rla_preslev(j,1).GT.rla_preslev(j,2)) THEN
                    rl_switch = rla_preslev(j,2)
                    rla_preslev(j,2) = rla_preslev(j,1)
                    rla_preslev(j,1) = rl_switch
                END IF
C
CCC 3.6.2 For pure pressure vert. coord, bound the slices to the model domain
C
                IF (.NOT. lg_hybrid_sigma) THEN
                    rla_preslev(j,1) = max(rla_preslev(j,1),rl_presmin)
                    rla_preslev(j,2) = min(rla_preslev(j,2),rl_presmax)
                END IF
            ELSE
                IF (lga_avg_kernel(ib_inst).and.
     +              (j.le.il_npreslev))
     +              READ(ila_unit(il_avk),103)
     +              rl_dummy1,ila_nbavk(j)
C
                READ(ila_unit(il_dat),103)
     +              rla_preslev(j,1),ila_nbconst(j)
                rla_preslev(j,2)=rla_preslev(j,1)
C
                IF (lla_hcov(ib_inst).and.
     +              (j.le.il_npreslev)) THEN
                    READ(ila_unit(il_cov),103)
     +                  rl_dummy1,ila_nbcov(j)
                ELSE
                    ila_nbcov(j) =  ila_nbconst(j)
                END IF
            END IF
C
CCC 3.6.3 Select the levels
C
            IF ((rl_obslat.GE.rl_latmin).AND.
     +          (rl_obslat.LE.rl_latmax).AND.
     +          (rl_obslon.GE.rl_lonmin).AND.
     +          (rl_obslon.LE.rl_lonmax)) THEN 
                IF (ALL(rla_preslev(j,:).GE.rl_presmin).AND.
     +              (ALL(rla_preslev(j,:).LE.rl_presmax))) THEN
C
                    lla_take(j) = .TRUE.
                    IF (j.le.il_npreslev) THEN
                        il_countniv = il_countniv + 1
                        IF (il_countniv .gt. ip_nbnivpresmax) THEN
                            WRITE(*,*) '+ !! Number of levels ',
     +                          il_countniv,'exceeds ip_nbnivpresmax = '
     +                          ,ip_nbnivpresmax
                            WRITE(PL_OUT,*) '+ !! Number of levels ',
     +                          il_countniv, 
     +                          'exceeds ip_nbnivpresmax = ',
     +                          ip_nbnivpresmax
                            CALL PALM_Abort(il_err)
                        END IF
                    END IF
                ELSE
                    IF (il_loader_verb .GT. 1) THEN
                        WRITE(*,*) '+ DISCARDED level number ',j
                        WRITE(*,*) '  of profile number ', iobs
                        WRITE(*,*) '  from DATA set ',trim(cga_inst(1))
                        WRITE(*,*) '  Longitude ',rl_obslon
                        WRITE(*,*) '  Latitude ',rl_obslat
                        WRITE(*,*) '  Pressure bounds',rla_preslev(j,:)
C
                        WRITE(PL_OUT,*) '+ DISCARDED level number ',j
                        WRITE(PL_OUT,*) '  of profile number ', iobs
                        WRITE(PL_OUT,*) '  from DATA set ',
     $                      trim(cga_inst(1))
                        WRITE(PL_OUT,*) '  Longitude ',rl_obslon
                        WRITE(PL_OUT,*) '  Latitude ',rl_obslat
                        WRITE(PL_OUT,*) '  Pressure bounds',
     $                      rla_preslev(j,:)
C
                    END IF
                END IF
            END IF
C
CCC 3.7 LOOP OVER DATA CONSTITUENTS
C
            DO k=1,ila_nbconst(j)
              READ(ila_unit(il_dat),104)
     +            cla_constname(j,k),rla_val(j,k)
            END DO
C
CCC 3.8 LOOP OVER COVARIANCE ROWS
C
            IF (j.le.il_npreslev) THEN
                IF (lla_hcov(ib_inst)) THEN
                    DO k=1,ila_nbcov(j)
                      READ(ila_unit(il_cov),105) cl_label,
     $                    rla_cov_rd(j,k,1)
                    END DO
                ELSE
                    DO k=1,ila_nbconst(j)
                      rla_cov_rd(j,k,1) = rla_val(j,k)
                    END DO
                ENDIF
            END IF
C
CCC 3.9 LOOP OVER AVG KERNEL ROWS
C
            IF (lga_avg_kernel(ib_inst).and.
     +              (j.le.il_npreslev)) THEN
                DO k=1,ila_nbavk(j)
                  READ(ila_unit(il_avk),105) cl_label,rla_avk(j,k)
                END DO
            END IF
            
C
CCC 3.6 END OF LOOP OVER PRESSURE LEVELS
C
          END DO
C
CCC 3.10 With avg. kern. discard a whole profile if a level is rejected
C
          IF (lga_avg_kernel(ib_inst).AND.il_countniv.LT.il_npreslev
     $        .AND.il_countniv.GT.0) THEN
              il_countniv=0
              IF(il_loader_verb .GT. 1) THEN
                  WRITE(*,*) '+ DISCARDED Profile number ',iobs
                  WRITE(*,*) '  from DATA set ',trim(cga_inst(1))
                  WRITE(*,*) '  Longitude ',rl_obslon
                  WRITE(*,*) '  Latitude ',rl_obslat
C
                  WRITE(PL_OUT,*) '+ DISCARDED Profile number ',iobs
                  WRITE(PL_OUT,*) '  from DATA set ',trim(cga_inst(1))
                  WRITE(PL_OUT,*) '  Longitude ',rl_obslon
                  WRITE(PL_OUT,*) '  Latitude ',rl_obslat
              ENDIF
          ENDIF
C
CCC 4. STORE A PROFILE IN DATA STRUCTURES
C
          IF (il_countniv .GT. 0) THEN
C
CCC 4.1 IF THERE ARE VALID OBSERVATIONS, INCREASE THE PROFILE COUNTER
C
              ig_nobs = ig_nobs + 1
C
              IF (ig_nobs .gt. ip_nobsmax) THEN
                  WRITE(*,*) '+ !! Number of profiles ', ig_nobs,
     +                'exceeds ip_nobsmax = ',
     +                ip_nobsmax
                  WRITE(PL_OUT,*) '+ !! Number of profiles ', ig_nobs,
     +                'exceeds ip_nobsmax = ',
     +                ip_nobsmax
                  CALL PALM_Abort(il_err)
              END IF 
C
CCC 4.2 STORE OBSERVATION DATA
C
              iga_dateobs(ig_nobs,1) = ila_dateobs(1)
              iga_dateobs(ig_nobs,2) = ila_dateobs(2)
              iga_dateobs(ig_nobs,3) = ila_dateobs(3)
              iga_dateobs(ig_nobs,4) = ila_dateobs(4)
              iga_dateobs(ig_nobs,5) = ila_dateobs(5)
              cga_str_id(ig_nobs)= cl_str_id
              iga_int_id(ig_nobs)= il_int_id
              iga_inst(ig_nobs)  = ib_inst
              rga_obslat(ig_nobs)   = rl_obslat
              rga_obslon(ig_nobs)   = rl_obslon
              IF (lga_tot_col(ib_inst)) THEN
                  iga_nobslev(ig_nobs) = il_countniv
                  iga_zerolev(ig_nobs) = 0
                  IF (ip_tc_as_prof.eq.1) THEN
                      iga_npreslev(ig_nobs) = ip_nivm1
                  ELSE
                      iga_npreslev(ig_nobs) = il_countniv
                  END IF
              ELSE
                  iga_nobslev(ig_nobs) = il_countniv
                  iga_npreslev(ig_nobs) = il_countniv
                  iga_zerolev(ig_nobs) = 0
              ENDIF
              iga_iq_vertlev(ig_nobs) = ila_iq_vertlev(ib_inst)
C
              ib_j = 0
              DO j=1,max(il_npreslev,ila_iq_vertlev(ib_inst))
                IF (lla_take(j)) THEN 
                    ib_j = ib_j + 1
                    rga_preslev(ig_nobs,ib_j,:) = rla_preslev(j,:)
                    iga_nbconst(ig_nobs,ib_j) = ila_nbconst(j)
                    IF (iga_nbconst(ig_nobs,ib_j).GT.ip_nbconstmax) THEN
                        WRITE(*,*) '+ !! Number of observed species ', 
     +                      iga_nbconst(ig_nobs,ib_j),
     +                      'exceeds ip_nbconstmax = ',
     +                      ip_nbconstmax
                        WRITE(PL_OUT,*) 
     +                      '+ !! Number of observed species ',
     +                      iga_nbconst(ig_nobs,ib_j),
     +                      'exceeds ip_nbconstmax = ',ip_nbconstmax
                        CALL PALM_Abort(il_err)
                    END IF              
                    DO k=1,ila_nbconst(j)
                      cga_constname(ig_nobs,ib_j,k) = cla_constname(j,k)
                      rga_val(ig_nobs,ib_j,k) =
     $                    MAX(rla_val(j,k),
     $                    REAL(ip_enforced_obs_min)*1.e-9)
                      iga_tshold(ig_nobs,ib_j,k) = 3
                    END DO
                END IF
              END DO
C
CCC 4.3 STORE COVARIANCE DATA
C
              il_ncov = il_ncov + 1
C
              ila_covnpreslev(il_ncov) = il_countniv
C
              ib_j = 0
              DO j=1,il_npreslev
                IF (lla_take(j)) THEN 
                    ib_j = ib_j + 1
                    ila_nbcovconst(il_ncov,ib_j) = ib_j 
                    IF (lla_hcov(ib_inst)) THEN
                        ib_k = 0
                        DO k=1,ila_nbcov(j)
                          IF (lla_take(k)) THEN
                              ib_k = ib_k+1
                              rla_cov(il_ncov,ib_j,ib_k,1) =
     $                            rla_cov_rd(j,k,1)
                          END IF
                        END DO
                    ELSE
                        DO k=1,ib_j-1
                          rla_cov(il_ncov,ib_j,k,1) = 0.
                        END DO
                        rla_cov(il_ncov,ib_j,ib_j,1) = 
     $                      (100./(rla_cov_rd(j,1,1)*
     $                      rla_percent(ib_inst)))**2.
                    ENDIF
                END IF
              END DO
C
CCC 4.4 STORE AVERAGING KERNEL DATA
C
              IF (lga_avg_kernel(ib_inst)) THEN
C
                  il_navk = il_navk+1
C
                  ib_j = 0
                  DO j=1,il_npreslev
                    IF (lla_take(j)) THEN 
                        ib_j = ib_j + 1
                        ib_k=0
                        DO k=1,ila_nbavk(j)
                          IF (lla_take(k)) THEN
                              ib_k = ib_k+1
                              rga_avk(il_navk,ib_j,ib_k,1) =
     $                            rla_avk(j,k)
                          END IF
                        END DO
                    END IF
                  END DO
C
                  iga_avk_idx(ig_nobs) = il_navk
C
              END IF
C
          END IF
C
CCC 3.1 END OF LOOP OVER PROFILES
C
        END DO
C
CCC 5. DIAGNOSTIC PRINTS
C
C
        WRITE(*,'(A2)') ' +'
        WRITE(PL_OUT,'(A2)') ' +'
C
        WRITE(*,'(A3,I6,A24,A13)')
     $      '++ ',ig_nobs,' OBSERVATIONS READ IN : ',clfdat
        WRITE(PL_OUT,'(A3,I6,A24,A13)') 
     $      '++ ',ig_nobs,' OBSERVATIONS READ IN : ',clfdat
C
        IF (ANY(lla_hcov)) THEN
            WRITE(*,'(A3,I6,A35,A13)')
     $          '++ ',il_noscov,' INVERSE OS CORRELATIONS READ IN : ',
     $          clfcov
            WRITE(PL_OUT,'(A3,I6,A35,A13)')
     $          '++ ',il_noscov,
     $          ' INVERSE OS CORRELATIONS READ IN : ',
     $          clfcov
        ELSE
            WRITE(*,'(A39)')
     $          '++  USING ONLY PERCENTUAL OBS ERROR RMS'
            WRITE(PL_OUT,'(A39)')
     $          '++  USING ONLY PERCENTUAL OBS ERROR RMS'
        ENDIF
C
        IF (ANY(lga_avg_kernel)) THEN
            WRITE(*,'(A3,I6,A29,A13)')
     $          '++ ',il_nosavk,' AVERAGING KERNELS READ IN : ',clfavk
            WRITE(PL_OUT,'(A3,I6,A29,A13)') '++ ',il_nosavk,
     $          ' AVERAGING KERNELS READ IN : ',clfavk
        END IF
C
        WRITE(*,'(A2)') ' +'
        WRITE(PL_OUT,'(A2)') ' +'
C
        WRITE(*,'(A3,I2,A25)') '++ ',ig_ninst,
     $      ' DIFFERENT INSTRUMENTS : '
        WRITE(PL_OUT,'(A3,I2,A25)') '++ ',ig_ninst,
     $      ' DIFFERENT INSTRUMENTS : '
C
        DO i=1,ig_ninst
          WRITE(*,'(A3,I2,A2,A68)') ' + ',i,') ',cga_inst(i)
          WRITE(PL_OUT,'(A3,I2,A2,A68)') ' + ',i,') ',cga_inst(i)
          IF (lla_hcov(i)) THEN
              WRITE(*,'(A7,A34,A13)')
     $            ' +     ','INVERSE OBS CORRELATION READ IN : ',
     $            clfcov
              WRITE(*,'(A7,A68)') ' +     ',cla_defcov(i)
              WRITE(PL_OUT,'(A7,A34,A13)')
     $            ' +     ','INVERSE OBS CORRELATION READ IN : ',
     $            clfcov
              WRITE(PL_OUT,'(A7,A68)') ' +     ',cla_defcov(i)
          ELSE
              WRITE(*,'(A38,F6.2,A1)')
     $            ' +     USING PERCENTUAL OBS ERROR RMS ',
     $            rla_percent(i),'%'
              WRITE(PL_OUT,'(A38,F6.2,A1)')
     $            ' +     USING PERCENTUAL OBS ERROR RMS ',
     $            rla_percent(i),'%'
          ENDIF
          IF (lga_avg_kernel(i)) THEN
              WRITE(*,'(A7,A27,A13)')
     $            ' +     ','AVERAGING KERNEL READ IN : ',
     $            clfavk
              WRITE(*,'(A7,A68)') ' +     ',cla_defavk(i)
              WRITE(PL_OUT,'(A7,A27,A13)')
     $            ' +     ','AVERAGING KERNEL READ IN : ',
     $            clfavk
              WRITE(PL_OUT,'(A7,A68)') ' +     ',cla_defavk(i)
          END IF
        END DO
C
        WRITE(*,'(A2)') ' +'
        WRITE(PL_OUT,'(A2)') ' +'
C
CCC 6. FORMATS
C
 101    FORMAT(A68)
 102    FORMAT(I4,I2.2,I2.2,I2.2,I2.2,1X,A6,1X,I5.5,1X,
     +         F6.2,1X,F7.2,1X,I2)
 103    FORMAT(F7.0,1X,I2)
 106    FORMAT(F7.0,1X,F7.0,1X,I2)
 104    FORMAT(A5,1X,E12.5)
 105    FORMAT(A7,1X,E12.5)
C
CCC 7. CLOSE UNITS
C
        CLOSE(ila_unit(il_dat))
        IF (ANY(lla_hcov)) THEN
            CLOSE(ila_unit(il_cov))
            DEALLOCATE (cla_defcov)
        END IF
        IF (ANY(lga_avg_kernel)) THEN
            CLOSE(ila_unit(il_avk))
            DEALLOCATE (cla_defavk)
        END IF

C
CCC 8. COMPUTE TIME FROM START (sec) FOR ALL OBSERVATIONS
C
        DO iobs=1,ig_nobs
          DO j=1,5
            idate1(j)=iga_dateobs(iobs,j)
          ENDDO
          CALL assim_timesec(idate0,idate1,iga_secobs(iobs))
        ENDDO
C
CCC 9. INITIALIZE MISFIT TO DISCARDED VALUE -999.
C
        rga_Hx(:,:,:)=-999.
C
CCC 10. CLEAN UP ALLOCATIONS
C
        IF (ANY(lga_avg_kernel)) DEALLOCATE(rla_avk)
        DEALLOCATE (idate1)
        DEALLOCATE (nbjour)
C
CCC 2. END OF OPEN NEW OBSERVATIONAL FILES EVERY DAY
C
      ENDIF
C
CCC 11. SELECT OBSERVATIONS IN CURRENT TIMESTEP
C
      icount=0
      DO iobs=1,ig_nobs
        IF ((INT(daynum).LE.iga_secobs(iobs)).AND.
     +      (INT(daynum+REAL(ip_obs_slot)).GT.iga_secobs(iobs))
     +      ) THEN
          icount=icount+1  
          IF (icount.GT.ip_nobstspmax) THEN
              WRITE(*,*) '+ Number of observations in time slot',
     $            INT(daynum),'->',INT(daynum+REAL(ip_obs_slot))
              WRITE(*,*) '  exceeds ip_nobstspmax = ',ip_nobstspmax
              WRITE(PL_OUT,*) '+ Number of observations in time slot',
     $            INT(daynum),'->',INT(daynum+REAL(ip_obs_slot))
              WRITE(PL_OUT,*) '  exceeds ip_nobstspmax = ',ip_nobstspmax
              CALL PALM_Abort(il_err)
          ENDIF
          ilobs(icount)=iobs
        ENDIF
      ENDDO
C
CCC 11.3. SELECT invR IN CURRENT TIMESTEP
C
      cl_space = 'one_integer'
      il_time = int(daynum) 
      il_usrtag = PL_NO_TAG
      cl_name = 'il_sigmaobs'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     +               il_sigobs,il_err)
C
      IF (il_sigobs .NE. 100 .AND. 
     $    (INT(daynum).EQ.0).AND.ll_initobs) THEN
          WRITE(*,'(A2)') '++'
          WRITE(*,'(A25,A14)') '++ RESCALING OBSERVATION ',
     $        'ERROR VARIANCE'
          WRITE(*,'(A23,I3,A1)') 
     $        ' + USING A % FACTOR OF ', il_sigobs,'%'
          WRITE(*,'(A2)') '++'
          WRITE(PL_OUT,'(A25,A14)') '++ RESCALING OBSERVATION ',
     $        'ERROR VARIANCE'
          WRITE(PL_OUT,'(A23,I3,A1)') 
     $        ' + USING A % FACTOR OF ', il_sigobs,'%'
      ENDIF
C
      DO icov=1,icount
        lcov=ilobs(icov)
        DO j=1,ila_covnpreslev(lcov)
          DO k=1,ila_nbcovconst(lcov,j)
            rga_invR(icov,j,k,1) = rla_cov(lcov,j,k,1)*100.
     $          /REAL(il_sigobs)
            rga_invR(icov,k,j,1) = rga_invR(icov,j,k,1)
         ENDDO
        ENDDO
      END DO
C
CCC 14. CLOSING UP
C
      RETURN
C
      END
C
CCC
C
