SUBROUTINE MCG2NC(cd_filename,idate0,id_time,cda_fieldname,cd_fieldunit,&
   rda_field, ld_hybrid_sigma, dda_dim_lo, dda_dim_la, dda_dim_pres, &
   rda_ALM1, rda_BLM1, rda_psol, rd_ja, id_nobs, &
   id_sigmab, id_filterh, id_filterz, id_Bvar_rep, id_Bhcor_rep, &
   id_Bvcor_rep, id_vcor_1Dapprox)
!
!***
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!  MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!
!
!  Centre National de Recherches Meteorologiques
!
!***
!
! Purpose   : PLOT FIELDS IN NETCDF FORM
!
! Original  : S. Massart, CERFACS/GLOBC, 09/2003
!
!**
!
  USE palmlib
  USE palm_user_param
!
  IMPLICIT NONE
!  
  INCLUDE 'netcdf.inc'
!
!  DUMMY DECLARATION
!
  INTEGER :: id_time
  CHARACTER(LEN=26) :: cd_filename
  CHARACTER(LEN=*), DIMENSION(ip_nassim)  :: cda_fieldname
  CHARACTER(LEN=*)  :: cd_fieldunit
  REAL, DIMENSION(ip_lonm1,ip_latm1,ip_nivm1,ip_nassim) :: rda_field 
  INTEGER, DIMENSION(5) :: idate0
  REAL, DIMENSION(0:ip_nivm1) :: rda_ALM1, rda_BLM1
  REAL, DIMENSION(ip_lonm1,ip_latm1) :: rda_psol
  REAL :: rd_ja
  INTEGER :: id_nobs, id_sigmab, id_filterh, id_filterz
  INTEGER :: id_Bvar_rep, id_Bhcor_rep, id_Bvcor_rep, id_vcor_1Dapprox

!  LOCAL DECLARATION
!    
  REAL(KIND=4), DIMENSION(ip_lonm1,ip_latm1,ip_nivm1) :: rla_field 
  INTEGER :: ib, ib_i, ib_j, ib_k
!
!  NETCDF DECLARATION
!    
  INTEGER :: il_ncfile, il_dlon, il_dlat, il_dniv, il_diter
  INTEGER :: il_flon, il_flat, il_fniv, il_fiter, il_one
  INTEGER :: il_nc_psol, il_nc_ahyb, il_nc_bhyb
  INTEGER :: il_nc_ja, il_nc_nobs
  INTEGER :: il_nc_sigmab, il_nc_filterh, il_nc_filterz
  INTEGER :: il_nc_Bvar_rep, il_nc_Bhcor_rep, il_nc_Bvcor_rep 
  INTEGER :: il_nc_vcor_1Dapprox
  INTEGER :: ila_ncdim(1)
  INTEGER :: ila_ncdims(4), ila_ncstart(4), ila_nccount(4)
  INTEGER, DIMENSION(ip_nassim) :: ila_nc_field
  INTEGER :: il_err
  LOGICAL, INTENT(IN) :: ld_hybrid_sigma
  REAL, DIMENSION(ip_lonm1), INTENT(IN) :: dda_dim_lo
  REAL, DIMENSION(ip_latm1), INTENT(IN) :: dda_dim_la
  REAL(KIND=4), DIMENSION(ip_lonm1) :: rla_dim_lo
  REAL(KIND=4), DIMENSION(ip_latm1) :: rla_dim_la
  REAL(KIND=4), DIMENSION(ip_nivm1) :: rla_dim_ni
  REAL, DIMENSION(ip_nivm1), INTENT(IN) :: dda_dim_pres
  REAL(KIND=4), DIMENSION(ip_nivm1) :: rla_dim_pres
  REAL(KIND=4), DIMENSION(ip_nivm1) :: rla_coef
  REAL(KIND=4), DIMENSION(:,:), ALLOCATABLE :: rla_psol
  REAL(KIND=4) :: rl_ja
  REAL(KIND=4) :: rl_daynum
  CHARACTER(LEN=48) :: cldat, cltori, cl_source
!
!** OPEN NETCDF FILE
!
  il_err = NF_CREATE(cd_filename, NF_CLOBBER, il_ncfile)
!
!** DEFINE DIMENSIONS
!
  il_err = NF_DEF_DIM(il_ncfile, 'time', NF_UNLIMITED, il_diter)
  il_err = NF_DEF_DIM(il_ncfile, 'lat', ip_latm1, il_dlat)
  il_err = NF_DEF_DIM(il_ncfile, 'lon', ip_lonm1, il_dlon)
  IF (ld_hybrid_sigma) THEN
      il_err = NF_DEF_DIM(il_ncfile, 'lev', ip_nivm1, il_dniv)
  ELSE
      il_err = NF_DEF_DIM(il_ncfile, 'pres', ip_nivm1, il_dniv)
  END IF
  il_err = NF_DEF_DIM(il_ncfile, 'one',  1, il_one)
!
!** DEFINE VARIABLES
!
  ila_ncdim(1) = il_diter
  il_err = NF_DEF_VAR(il_ncfile, 'time', NF_REAL, 1, ila_ncdim, il_fiter)
!
  ila_ncdim(1) = il_dlat
  il_err = NF_DEF_VAR(il_ncfile, 'lat', NF_REAL, 1, ila_ncdim, il_flat)
!
  ila_ncdim(1) = il_dlon
  il_err = NF_DEF_VAR(il_ncfile, 'lon', NF_REAL, 1, ila_ncdim, il_flon)
!
  ila_ncdim(1) = il_dniv
  IF (ld_hybrid_sigma) THEN
      il_err = NF_DEF_VAR(il_ncfile, 'lev', NF_REAL, 1, ila_ncdim, il_fniv)
  ELSE
      il_err = NF_DEF_VAR(il_ncfile, 'pres', NF_REAL, 1, ila_ncdim, il_fniv)
  END IF
!
!**  DEFINE ATTRIBUTES
!
  il_ERR = NF_PUT_ATT_TEXT(il_ncfile, NF_GLOBAL, 'Conventions',&
     6, 'CF-1.0')
  cl_source='MOCAGE 3DVAR' 
  il_ERR = NF_PUT_ATT_TEXT(il_ncfile, NF_GLOBAL, 'source',&
     len_trim(cl_source), trim(cl_source))
  il_ERR = NF_PUT_ATT_INT(il_ncfile, NF_GLOBAL, 'levels',&
     NF_INT,1,ip_nivm1)
  il_ERR = NF_PUT_ATT_TEXT(il_ncfile, NF_GLOBAL, 'institution',&
     16, 'CNRM and CERFACS')
!
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_flon,  'standard_name',&
     9, 'longitude')
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_flon, 'units', 12,&
     'degrees_east')
!
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_flat,  'standard_name',&
     8, 'latitude')
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_flat, 'units',13,&
     'degrees_north')
!
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fiter, 'standard_name', 9,&
     'Time axis')
  write(cldat,fmt=&
     '(''seconds since '',&
     &I4.4,"-",I2.2,"-",I2.2," ",I2.2,":",I2.2,":00")')&
     idate0(1),idate0(2),idate0(3),idate0(4),idate0(5)
  write(cltori,fmt=&
     '(I4.4,"-",I2.2,"-",I2.2," ",I2.2,":",I2.2,":00")')&
     idate0(1),idate0(2),idate0(3),idate0(4),idate0(5)
!
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fiter,'axis',  1, 't')
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fiter,'units',&
     LEN_TRIM(cldat), cldat)
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fiter,'time_origin',&
     LEN_TRIM(cltori), cltori)
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fiter,'calendar', 6,&
     'noleap')
!
  IF (ld_hybrid_sigma) THEN
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fniv,  'standard_name',&
	 43, 'atmosphere_hybrid_sigma_pressure_coordinate')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fniv,  'formula_terms'&
	 , 60,&
	 'ap: a_hybr_coord b: b_hybr_coord ps: air_pressure_at_surface')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fniv, 'units', 3, 'lev')
!
!   SURFACE PRESSURE (2D)
!
      ila_ncdims(1) = il_dlon
      ila_ncdims(2) = il_dlat
      ila_ncdims(3) = il_diter
!
      il_err = NF_DEF_VAR(il_ncfile, 'air_pressure_at_surface',  &
         NF_REAL,3, ila_ncdims, il_nc_psol)
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_psol, 'standard_name', &
         23, 'air_pressure_at_surface')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_psol, 'units', &
         2, 'Pa')
!
!  HYBRID COORDINATE COEFFICIENTS (2*1D)
!
      ila_ncdims(1) = il_dniv
!
      il_err = NF_DEF_VAR(il_ncfile, 'a_hybr_coord', &
         NF_REAL,1, ila_ncdims, il_nc_ahyb)
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_ahyb, 'long_name', &
         34, 'a coefficient of hybrid coordinate')
!
      il_err = NF_DEF_VAR(il_ncfile, 'b_hybr_coord', &
         NF_REAL,1, ila_ncdims, il_nc_bhyb)
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_bhyb, 'long_name', &
	 34, 'b coefficient of hybrid coordinate')
!
  ELSE
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fniv, 'units', 2, 'Pa')
  END IF
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fniv,  'positive',&
     4, 'down')
!
!** DEFINE VARIABLES AND ATTRIBUTES
!
  ila_ncdims(1) = il_one
  ila_ncdims(2) = il_diter
  il_err = NF_DEF_VAR(il_ncfile, 'j_xa',   &
     NF_REAL,2, ila_ncdims, il_nc_ja)
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_ja, &
     'long_name', 24, 'Cost function at optimum')
!
  ila_ncdims(1) = il_one
  ila_ncdims(2) = il_diter
  il_err = NF_DEF_VAR(il_ncfile, 'nobs',   &
     NF_INT,2, ila_ncdims, il_nc_nobs)
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_nobs, &
     'long_name', 40, 'Total number of assimilated observations')
!
  ila_ncdims(1) = il_one
  il_err = NF_DEF_VAR(il_ncfile, 'Bvar_rep',   &
     NF_INT,1, ila_ncdims, il_nc_Bvar_rep)
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_Bvar_rep, &
     'long_name', 33, 'Bckg err. variance representation')
!
  ila_ncdims(1) = il_one
  il_err = NF_DEF_VAR(il_ncfile, 'sigmab',   &
     NF_INT,1, ila_ncdims, il_nc_sigmab)
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_sigmab, &
     'long_name', 21, 'Rms on background (%)')
!
  ila_ncdims(1) = il_one
  il_err = NF_DEF_VAR(il_ncfile, 'Bhcor_rep',   &
     NF_INT,1, ila_ncdims, il_nc_Bhcor_rep)
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_Bhcor_rep, &
     'long_name', 35, 'Bckg err. hor correl representation')
!
  ila_ncdims(1) = il_one
  il_err = NF_DEF_VAR(il_ncfile, 'filterh',   &
     NF_INT,1, ila_ncdims, il_nc_filterh)
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_filterh, &
     'long_name', 28, 'Horizontal correlation scale')
!
  ila_ncdims(1) = il_one
  il_err = NF_DEF_VAR(il_ncfile, 'Bvcor_rep',   &
     NF_INT,1, ila_ncdims, il_nc_Bvcor_rep)
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_Bvcor_rep, &
     'long_name', 36, 'Bckg err. vert correl representation')
!
  ila_ncdims(1) = il_one
  il_err = NF_DEF_VAR(il_ncfile, 'filterz',   &
     NF_INT,1, ila_ncdims, il_nc_filterz)
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_filterz, &
     'long_name', 26, 'Vertical correlation scale')
!
  ila_ncdims(1) = il_one
  il_err = NF_DEF_VAR(il_ncfile, 'vcor_1Dapprox',   &
     NF_INT,1, ila_ncdims, il_nc_vcor_1Dapprox)
  il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_vcor_1Dapprox, &
     'long_name', 36, 'Press discretisation for vert correl')
!
  ila_ncdims(1) = il_dlon
  ila_ncdims(2) = il_dlat
  ila_ncdims(3) = il_dniv
  ila_ncdims(4) = il_diter
!
  DO ib = 1, ip_nassim
    il_err = NF_DEF_VAR(il_ncfile, cda_fieldname(ib), NF_REAL,4,&
       & ila_ncdims, ila_nc_field(ib))
    il_err = NF_PUT_ATT_TEXT(il_ncfile, ila_nc_field(ib), 'units',&
       & LEN_TRIM(cd_fieldunit), cd_fieldunit)
  END DO
!
!** END OF DEFINE MODE
!
  il_err= NF_ENDDEF(il_ncfile)
!
!** WRITE DATA TO FILE
!
  rla_dim_lo(:) = REAL(dda_dim_lo)
  il_err = NF_PUT_VAR_REAL(il_ncfile,il_flon,rla_dim_lo)
!
  rla_dim_la(:) = REAL(dda_dim_la)
  il_err = NF_PUT_VAR_REAL(il_ncfile,il_flat,rla_dim_la)
!
  IF (ld_hybrid_sigma) THEN
      DO ib_k = 1, ip_nivm1
	rla_dim_ni(ib_k) = REAL(ib_k)
      END DO
      il_err = NF_PUT_VAR_REAL(il_ncfile,il_fniv,rla_dim_ni)
!
      ALLOCATE (rla_psol(ip_lonm1, ip_latm1))
      rla_psol = REAL(rda_psol)
      ila_ncstart(1) = 1
      ila_ncstart(2) = 1
      ila_ncstart(3) = 1
      ila_nccount(1) = ip_lonm1
      ila_nccount(2) = ip_latm1
      ila_nccount(3) = 1
      il_err = NF_PUT_VARA_REAL(il_ncfile,il_nc_psol, ila_ncstart, &
	 ila_nccount,rla_psol)
      DEALLOCATE (rla_psol)
!      
      ila_ncstart(1) = 1
      ila_nccount(1) = ip_nivm1
!
      rla_coef=REAL(rda_alm1(1:ip_nivm1))
      il_err = NF_PUT_VARA_REAL(il_ncfile,il_nc_ahyb, ila_ncstart, &
         ila_nccount,rla_coef)
!
      rla_coef=REAL(rda_blm1(1:ip_nivm1))
      il_err = NF_PUT_VARA_REAL(il_ncfile,il_nc_bhyb, ila_ncstart, &
         ila_nccount,rla_coef)
!      
  ELSE
      rla_dim_pres(:) = REAL(dda_dim_pres)
      il_err = NF_PUT_VAR_REAL(il_ncfile,il_fniv,rla_dim_pres)
  END IF
!  
  ila_ncstart(1) = 1
  ila_nccount(1) = 1
  rl_daynum=REAL(id_time)
  il_err = NF_PUT_VARA_REAL(il_ncfile,il_fiter,ila_ncstart,&
     ila_nccount,rl_daynum)
!
  ila_ncstart(1) = 1
  ila_ncstart(2) = 1
  ila_ncstart(3) = 1
  ila_ncstart(4) = 1
  ila_nccount(1) = ip_lonm1
  ila_nccount(2) = ip_latm1
  ila_nccount(3) = ip_nivm1
  ila_nccount(4) = 1
  DO ib = 1,ip_nassim
    rla_field=REAL(rda_field(:,:,:,ib))
    il_err = NF_PUT_VARA_REAL(il_ncfile,ila_nc_field(ib), ila_ncstart,&
     & ila_nccount,rla_field)
  END DO
!
  ila_ncstart(1) = 1
  ila_ncstart(2) = 1
  ila_nccount(1) = 1
  ila_nccount(2) = 1
!
  rl_ja = REAL(rd_ja)
  il_err = NF_PUT_VARA_REAL(il_ncfile,il_nc_ja, ila_ncstart,&
      ila_nccount,rl_ja)
  il_err = NF_PUT_VARA_INT(il_ncfile,il_nc_nobs, ila_ncstart,&
      ila_nccount,id_nobs)
!
  il_err = NF_PUT_VAR_INT(il_ncfile,il_nc_Bvar_rep,id_Bvar_rep)
  il_err = NF_PUT_VAR_INT(il_ncfile,il_nc_sigmab,id_sigmab)
!
  il_err = NF_PUT_VAR_INT(il_ncfile,il_nc_Bhcor_rep,id_Bhcor_rep)
  il_err = NF_PUT_VAR_INT(il_ncfile,il_nc_filterh,id_filterh)
  il_err = NF_PUT_VAR_INT(il_ncfile,il_nc_Bvcor_rep,id_Bvcor_rep)
  il_err = NF_PUT_VAR_INT(il_ncfile,il_nc_filterz,id_filterz)
!
  il_err = NF_PUT_VAR_INT(il_ncfile,il_nc_vcor_1Dapprox,id_vcor_1Dapprox)
!
!** CLOSE THE NETCDF FILE
!
  il_err = NF_CLOSE(il_ncfile)
!
  RETURN
END SUBROUTINE MCG2NC
