      SUBROUTINE PROPAGATE_INCR
C
C***
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C  MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C
C
C  Centre National de Recherches Meteorologiques
C
C***
C
C Purpose   : Propagate dx along the assimilation window
C             It is the identity for the 3DFGAT
C             and the linear tangent model for the 4DINC
C
C Original  : A. Piacentini, CERFACS, 07/2006
C
C**
C
C
      USE palmlib
      USE palm_user_param
C
      IMPLICIT NONE
C
C Local Declaration
C
      INTEGER :: il_starttime, il_endtime
      INTEGER :: il_dimx
      INTEGER :: ib_time
      INTEGER :: il_dt0

      REAL, DIMENSION(:), ALLOCATABLE :: rda_dx
C
C PALM Declaration
C
      CHARACTER(LEN=PL_LNAME) cl_name, cl_space
      INTEGER il_time, il_usrtag, il_err, il_err2
      INTEGER il_day, il_month, il_year, il_hour, il_min, il_sec
C
C***
C
C**  ALLOCATE MEMORY
C
      il_dimx = ip_lono1*ip_lato1*ip_nivm1*ip_nassim
      ALLOCATE( rda_dx(il_dimx) )
C
C**  Get Loop informations
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_starttime'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $    il_starttime,il_err)
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_endtime'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $    il_endtime,il_err)
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_step'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $    il_dt0,il_err)
C
C
C**  GET CONTROL VECTOR dx
C
      cl_space = 'increment_space'
      il_time = il_starttime
      il_usrtag = PL_NO_TAG
      cl_name = 'rda_dx'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $    rda_dx,il_err) 
C  
C**  PROPAGATE AND PUT CONTROL VECTOR dx
C
      DO ib_time = il_starttime, il_endtime, il_dt0
        IF (ip_iprint .EQ. 1) THEN
            WRITE(*,*) 'FWD TL Sending increment at time ', ib_time
            WRITE(PL_OUT,*) 'FWD TL Sending increment at time ', ib_time
            CALL FLUSH(PL_OUT)
        END IF
C*   HERE in the 4DINC there would be a forward propagation
C*   of rda_dx from ib_time to ib_time+rl_dt0
C*   (more exactly rda_dx for il_starttime would be sent before the 
C*   loop and the others inside the loop after the propagation step)
C*   in the 3DFGAT this is the identity
        cl_space = 'increment_space'
        il_time = ib_time
        il_usrtag = PL_NO_TAG
        cl_name = 'rda_dxi'
        CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,
     $      rda_dx,il_err) 
      END DO
C
C**  DEALLOCATE MEMORY
C
      DEALLOCATE(rda_dx)
C
      END SUBROUTINE PROPAGATE_INCR
