 SUBROUTINE VERT_COREL_ADJ(rda_v,id_nprocs,id_myid)
    !
    ! 0. DECLARATION
    !------------------
    !
    ! 0.1. Modules declaration
    !
    USE palm_user_param
    USE SQRTB_MOD
    USE palmlib
    !
    ! 0.2. Implicit
    !
    IMPLICIT NONE
    !
#include "mpif.h"
    !
    ! 0.3. Dummy variables
    !
    INTEGER :: id_nprocs,id_myid
    REAL, DIMENSION(ip_lono1,ip_lato1,ip_nivm1) :: rda_v
    !
    ! 0.3. Local variables
    !
    INTEGER :: il_loopstart, il_loopend, il_err
    REAL, DIMENSION(:,:,:,:), ALLOCATABLE :: rla_work
    REAL :: rl_int
    !
    ! 0.3. Loop variables
    !
    INTEGER :: ib_iter
    INTEGER :: ib_lat
    INTEGER :: ib_lon
    INTEGER :: ib_niv
    !
    !
    ! 1. DOMAIN BOUNDS
    !------------------
    !
    IF (id_nprocs.GT.1) THEN
      il_loopstart=id_myid*(ip_lono1/id_nprocs)+1
      IF (id_myid .NE. id_nprocs-1) THEN
	  il_loopend=(id_myid+1)*(ip_lono1/id_nprocs)
      ELSE
	  il_loopend=ip_lono1
      ENDIF
    ELSE
      il_loopstart = 1
      il_loopend = ip_lono1
    ENDIF
    !
    ! 2. COEFFICIENT DE NORMALISATION
    !------------------------------------------
    !
    ALLOCATE(rla_work(ip_lono1,ip_lato1,1:ip_nivm1,1))
    rla_work = 0. 
    rla_work(il_loopstart:il_loopend,:,:,1) &
        =  rda_v(il_loopstart:il_loopend,:,:)
    !
    IF (lg_vcor_1D) THEN
      DO ib_niv = 1, ip_nivm1
        rla_work(il_loopstart:il_loopend,:,ib_niv,1) = &
            rla_work(il_loopstart:il_loopend,:,ib_niv,1)  &
            * rga_filnoz(1,1,ib_niv)
      ENDDO
    ELSE
      rla_work(il_loopstart:il_loopend,:,:,1) = &
          rla_work(il_loopstart:il_loopend,:,:,1) &
          * rga_filnoz(il_loopstart:il_loopend,:,:)
    ENDIF
    !
    ! 3. BOUCLE TEMPORELLE
    !------------------------------------------
    !
    DO ib_lon=il_loopstart,il_loopend
    DO ib_lat=1, ip_lato1
      IF (lg_vcor_1D) THEN
        rla_work(ib_lon,ib_lat,:,:) = MATMUL(&
           TRANSPOSE(rga_mat_vcor(1,1,:,:)),&
           rla_work(ib_lon,ib_lat,:,:))
      ELSE
        rla_work(ib_lon,ib_lat,:,:) = MATMUL(TRANSPOSE(rga_mat_vcor(ib_lon,ib_lat,:,:)),&
                                         rla_work(ib_lon,ib_lat,:,:))
!        CALL DGEMV('T',ip_nivm1,ip_nivm1,1.,rga_mat_vcor(ib_lon,ib_lat,:,:),&
!           ip_nivm1, &
!           rla_work(ib_lon,ib_lat,:,1),1,0.,rla_work(ib_lon,ib_lat,:,1),1)

      ENDIF
    ENDDO
    ENDDO
    !
    ! 4. COEFICIENT METRIC
    !------------------------------------------
    !
    IF (lg_vcor_1D) THEN
      DO ib_niv = 1, ip_nivm1
        rla_work(il_loopstart:il_loopend,:,ib_niv,1) =&
            rla_work(il_loopstart:il_loopend,:,ib_niv,1) &
            / SQRT(rga_Wz(1,1,ib_niv))
      ENDDO
    ELSE
      rla_work(il_loopstart:il_loopend,:,:,1) = &
            rla_work(il_loopstart:il_loopend,:,:,1)&
            / SQRT(rga_Wz(il_loopstart:il_loopend,:,:))
    ENDIF
    !
    ! 5. COLLECT THE RESULT
    !------------------------------------------
    !
    IF (id_nprocs.GT.1) THEN
      CALL MPI_AllReduce(rla_work(:,:,1:ip_nivm1,1),rda_v,&
                        ip_lono1*ip_lato1*ip_nivm1, &
                  	MPI_DOUBLE_PRECISION,MPI_SUM,PL_COMM_EXEC,il_err)
    ELSE
      rda_v(:,:,:) = rla_work(:,:,1:ip_nivm1,1)
    ENDIF
    DEALLOCATE(rla_work)
    !
    ! 5. FIN DE LA ROUTINE
    !------------------------------------------
    !
END SUBROUTINE VERT_COREL_ADJ

