SUBROUTINE pl_trigger_unit(cd_unit)
!
!****  pl_trigger_unit - List of units and their identifier
!
!     Purpose:
!     --------
!       User subroutine to describe the unit list.
!       This subroutine is called by the PALM library to active
!       a new unit.
!
!***  Interface:
!     ----------
!         CALL pl_trigger_unit(cd_unit)
!
!      Input:
!      ------
!         cd_unit: Unit identifier
!
!     Externals:
!     ----------
!       Modules:
!       --------
!          palmlib, pl_def_err
!
!     History:
!     -------
!     file automaticaly generated by PREPALM, don't edit this file
!*----------------------------------------------------------------
!
!** 1. DECLARATION
!
!** 1.1 Modules
!
!
  USE palmlib
  USE pl_def_err
  

  IMPLICIT NONE

  CHARACTER(LEN=PL_LNAME) :: cd_unit

  
  cd_unit = ADJUSTL(cd_unit)
  SELECT CASE(cd_unit)
  CASE('Branch_one')
      CALL Branch_one
  CASE('Branch_obs')
      CALL Branch_obs
  CASE('MODEL')
      CALL MODEL
  CASE('MODEL_fcst')
      CALL MODEL
  CASE('INV_R')
      CALL INV_R
  CASE('WRTOBS')
      CALL WRTDAT
  CASE('Bkwd_ADJ')
      CALL backward_propagation
  CASE('Fwd_TL')
      CALL propagate_incr
  CASE('increment_to_model')
      CALL gauss2model
  CASE('p_sigma_to_gauss')
      CALL model2gauss
  CASE('model2gauss_ini')
      CALL model2gauss
  CASE('model2gauss')
      CALL model2gauss
  CASE('INIT')
      CALL INIT
  CASE('INIT_OBS')
      CALL OBSINIT
  CASE('sqrtB')
      CALL sqrtB
  CASE('sqrtB_fcst')
      CALL sqrtB
  CASE('sqrtB_T')
      CALL sqrtB_adj
  CASE('INISTATS')
      CALL INISTATS
  CASE('plot_inc')
      CALL plot_inc
  CASE('LECTOBS')
      CALL LECTDAT
  CASE('H')
      CALL H_NOV
  CASE('H_LT')
      CALL H_NOV_LT
  CASE('Hstar')
      CALL H_NOV_ADJ
  CASE('INITv')
      CALL INITv
  CASE DEFAULT
	WRITE(PL_OUT,*) 'Error: Unit Not found!'
        call flush(PL_OUT)
  END SELECT
  
END SUBROUTINE pl_trigger_unit
!
!*=========================================================================
!
SUBROUTINE pl_trigger_brch(cd_branch)
!
!****  pl_trigger_brch - List of branches and their identifier
!
!     Purpose:
!     --------
!       User subroutine to describe the branch list.
!       This subroutine is called by the PALM library to active
!       a new branch.
!
!***  Interface:
!     ----------
!           CALL pl_trigger_branch(cd_branch)
!
!      Input:
!      ------
!         cd_unit: branch identifier
!
!
!     Externals:
!     ----------
!       Modules:
!       --------
!          palmlib, pl_def_err
!
!     History:
!     -------
!     file automaticaly generated by PREPALM, don't edit this file
!*----------------------------------------------------------------
!
!** 1. DECLARATION
!
!** 1.1 Modules
!
  USE palmlib
  USE pl_def_err

  IMPLICIT NONE

  CHARACTER(LEN=PL_LNAME) :: cd_branch

  cd_branch = ADJUSTL(cd_branch)

  SELECT CASE(cd_branch)
  CASE('Branch_one')
      CALL Branch_one
  CASE('Branch_obs')
      CALL Branch_obs
  CASE DEFAULT
  WRITE(PL_OUT,*) 'Error: Branch Not found!'
  call flush(PL_OUT)

  END SELECT

END SUBROUTINE pl_trigger_brch
!
!*=================================================================
!

SUBROUTINE pl_descriptor_trigger(cd_function_name, id_action, id_nproc, &
   id_arank, ida_shape, id_size, ida_descr)
!
!****  pl_descriptor_trigger
!
!     Purpose:
!     --------
!       User subroutine for calling a descriptor function.
!       These functions allow to describe the distribution of an object on
!       a unit.  
!       It must have in input : 
!         id_action        : contains PL_GIVE_SIZE if the user function must
!                            return the size of the descriptor vector, 
!                            PL_GIVE_DESCR if the user function must return the
!                            filled descriptor vector
!         id_nproc         : number of processor of the unit on which the 
!                            object is distributed
!         id_arank         : rank of the global array
!         ida_shape        : shape of the global array
!       in output :
!         id_size : the size of the descriptor vector ida_descr
!         ida_descr : the descriptor vector 
!       For the custom descriptor, the descriptor vector must contains :
!       For each processor :
!          - number of blocs
!          - shape of the local array
!          For each bloc :
!             - shape of the bloc
!             - global start of the bloc
!             - local start of the bloc
!          End for
!       End for  
!       
!
!***  Interface:
!     ----------
!         CALL pl_descriptor_trigger(cd_function_name, id_action, id_nproc, &
!   id_arank, ida_shape, id_size, ida_descr)
!
!      Inputs:
!      -------
!         cd_function_name : name of the function to call
!         id_action        : contains PL_GIVE_SIZE if the user function must
!                            return the size of the descriptor vector, 
!                            PL_GIVE_DESCR if the user function must return the
!                            filled descriptor vector
!         id_nproc         : number of processor of the unit on which the 
!                            object is distributed
!         id_arank         : rank of the global array
!         ida_shape        : shape of the global array
!
!      Outputs:
!      --------
!         id_size : the size of the descriptor vector ida_descr
!         ida_descr : the descriptor vector 
!
!

!     Externals:
!     ----------
!       Modules:
!       --------
!          palmlib, pl_def_err
!
!     History:
!     -------
!     file automaticaly generated by PREPALM, don't edit this file
!*----------------------------------------------------------------
!
!** 1. DECLARATION
!
!** 1.1 Modules
!
  USE palmlib
!  
  IMPLICIT NONE
!

  CHARACTER(LEN=PL_LNAME)       :: cd_function_name
  INTEGER                       :: id_nproc, id_arank, il_err
  INTEGER, DIMENSION(id_arank)  :: ida_shape
  INTEGER                       :: id_action, id_size
  INTEGER, DIMENSION(id_size)   :: ida_descr
!
!*----------------------------------------------------------------
!
  SELECT CASE(cd_function_name)

  CASE DEFAULT
         WRITE(PL_OUT,*) 'Error: Function Not found!'
         call flush(PL_OUT)
         CALL PALM_ABORT(il_err)

  END SELECT

END SUBROUTINE pl_descriptor_trigger
SUBROUTINE pl_algebra_trigger(cd_routine_name, sda_arguments, id_alg_index, &
   id_place)
!
!**** pl_algebra_trigger
!
!     Purpose:
!       call the palm routine envelopping a given algebra routine
!
!
!** + Declarations
!
!** ++ Include files and modules
!
  USE palmlib  !*I contains some parameters (PL_LNAME)
  USE palmdef  !*I contains some parameters and global variables (cg_msg)
  USE pl_def_err !*I for the treatment of the errors
  USE pl_algebra
!
!** ++ Local variables 
!
  IMPLICIT NONE
!
  !*V cp_sub : a parameter for tracking the subroutine
  CHARACTER(LEN=PL_SLGTH), PARAMETER :: cp_sub = 'pl_algebra_trigger' 
!
  !** number of argument of the algebra routine
  INTEGER :: il_nb_arg
!

  INTEGER :: A1(*)
  INTEGER :: A2(*)
  INTEGER :: A3(*)
  INTEGER :: A4(*)
  INTEGER :: A5(*)
  INTEGER :: A6(*)
  INTEGER :: A7(*)
  INTEGER :: A8(*)
  INTEGER :: A9(*)
  INTEGER :: A10(*)
  INTEGER :: A11(*)
  INTEGER :: A12(*)
  INTEGER :: A13(*)
  INTEGER :: A14(*)
  POINTER (P1,A1)
  POINTER (P2,A2)
  POINTER (P3,A3)
  POINTER (P4,A4)
  POINTER (P5,A5)
  POINTER (P6,A6)
  POINTER (P7,A7)
  POINTER (P8,A8)
  POINTER (P9,A9)
  POINTER (P10,A10)
  POINTER (P11,A11)
  POINTER (P12,A12)
  POINTER (P13,A13)
  POINTER (P14,A14)
!
!
!** ++ Dummy variables 
!
  !*V sda_arguments : array containing the arguments of the algebra routine
  !*V                 but converted in a vector of integers format  
  TYPE(argument_t), DIMENSION(*) :: sda_arguments
!
  !*V cd_routine_name : the name of the palm routine envelopping the algebra 
  !*V                   routine we want to call 
  CHARACTER(LEN=PL_LNAME) :: cd_routine_name  
!
  !*V id_alg_index : index of the algebra routine in the algebra table
  INTEGER :: id_alg_index
!
  INTEGER :: id_place, il_err
!
!*------------------------------------------------------------------------
!
  il_nb_arg = sga_alg_table(id_alg_index)%il_nb_arguments
!
  !** Set the pointers of the arguments of the algebra routine
  !** It's a little bit ugly but indispensable because the mslaves uses CRAY 
  !** pointers
  IF (id_place.eq.PL_BUFF) THEN

      IF (il_nb_arg.ge.1) CALL pl_set_P(sda_arguments(1),P1)  
      IF (il_nb_arg.ge.2) CALL pl_set_P(sda_arguments(2),P2)  
      IF (il_nb_arg.ge.3) CALL pl_set_P(sda_arguments(3),P3)  
      IF (il_nb_arg.ge.4) CALL pl_set_P(sda_arguments(4),P4)  
      IF (il_nb_arg.ge.5) CALL pl_set_P(sda_arguments(5),P5)  
      IF (il_nb_arg.ge.6) CALL pl_set_P(sda_arguments(6),P6)  
      IF (il_nb_arg.ge.7) CALL pl_set_P(sda_arguments(7),P7)  
      IF (il_nb_arg.ge.8) CALL pl_set_P(sda_arguments(8),P8)  
      IF (il_nb_arg.ge.9) CALL pl_set_P(sda_arguments(9),P9)  
      IF (il_nb_arg.ge.10) CALL pl_set_P(sda_arguments(10),P10)  
      IF (il_nb_arg.ge.11) CALL pl_set_P(sda_arguments(11),P11)  
      IF (il_nb_arg.ge.12) CALL pl_set_P(sda_arguments(12),P12)  
      IF (il_nb_arg.ge.13) CALL pl_set_P(sda_arguments(13),P13)  
      IF (il_nb_arg.ge.14) CALL pl_set_P(sda_arguments(14),P14)  
  ELSE
      IF (il_nb_arg.ge.1) P1 = LOC(sda_arguments(1)%ila_value(1)) 
      IF (il_nb_arg.ge.2) P2 = LOC(sda_arguments(2)%ila_value(1)) 
      IF (il_nb_arg.ge.3) P3 = LOC(sda_arguments(3)%ila_value(1)) 
      IF (il_nb_arg.ge.4) P4 = LOC(sda_arguments(4)%ila_value(1)) 
      IF (il_nb_arg.ge.5) P5 = LOC(sda_arguments(5)%ila_value(1)) 
      IF (il_nb_arg.ge.6) P6 = LOC(sda_arguments(6)%ila_value(1)) 
      IF (il_nb_arg.ge.7) P7 = LOC(sda_arguments(7)%ila_value(1)) 
      IF (il_nb_arg.ge.8) P8 = LOC(sda_arguments(8)%ila_value(1)) 
      IF (il_nb_arg.ge.9) P9 = LOC(sda_arguments(9)%ila_value(1)) 
      IF (il_nb_arg.ge.10) P10 = LOC(sda_arguments(10)%ila_value(1)) 
      IF (il_nb_arg.ge.11) P11 = LOC(sda_arguments(11)%ila_value(1)) 
      IF (il_nb_arg.ge.12) P12 = LOC(sda_arguments(12)%ila_value(1)) 
      IF (il_nb_arg.ge.13) P13 = LOC(sda_arguments(13)%ila_value(1)) 
      IF (il_nb_arg.ge.14) P14 = LOC(sda_arguments(14)%ila_value(1)) 
  END IF
!
!
  SELECT CASE (cd_routine_name)
!

   CASE ('PL_DTESTGRAD')
       CALL PL_DTESTGRAD(A1,A2,A3,A4,A5,A6,A7,A8,A9,A10,A11,A12,A13, id_alg_index)
!
   CASE ('PL_SAXPY')
       CALL PL_SAXPY(A1,A2,A3,A4,A5,A6, id_alg_index)
!
   CASE ('PL_SDOT')
       CALL PL_SDOT(A1,A2,A3,A4,A5,A6,A7, id_alg_index)
!
  CASE default
!
     WRITE(PL_OUT,*) 'unknown algebra routine name ',cd_routine_name
     CALL PALM_ABORT(il_err)
!
  END SELECT
!
END SUBROUTINE pl_algebra_trigger
! 
!
!
! algebra triggers or dummy trigger
!
!
SUBROUTINE pl_saxpy(N, alpha, X, incx, Y, incy, id_alg_index)
  USE palmlib  !*I contains some parameters (PL_LNAME)
  USE pl_algebra
  IMPLICIT NONE
  INTEGER, INTENT(IN)                                :: N, incx, incy 
  REAL, INTENT(IN)                  :: alpha
  REAL, DIMENSION(*), INTENT(IN)    :: X
  REAL, DIMENSION(*), INTENT(INOUT) :: Y
  !*V id_alg_index : index of the algebra routine in the algebra table
  INTEGER :: id_alg_index
  !*V cp_sub : a parameter for tracking the subroutine
  CHARACTER(LEN=PL_SLGTH), PARAMETER :: cp_sub = 'pl_daxpy' 
  CHARACTER(LEN=PL_LNAME) :: cl_name
  CALL saxpy(N, alpha, X, incx, Y, incy)
  cl_name = 'All'
  CALL pl_put_obj(cl_name, .true., id_alg_index)
END SUBROUTINE pl_saxpy

SUBROUTINE pl_sdot(N, alpha, X, incx, Y, incy, dot, id_alg_index)
  USE palmlib  !*I contains some parameters (PL_LNAME)
  USE pl_algebra
  USE pl_def_err  !*I for the treatment of errors
  IMPLICIT NONE
  INTEGER, INTENT(IN)                                :: N, incx, incy 
  REAL, INTENT(IN)                  :: alpha
  REAL, DIMENSION(*), INTENT(IN)    :: X
  REAL, DIMENSION(*), INTENT(INOUT) :: Y
  REAL                              :: dot
  !*V id_alg_index : index of the algebra routine in the algebra table
  INTEGER :: id_alg_index
  !*V cp_sub : a parameter for tracking the subroutine
  CHARACTER(LEN=PL_SLGTH), PARAMETER :: cp_sub = 'pl_sdot' 
  CHARACTER(LEN=PL_LNAME) :: cl_name
  REAL   :: rf_sdot_doit
  dot = rf_sdot_doit(N, alpha, X, incx, Y, incy)
  cl_name = 'All'
  CALL pl_put_obj(cl_name, .true., id_alg_index)
END SUBROUTINE pl_sdot

REAL FUNCTION rf_sdot_doit(N, alpha, X, incx, Y, incy)
  IMPLICIT NONE
  INTEGER, INTENT(IN)                                :: N, incx, incy 
  REAL, INTENT(IN)                  :: alpha
  REAL, DIMENSION(*), INTENT(IN)    :: X
  REAL, DIMENSION(*), INTENT(INOUT) :: Y
  !*V cp_sub : a parameter for tracking the subroutine
  REAL   :: sdot
  rf_sdot_doit = alpha * sdot(N, X, incx, Y, incy)
END FUNCTION rf_sdot_doit

SUBROUTINE PL_DTESTGRAD(id_mode, id_n, dda_x, dda_dx, dda_y, dd_f,  &
                                dda_g, dda_residu, dd_alpha0, dd_alphastep, &
				id_niter, id_iprint, id_flushflag, id_alg_index)

    USE palmlib
    USE pl_algebra
    IMPLICIT NONE
    INTEGER :: id_alg_index
    INTEGER, INTENT( IN )   :: id_n
    INTEGER, INTENT( IN )   :: id_niter
    INTEGER, INTENT( IN )   :: id_iprint
    INTEGER, INTENT( IN )   :: id_flushflag
    INTEGER, INTENT( OUT )  :: id_mode
    
    DOUBLE PRECISION, INTENT( IN )  :: dd_f
    DOUBLE PRECISION, INTENT( IN )     :: dd_alpha0
    DOUBLE PRECISION, INTENT( IN )     :: dd_alphastep

    DOUBLE PRECISION, DIMENSION(id_n), INTENT( IN )     :: dda_dx
    DOUBLE PRECISION, DIMENSION(id_n), INTENT( IN )     :: dda_g
    DOUBLE PRECISION, DIMENSION(id_n), INTENT( INOUT )  :: dda_x
    DOUBLE PRECISION, DIMENSION(id_n), INTENT( OUT )    :: dda_y
    
    DOUBLE PRECISION, DIMENSION(id_niter,5), INTENT( OUT )    :: dda_residu
    CHARACTER(LEN=PL_LNAME) :: cl_name
    INTEGER :: ib_do
    INTEGER :: il_io
    INTEGER, SAVE :: il_nbcall
    DOUBLE PRECISION, SAVE :: dl_f0
    DOUBLE PRECISION, SAVE :: dl_alpha 
    DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE, SAVE :: dla_dx0
    DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE, SAVE :: dla_x0
    DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE, SAVE :: dla_g0
    DOUBLE PRECISION, DIMENSION(:,:), ALLOCATABLE, SAVE :: dla_results
    DOUBLE PRECISION :: dnrm2
    LOGICAL, SAVE :: FIRSTIME = .TRUE. 
  IF (FIRSTIME) THEN
    il_nbcall = 0
    IF (id_iprint.gt.0) THEN
	il_io = 87654
	OPEN(il_io, file = 'testgrad.out', position='append')

	WRITE(il_io,*)
	WRITE(il_io,*)'----------------------'
	WRITE(il_io,*)'TEST_GRAD: entry point'
	WRITE(il_io,*)'----------------------'
	WRITE(il_io,201) id_n
	WRITE(il_io,202) dl_alpha
	WRITE(il_io,203) dd_alphastep
	WRITE(il_io,204) id_niter
	WRITE(il_io,205) id_iprint
	WRITE(il_io,*)

	IF (id_flushflag .eq. 1) CALL flush(il_io)

    END IF
    IF (id_n.le.0 .OR. id_niter.le.0) THEN

	id_mode = -2

	IF (id_iprint.gt.0) THEN
	    WRITE(il_io,*) 'ERROR: Invalid value for n or niter.'
	    WRITE(il_io,301) id_mode
	    IF (id_flushflag .eq. 1) CALL flush(il_io)
	END IF
	
	! DIRECTLY GO TO THE END OF THE ROUTINE
	GOTO 999

    END IF
    IF (id_iprint.gt.0) THEN

	WRITE(il_io,206) dnrm2(id_n, dda_x, 1)
	WRITE(il_io,207) dnrm2(id_n, dda_dx, 1)
	WRITE(il_io,208) dd_f
	WRITE(il_io,209) dnrm2(id_n, dda_g, 1)
	IF (id_iprint .eq. 2) THEN
	    WRITE(il_io,*)'----------------------'
	    WRITE(il_io,*)'iter  a            f(x+a.dx)   f(x+a.dx)-f(x) a.g(x).dx    R(a)'	
	END IF

	IF (id_flushflag .eq. 1) CALL flush(il_io)

    END IF
    ALLOCATE( dla_x0(id_n) )
    dla_x0(:) = dda_x(:)
    ALLOCATE( dla_dx0(id_n) )
    dla_dx0(:) = dda_dx(:)
    dl_f0 = dd_f
    ALLOCATE( dla_g0(id_n) )
    dla_g0(:) = dda_g(:)
    ALLOCATE( dla_results(id_niter,5) )
    id_mode = 2
    dl_alpha = dd_alpha0
    dda_x(:) = dla_x0(:) + dl_alpha * dla_dx0(:)
  ELSE
      dla_results(il_nbcall,1) = dl_alpha
      dla_results(il_nbcall,2) = dd_f
      dla_results(il_nbcall,3) = dd_f - dl_f0
      dla_results(il_nbcall,4) = dl_alpha * DOT_PRODUCT(dla_g0,dla_dx0)
      dla_results(il_nbcall,5) = dla_results(il_nbcall,3) &
                               - dla_results(il_nbcall,4)
      IF (id_iprint .eq. 2) THEN
        WRITE(il_io,101) il_nbcall, dla_results(il_nbcall, :)
	IF (id_flushflag .eq. 1) CALL flush(il_io)
      ENDIF 
      IF (il_nbcall == id_niter) THEN
	  id_mode = 1
	  dda_residu(:,:) = dla_results(:,:)
	  IF (id_iprint .gt. 0) THEN
	      WRITE(il_io,*)
	      WRITE(il_io,*)'----------------------'
	      WRITE(il_io,*)
	      WRITE(il_io,301) id_mode
	      WRITE(il_io,*)'At last iteration:'
	      WRITE(il_io,302) dla_results(il_nbcall,1)
	      WRITE(il_io,303) dla_results(il_nbcall,2)
	      WRITE(il_io,304) dla_results(il_nbcall,3)
	      WRITE(il_io,305) dla_results(il_nbcall,4)
	      WRITE(il_io,306) dla_results(il_nbcall,5)

	      IF (id_flushflag .eq. 1) CALL flush(il_io)

	  ENDIF
      ELSE
	  id_mode = 2
	  dl_alpha = dl_alpha * dd_alphastep
	  dda_x(:) = dla_x0(:) + dl_alpha * dla_dx0(:)
	  
      ENDIF
  ENDIF
999 CONTINUE
  SELECT CASE(id_mode)
  CASE(1)
      
      ! SELECT THE VARIABLE TO PUT AND THE NEXT CALL GETS
      cl_name = 'All'
      CALL pl_put_obj(cl_name, .FALSE., id_alg_index)
      
      cl_name = 'mode'
      CALL pl_put_obj(cl_name, .TRUE., id_alg_index)
      
      cl_name = 'residu'
      CALL pl_put_obj(cl_name, .TRUE., id_alg_index)

      cl_name = 'All'
      CALL pl_get_at_next_step(cl_name, .TRUE., id_alg_index)

      ! CLOSE THE OUTPUT FILE IF NECESSARY AND RE-INITIALIZE FIRSTIME
      IF (id_iprint .gt. 0) THEN
	  CLOSE(il_io)
      ENDIF
      FIRSTIME = .TRUE.

      ! DEALLOCATE SAVE ARRAYS
      DEALLOCATE(dla_dx0)
      DEALLOCATE(dla_x0)
      DEALLOCATE(dla_g0)
      DEALLOCATE(dla_results)
      
  CASE(2,3,4)

      ! SELECT THE VARIABLE TO PUT AND THE NEXT CALL GETS
      cl_name = 'All'
      CALL pl_put_obj(cl_name, .FALSE., id_alg_index)
      cl_name = 'mode'
      CALL pl_put_obj(cl_name, .TRUE., id_alg_index)
      cl_name = 'x'
      CALL pl_put_obj(cl_name, .TRUE., id_alg_index)
      
      cl_name = 'x'
      CALL pl_get_at_next_step(cl_name, .FALSE., id_alg_index)
      cl_name = 'dx'
      CALL pl_get_at_next_step(cl_name, .FALSE., id_alg_index)
      cl_name = 'g'
      CALL pl_get_at_next_step(cl_name, .FALSE., id_alg_index)

      ! RE-INITIALIZE FIRSTIME AND NBCALL
      FIRSTIME = .FALSE.
      il_nbcall = il_nbcall + 1

  CASE(-2)

      ! SELECT THE VARIABLE TO PUT AND THE NEXT CALL GETS
      cl_name = 'All'
      CALL pl_put_obj(cl_name, .FALSE., id_alg_index)
      cl_name = 'mode'
      CALL pl_put_obj(cl_name, .TRUE., id_alg_index)
      
      cl_name = 'All'
      CALL pl_get_at_next_step(cl_name, .TRUE., id_alg_index)

      ! CLOSE THE OUTPUT FILE IF NECESSARY AND RE-INITIALIZE FIRSTIME
      IF (id_iprint .gt. 0) THEN
	  CLOSE(il_io)
      ENDIF
      FIRSTIME = .TRUE.

      ! DEALLOCATE SAVE ARRAYS IF NECESSARY
      IF (ALLOCATED(dla_dx0)) DEALLOCATE(dla_dx0)
      IF (ALLOCATED(dla_x0)) DEALLOCATE(dla_x0)
      IF (ALLOCATED(dla_g0)) DEALLOCATE(dla_g0)
      IF (ALLOCATED(dla_results)) DEALLOCATE(dla_results)


  END SELECT
101    FORMAT(I5,X,E12.5,X,E12.5,X,E12.5,X,E12.5,X,E12.5)
201    FORMAT('   dimension of the problem (n):         ',I12)
202    FORMAT('   alpha:                                ',E12.5)
203    FORMAT('   alpha increasing factor (alpha step): ',E12.5)
204    FORMAT('   number of iterations (niter):         ',I12)
205    FORMAT('   printing level (iprint):              ',I12)
206    FORMAT('   norm of x0     = ',E12.5)
207    FORMAT('   norm of dx     = ',E12.5)
208    FORMAT('   f(x0)          = ',E12.5)
209    FORMAT('   norm of g(x0)  = ',E12.5)
301    FORMAT(' TEST_GRAD: output mode is ',I1)
302    FORMAT('   a              = ',E12.5)
303    FORMAT('   f(x+a.dx)      = ',E12.5)
304    FORMAT('   f(x+a.dx)-f(x) = ',E12.5)
305    FORMAT('   a.g(x).dx      = ',E12.5)
306    FORMAT('   R(a)           = ',E12.5)
END SUBROUTINE PL_DTESTGRAD

 
!
!
!
!
!=========================================================================
!
!
!
!
SUBROUTINE pl_set_P(sd_argument,P)
!
!**** pl_set_P
!
!     Purpose:
!       Make P point on the adress of an algebra argument (described by its 
!       handle in the buffer/mailbuff or by the variable containing the 
!       argument)
!       Called by pl_algebra_trigger
!
!     Interface:
!       call pl_set_P(sd_argument,P)
!       Input : 
!         sd_argument : structure containing either the handle of the argument either its value
!       Ouput :
!         P : CRAY pointer to the argument 
!
!***  Method:
!       If there is a handle, call the dmem_lock subroutine, otherwise make a 
!       loc to make P point on sd_argument%ila_value
!
!     Externals:
!
!       pl_get_at_next_step
!
!     History:
!     --------
!      Version    Programmer      Date         Description
!      ---------------------------------------------------
!      1.0        SB            2001/05/11     Created
!*------------------------------------------------------------------------
!
!** + Declarations
!
!** ++ Include files and modules
!
  USE palmlib  !*I contains some parameters (PL_LNAME)
  USE palmdef  !*I contains some parameters and global variables (cg_msg)
  USE pl_def_err !*I for the treatment of the errors
  USE pl_algebra
!
!** ++ Local variables 
!
  IMPLICIT NONE
  INTEGER :: A(*), il_err
!
!** ++ Dummy variables 
!
!
  pointer (P,A)
  TYPE(argument_t) :: sd_argument
!
!*------------------------------------------------------------------------
!
!
  IF (sd_argument%il_hdle.le.0) THEN

      P = loc(sd_argument%ila_value(1))

  ELSE

      CALL fdmem_lock(sd_argument%il_hdle,P,il_err)

  END IF
!
!
END SUBROUTINE pl_set_P
!
!
!
!
!=========================================================================
!
!
!
!
SUBROUTINE pl_algebra_trigger_init(cd_routine_name, id_alg_index)
!
!**** pl_algebra_trigger_init
!
!     Purpose:
!       Initialize the communication context of an algebra routine \
!       (establish what sent inputs will be received for the first call \
!        of the algebra routine
!        Called by pl_fill_alg_table
!
!     Interface:
!       call pl_algebra_trigger_init(cd_routine_name, id_alg_index)
!       input : 
!egin{itemize} 
!item          cd_routine_name : name of the palm routine envelopping the \
!                                 algebra routine we want to run
!item          id_alg_index : index of the algebra unit in the algebra table
!end{itemize} 
!
!***  Method:
!       Select the name of the palm routine envelopping the algebra routine \
!       and depending on it, set sla_sent_inputs()%ll_communicate and 
!       sla_outputs to .false. or .true. thanks to pl_get_at_next_step and
!       pl_put_obj
!
!     Externals:
!
!       pl_get_at_next_step
!
!     History:
!     --------
!      Version    Programmer      Date         Description
!      ---------------------------------------------------
!      1.0        SB            2000/10/21     Created
!*------------------------------------------------------------------------
!
!** + Declarations
!
!** ++ Include files and modules
!
  USE palmlib  !*I contains some parameters (PL_LNAME)
  USE palmdef  !*I contains some parameters and global variables (cg_msg)
  USE pl_def_err !*I for the treatment of the errors
  USE pl_algebra
!
!** ++ Local variables 
!
  IMPLICIT NONE
!
  !*V cp_sub : a parameter for tracking the subroutine
  CHARACTER(LEN=PL_SLGTH), PARAMETER :: cp_sub = 'pl_init_algebra_trigger' 
  CHARACTER(LEN=PL_LNAME) :: cl_name
!
!** ++ Dummy variables 
!
!
  !*V cd_routine_name : the name of the palm routine envelopping the algebra 
  !*V                   routine we want to call 
  CHARACTER(LEN=PL_LNAME) :: cd_routine_name  
!
  !*V id_alg_index : index of the algebra routine in the algebra table
  INTEGER :: id_alg_index,il_err
!
!*------------------------------------------------------------------------
!
  SELECT CASE (cd_routine_name)
!
  CASE ('PL_SAXPY')
!
      cl_name = 'All'
      CALL pl_get_at_next_step(cl_name,.true., id_alg_index)
!
  CASE ('PL_SDOT')
!
      cl_name = 'All'
      CALL pl_get_at_next_step(cl_name,.true., id_alg_index)
!
  CASE ('PL_DTESTGRAD')
!
      cl_name = 'All'
      CALL pl_get_at_next_step(cl_name,.true., id_alg_index)
!
  CASE default
!
      WRITE(PL_OUT,*) 'unknown algebra routine name ',cd_routine_name
      CALL PALM_ABORT(il_err)
!
  END SELECT
!
END SUBROUTINE pl_algebra_trigger_init 
