      SUBROUTINE INIT()
C
C***
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C  MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C
C
C  Centre National de Recherches Meteorologiques
C
C***
C
C Purpose   : Initialise constant and dimensions from a NetCDF file
C
C Original  : A. Piacentini, CERFACS, 12/2007
C
C**
C
      USE PARADI_MOD
      USE ALLOC_MOD
      USE MOCF_MOD
      USE MRAT_MOD
      USE EXPE_MOD
      USE CSTES_MOD
C      USE UNDCOLMT_MOD
      USE SRC_MOD
      USE OUTNETCDF_MOD
      USE FORM_MOD
      USE FORMT_MOD
      USE palmlib
      USE netcdf
C
      IMPLICIT NONE
C
C PALM VARIABLES DECLARATION
C
      CHARACTER(LEN=PL_LNAME) cl_name, cl_space
      CHARACTER(LEN=8) cl_domname
      INTEGER il_time, il_usrtag, il_err, il_err2
      INTEGER il_day, il_month, il_year, il_hour, il_min, il_sec
C
C NetCDF related variables
C
      REAL*4, ALLOCATABLE :: rla_fl1d(:)
      CHARACTER(LEN=28) :: cl_file, cl_seek
      INTEGER :: il_STATUS, il_ncid, il_ndim, il_nvar
      INTEGER :: il_dimid, il_dimlen  
      INTEGER :: il_varid,il_varndim
      INTEGER :: ib_dim,il_species
C
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: INICDF,IB,IDTMIN,IDT0MIN,NEND,NNDAYS,NNHOURS
      INTEGER :: IB_K,ISYNC,NCYCL,ISRC,II,NSECCY
      INTEGER :: NHCY,NOUT,NDOM,ierror
      INTEGER :: SELAPS,DELAPS,HELAPS,MELAPS
      REAL (KIND=8) :: DAYNUM,DAYNUM0,DAYNUMCTBTO
      CHARACTER(LEN=10) :: CHARAC1,CHARAC2
      LOGICAL :: LASTTIME
      INTEGER, DIMENSION(12) :: nbjour
      INTEGER, DIMENSION(5) :: idate0,idate1,idate00
C
      NPROCS=1
      MYID=0
      MASTER=0
C
      nbjour= (/31,28,31,30,31,30,31,31,30,31,30,31/)
C
      ig_run_mode = DIR_MODEL
C
      ierror = 0
      CALL LOAD_PARADI(ierror)
C
      IF (ierror .ne. 0) THEN
          WRITE(*,*) '!!! Error in load paradi'
          CALL MOCAGE_STOP
      ENDIF
C			
c     ___________________
c    |                   |
c    | 1. READ NAMELIST  |
c    |___________________| 
C
C
      open( unit = 66,
     $      file = 'namelist',
     $      form = 'formatted')
      read (66,*) idate00(1)
      read (66,*) idate00(2)
      read (66,*) idate00(3)
      read (66,*) idate00(4)
      read (66,*) idate1(1)
      read (66,*) idate1(2)
      read (66,*) idate1(3)
      read (66,*) idate1(4)
      read (66,*) NHCY
      read (66,*) NOUT
      read (66,*) NDOM
      read (66,*) NAMDOM1
      read (66,*) NAMDOM2
      read (66,*) NAMDOM3
      read (66,*) NAMDOM4
      read (66,*) CHEMSCHEME
      read (66,*) SOLID
      read (66,*) SURF
      read (66,*) PRMTRSP
      read (66,*) CLOUD
      read (66,*) OBS
      read (66,*) SRC
      read (66,*) TRANSINV
C
      read (66,*) NCDF
      if (NCDF .gt. 0) THEN 
          read (66,*) INICDF
          read (66,*) KTOTC
          READ (66,*) KNLEVELS
          IF (KNLEVELS .gt. 0) THEN
            ALLOCATE (iga_level(KNLEVELS))
            DO ib=1,KNLEVELS
              READ(66,*) iga_level(ib)
            END DO
          ELSE 
            KNLEVELS=nivm1
            ALLOCATE (iga_level(KNLEVELS)) ; iga_level(:) = 0
            DO ib_k = 1, nivm1
              iga_level(ib_k) = ib_k
            END DO
          END IF
          READ (66,*) KNSHORT
          IF (KNSHORT .gt. 0) THEN
            ALLOCATE (SPECIES(KNSHORT))
            DO ib=1,KNSHORT
              READ(66,*) SPECIES(ib)
            END DO
          ELSE
            ALLOCATE (SPECIES(1))
          ENDIF
        ENDIF
C END
      close(66)
C
C     Level sanity check
C
#ifdef NIV_60
      IF ( NIVM1 .ne. 60 ) THEN
        WRITE(PL_OUT,*) 'Model compiled for 60 levels but invoked for ',
#else
      IF ( NIVM1 .ne. 47 ) THEN
        WRITE(PL_OUT,*) 'Model compiled for 47 levels but invoked for ',
#endif
     +        NIVM1
        CALL PALM_Abort(il_err)
      ENDIF
c
c Dates :
c
c  - idate00 : YYYY,MM,DD,HH,MIN (begin)
c  - idate1  : YYYY,MM,DD,HH,MIN (end)
c
c At present, inputs/outputs are only possible
c for full hours (MIN=0)
c
      idate00(5)=0
      idate1(5)=0
      WRITE(*,*) '>> MOCAGE RUN :'
      WRITE (*,*) '>> RUN MODE is NetCDF LOADER for ASSIMILATION'
      WRITE(*,100) '>> ',NDOM,' DOMAINS ARE CONSIDERED'
      IFLAG1=.TRUE.
      IDGRI1=.TRUE.
      WRITE(*,101) ' > DOMAIN 1 IS : ',NAMDOM1
      if (NDOM.GE.2) then
        IFLAG2=.TRUE.
        IDGRI2=.TRUE.
        WRITE(*,101) ' > DOMAIN 2 IS : ',NAMDOM2
      else
        IFLAG2=.FALSE.
        IDGRI2=.FALSE.
      endif
      if (NDOM.GE.3) then
        IFLAG3=.TRUE.
        IDGRI3=.TRUE.
        WRITE(*,101) ' > DOMAIN 3 IS : ',NAMDOM3
      else
        IFLAG3=.FALSE.
        IDGRI3=.FALSE.
      endif
      if (NDOM.GE.4) then
        IFLAG4=.TRUE.
        IDGRI4=.TRUE.
        WRITE(*,101) ' > DOMAIN 4 IS : ',NAMDOM4
      else
        IFLAG4=.FALSE.
        IDGRI4=.FALSE.
      endif
c
 100  FORMAT(A3,I1,A23)
 101  FORMAT(A23)
c
C       MEMORY ALLOCATION
C
C* Needed allocations from CALL ALLOC_MRAT
      ALLOCATE (TRANAM1(NTRA1)) ; TRANAM1(:) = "123456789 12345"
      ALLOCATE (QQQM1(LONM1,LATM1,NIVM1,NTRA1)) ; QQQM1(:,:,:,:) = 0.0
      ALLOCATE (iga_assim_idx(NASSIM1)) ; iga_assim_idx(:) = 0
      ALLOCATE (cga_assim_sp(NASSIM1))
      cga_assim_sp (:) = "123456789 12345"
C* Needed allocations from CALL ALLOC_GRIDS
      ALLOCATE (RLONM1(LONM1)) ; RLONM1(:) = 0.0
      ALLOCATE (RLATM1(LATM1)) ; RLATM1(:) = 0.0
      ALLOCATE (ALM1(0:NIVM1)) ; ALM1(:) = 0.0
      ALLOCATE (BLM1(0:NIVM1)) ; BLM1(:) = 0.0
C* Needed allocations from CALL ALLOC_FORM
      ALLOCATE (PSOLM1A(LONM1,LATM1))	 ; PSOLM1A(:,:) = 0.0
      ALLOCATE (PSOLM1B(LONM1,LATM1))	 ; PSOLM1B(:,:) = 0.0
      ALLOCATE (TGRIM1A(LONM1,LATM1,NIVM1)) ; TGRIM1A(:,:,:) = 0.0
      ALLOCATE (TGRIM1B(LONM1,LATM1,NIVM1)) ; TGRIM1B(:,:,:) = 0.0
C* Needed allocations from CALL ALLOC_FORMT
      ALLOCATE (PSOLM1(LONM1,LATM1))       
      ALLOCATE (XPAM1(LONM1,LATM1,NIVM1))  
      ALLOCATE (XPHFM1(LONM1,LATM1,NIVM1)) 
      ALLOCATE (TGRIM1(LONM1,LATM1,NIVM1)) ; TGRIM1(:,:,:) = 0.0
#ifdef GAUSS
      CALL ALLOC_GAUSS
#endif
c
c Timestep controls :
c
c  - DT : timestep for physics/chemistry (15 min = 900 s)
c  - DTCONV : timestep for parameterized transport (30 min)
c  - NSCY : cycle length for 3D forcings (s)
c  - NITERT : number of iterations of dynamics/physics per cycle
c  - DT0 : timestep for resolved scale transport (s)
c  - NITAP : number of iterations for semi-lag. back-trajectories
c  - NITRA : number of points on the semi-lag. back-trajectories
c  - DTN : timestep for the semi-lag. back-trajectories 
c
      DT=REAL(ip_obs_slot)
c
c one call to convection every 2 timestep 
c
      DTCONV=2.*DT
      NHCY=NCDF
      NSCY=NHCY*3600
      NITERT=NHCY
      DT0=REAL(NSCY)/REAL(NITERT)
      NITAP=2
      NITRA=3
      DTN=DT0/REAL(NITRA)
c
      IDTMIN=int(DT)/60
      IDT0MIN=int(DT0)/60
      write(*,204) '>> TIMESTEP FOR PHYSICS/CHEMISTRY IS : '
     $             ,IDTMIN,' (min)'
      write(*,205) '>> TIMESTEP FOR DYNAMICS IS : '
     $             ,IDT0MIN,' (min)'
c
c Simulation controls :
c
c  - NOUT : frequency of output of HM history files (s)
c  - NCDF : frequency of output of NetCDF files (s) (<0 for no output)
c  - NEND : length of the simulation (s)
c 
      write(*,200) '>> SIMULATION BEGINS ON : ',idate00(1),
     $             idate00(2),idate00(3),idate00(4)
c If the model is working as direct model for assimilation, it can only
c be a forward model and its duration is set by assimilation
      NEND = ip_assimsize
      write(*,*) '>> AND LASTS ',NEND, 'seconds'
      NNDAYS=NEND/86400
      NNHOURS=mod(NEND,86400)/3600
      write(*,201) '>> LENGTH : ',NNDAYS,' (d) AND '
     $            ,NNHOURS,' (h)'
      NOUT=NOUT*3600
C
      write(*,202) '>> LOAD   NetCDF  FILE EVERY ',NCDF,
     $    '(h)                  '
      NCDF=NCDF*3600
C
      LASTTIME=.FALSE.
c
c Synchronize simulation beginning and forcing cycles
c 
c  - idate0 : date of first forcing cycle
c  - NCYCLT : total number of cycles
c
      idate0(1)=idate00(1)
      idate0(2)=idate00(2)
      idate0(3)=idate00(3)
      idate0(4)=idate00(4)
      idate0(5)=idate00(5)
C
      ISYNC=MOD(idate0(4),NHCY)
      IF (ISYNC.NE.0) THEN
        IF (.NOT.TRANSINV) THEN
          idate0(4)=idate0(4)-ISYNC
        ELSE
          ISYNC=NHCY-ISYNC
          idate0(4)=idate0(4)+ISYNC
          IF (idate0(4).GE.24) THEN
            idate0(4)=idate0(4)-24
            idate0(3)=idate0(3)+1
          ENDIF
          IF (mod(idate0(1),4).EQ.0) THEN
            nbjour(2)=29
          ENDIF 
          IF (idate0(3).GT.nbjour(idate0(2))) THEN 
            idate0(3)=idate0(3)-nbjour(idate0(2))
            idate0(2)=idate0(2)+1
          ENDIF 
          IF (idate0(2).GT.12) THEN
            idate0(1)=idate0(1)+1
            idate0(2)=1
          ENDIF 
        ENDIF
      ENDIF
C
      daynum0=ISYNC*3600
      if (ISYNC.NE.0) then
        NCYCLT=MAX(1,NEND/NSCY+1)
      else
        NCYCLT=MAX(1,NEND/NSCY)
      endif
      NCYCL=1
c
      print*,'NCYCLT=',NCYCLT
c
 200  FORMAT(A26,I4.4,I2.2,I2.2,I2.2)
 201  FORMAT(A12,I1.1,A9,I2.2,A4)
 202  FORMAT(A29,I2.2,A21)
 203  FORMAT(A14,I1.1,A5,I2.2,A5,I2.2,A21)
 204  FORMAT(A39,I2.2,A6)
 205  FORMAT(A30,I2.2,A6)
 206  FORMAT(A45,I2.2,A1,I2.2,A4,I2.2,A1,I2.2)
ccccc
c     ________________________________
c    |                                |
c    | 2. INITIALIZE TRANSPORT MODEL  |
c    |________________________________|
c
ccccc
c
      CALL INICSTE
C
C 0. DETERMINE FILE NAME
C
      CALL datencd(daynum0,idate0,NAMDOM1,nbjour,cl_file,idate1)
C
C 0. OPEN FILE
C
      il_status=NF90_OPEN(TRIM(cl_file),NF90_NOWRITE,il_ncid)
      IF (il_STATUS .ne. NF90_NOERR) THEN
          WRITE(*,*) 'Error opening ',trim(cl_file)
          WRITE(PL_OUT,*) 'Error opening ',trim(cl_file)
          CALL PALM_Abort(il_err)
      ENDIF
      WRITE(*,*) '>> OPENING INITIAL NetCDF FILE ',trim(cl_FILE)
      WRITE(PL_OUT,*) '>> OPENING INITIAL NetCDF FILE ',trim(cl_FILE)
C
C* LOAD DIMENSIONS SIZES
      il_status=NF90_INQUIRE(il_ncid,il_ndim,il_nvar)
C
C* LONGITUDE SANITY CHECK
      il_status=NF90_INQ_DIMID(il_ncid,"lon",il_dimid)
      il_status=NF90_INQUIRE_DIMENSION(il_ncid,il_dimid,
     $    len=il_dimlen)
      IF (il_dimlen .NE. ip_lonm1) THEN
          WRITE(*,*) 'ERROR: wrong lon DIMENSION'
          WRITE(*,*) 'In NetCDF  lon = ',il_dimlen
          WRITE(*,*) 'In PrePALM lon = ',ip_lonm1
          WRITE(PL_OUT,*) 'ERROR: wrong lon DIMENSION'
          WRITE(PL_OUT,*) 'In NetCDF  lon = ',il_dimlen
          WRITE(PL_OUT,*) 'In PrePALM lon = ',ip_lonm1
          CALL PALM_Abort(il_ERR)
      END IF
C
C* LATITUDE SANITY CHECK
      il_status=NF90_INQ_DIMID(il_ncid,"lat",il_dimid)
      il_status=NF90_INQUIRE_DIMENSION(il_ncid,il_dimid,
     $    len=il_dimlen)
      IF (il_dimlen .NE. ip_latm1) THEN
          WRITE(*,*) 'ERROR: wrong lat DIMENSION'
          WRITE(*,*) 'In NetCDF  lat = ',il_dimlen
          WRITE(*,*) 'In PrePALM lat = ',ip_latm1
          WRITE(PL_OUT,*) 'ERROR: wrong lat DIMENSION'
          WRITE(PL_OUT,*) 'In NetCDF  lat = ',il_dimlen
          WRITE(PL_OUT,*) 'In PrePALM lat = ',ip_latm1
          CALL PALM_Abort(il_ERR)
      END IF
C
C* LEVELS SANITY CHECK
      il_status=NF90_INQ_DIMID(il_ncid,"lev",il_dimid)
      il_status=NF90_INQUIRE_DIMENSION(il_ncid,il_dimid,
     $    len=il_dimlen)
      IF (il_dimlen .NE. ip_nivm1) THEN
          WRITE(*,*) 'ERROR: wrong lev DIMENSION'
          WRITE(*,*) 'In NetCDF  lev = ',il_dimlen
          WRITE(*,*) 'In PrePALM lev = ',ip_nivm1
          WRITE(PL_OUT,*) 'ERROR: wrong lev DIMENSION'
          WRITE(PL_OUT,*) 'In NetCDF  lev = ',il_dimlen
          WRITE(PL_OUT,*) 'In PrePALM lev = ',ip_nivm1
          CALL PALM_Abort(il_ERR)
      END IF
C
C* LOAD LONGITUDE COORDINATES
      cl_seek="lon"
      il_status=NF90_INQ_VARID(il_ncid,cl_seek,il_varid)
      IF (il_status .eq. NF90_NOERR) THEN
          ALLOCATE (rla_fl1d(ip_lonm1))
          il_status=NF90_GET_VAR(il_ncid,il_varid,rla_fl1d)
          RLONM1(:)=DBLE(rla_fl1d(:))/CONV
          DEALLOCATE (rla_fl1d)
      ELSE
          WRITE(*,*) 'Variable ',TRIM(cl_seek),' not in file ',
     $        TRIM(cl_file)
          WRITE(PL_OUT,*) 'Variable ',TRIM(cl_seek),' not in file ',
     $        TRIM(cl_file)
          CALL PALM_Abort(il_ERR)
      ENDIF
C
C* LOAD LATITUDE COORDINATES
      cl_seek="lat"
      il_status=NF90_INQ_VARID(il_ncid,cl_seek,il_varid)
      IF (il_status .eq. NF90_NOERR) THEN
          ALLOCATE (rla_fl1d(ip_latm1))
          il_status=NF90_GET_VAR(il_ncid,il_varid,rla_fl1d)
          RLATM1(:)=DBLE(rla_fl1d(:))/CONV
          DEALLOCATE (rla_fl1d)
      ELSE
          WRITE(*,*) 'Variable ',TRIM(cl_seek),' not in file ',
     $        TRIM(cl_file)
          WRITE(PL_OUT,*) 'Variable ',TRIM(cl_seek),' not in file ',
     $        TRIM(cl_file)
          CALL PALM_Abort(il_ERR)
      ENDIF
C
C* LOAD ALPHA HYBRID SIGMA COEFFICIENTS
      cl_seek="a_hybr_coord"
      il_status=NF90_INQ_VARID(il_ncid,cl_seek,il_varid)
      IF (il_status .eq. NF90_NOERR) THEN
          ALLOCATE (rla_fl1d(ip_nivm1))
          il_status=NF90_GET_VAR(il_ncid,il_varid,rla_fl1d)
          ALM1(:)=0.
          ALM1(1:ip_nivm1)=DBLE(rla_fl1d(:))
          DEALLOCATE (rla_fl1d)
      ELSE
          WRITE(*,*) 'Variable ',TRIM(cl_seek),' not in file ',
     $        TRIM(cl_file)
          WRITE(PL_OUT,*) 'Variable ',TRIM(cl_seek),' not in file ',
     $        TRIM(cl_file)
          CALL PALM_Abort(il_ERR)
      ENDIF
C
C* LOAD BETA HYBRID SIGMA COEFFICIENTS
      cl_seek="b_hybr_coord"
      il_status=NF90_INQ_VARID(il_ncid,cl_seek,il_varid)
      IF (il_status .eq. NF90_NOERR) THEN
          ALLOCATE (rla_fl1d(ip_nivm1))
          il_status=NF90_GET_VAR(il_ncid,il_varid,rla_fl1d)
          BLM1(:)=0.
          BLM1(1:ip_nivm1)=DBLE(rla_fl1d(:))
          DEALLOCATE (rla_fl1d)
      ELSE
          WRITE(*,*) 'Variable ',TRIM(cl_seek),' not in file ',
     $        TRIM(cl_file)
          WRITE(PL_OUT,*) 'Variable ',TRIM(cl_seek),' not in file ',
     $        TRIM(cl_file)
          CALL PALM_Abort(il_ERR)
      ENDIF
C
      il_status=NF90_CLOSE(il_ncid)
c
ccccc
c     ___________________________________________________________________
c    |                                                                   |
c    | 3. INITIALIZE CHEMISTRY, TRACER 3D DISTRIBUTIONS & TOP BOUNDARIES |
c    |___________________________________________________________________|
c
ccccc
c
C** Automatic renaming of the ozone variable following the scheme
C** and the number of levels
C
#ifdef NIV_60
#ifdef CARIOLLE
#define CPP_O3 'O_3'
#else
#define CPP_O3 'O_x'
#endif
#else
#define CPP_O3 'O_x'
#endif
C
C** Read the list of the assimilates species in an auxiliary file
C
      open( unit = 66,
     $    file = 'assimilated_species',
     $    form = 'formatted')
      DO ib = 1, NASSIM1
        iga_assim_idx(ib)=0
        READ(66,*) cga_assim_sp(ib)
        IF (TRIM(cga_assim_sp(ib)) .eq. 'O_3' .and.
     $      CPP_O3 .ne. 'O_3') THEN
            cga_assim_sp(ib)=CPP_O3
            WRITE(*,*) '+ WARNING CHANGING O_3 TO ',CPP_O3
            WRITE(PL_OUT,*) '+ WARNING CHANGING O_3 TO ',CPP_O3
        ENDIF
        IF (TRIM(cga_assim_sp(ib)) .eq. 'O_x' .and.
     $      CPP_O3 .ne. 'O_x') THEN
            cga_assim_sp(ib)=CPP_O3
            WRITE(*,*) '+ WARNING CHANGING O_x TO ',CPP_O3
            WRITE(PL_OUT,*) '+ WARNING CHANGING O_x TO ',CPP_O3
        ENDIF            
        IF (TRIM(cga_assim_sp(ib)) .eq. 'O_X') THEN
            cga_assim_sp(ib)=CPP_O3
            WRITE(*,*) '+ WARNING CHANGING O_X TO ',CPP_O3
            WRITE(PL_OUT,*) '+ WARNING CHANGING O_X TO ',CPP_O3
        ENDIF            
        TRANAM1(ib) = TRIM(cga_assim_sp(ib))
      END DO
      CLOSE(66) 
C
C
C Final puts of the INIT unit
#include "palm_iniputs.h"
C
      SUBROUTINE MODEL() 
c
      USE palmlib
C
      USE PARADI_MOD
      USE ALLOC_MOD
      USE MOCF_MOD
      USE MRAT_MOD
      USE EXPE_MOD
      USE CSTES_MOD
C      USE UNDCOLMT_MOD
      USE SRC_MOD
      USE OUTNETCDF_MOD
      USE FORM_MOD
      USE FORMT_MOD
      USE netcdf
C
      IMPLICIT NONE
C
      INTEGER idate0(5), idate00(5), idate1(5)
      REAL daynum
C
C
C       Local declaration
C
      LOGICAL DIAG, LOUT, ll_nooverlap, ll_FIRSTTIME, ll_LASTTIME
      DATA ll_FIRSTTIME/.FALSE./
      SAVE ll_FIRSTTIME
C
      INTEGER NCYCL
      DATA NCYCL/1/
      SAVE NCYCL
C    
      INTEGER NOUT
      INTEGER INICDF
      INTEGER iterassim
      INTEGER ib, ilon, ilat, iniv
      INTEGER :: SELAPS,DELAPS,HELAPS,MELAPS
      INTEGER NSECCY
      INTEGER assimtime
      INTEGER pastassimtime
      LOGICAL ld_out
      INTEGER, DIMENSION(12) :: nbjour
C
      CHARACTER(LEN=28) :: cl_FILE      
C
C     MOCAGE
C     
      REAL (KIND=8) :: DAYNUM0,CINTA,CINTB
      REAL (KIND=8) :: XPAPLUS,XPAMOIN
C
C       PALM
C
      CHARACTER(LEN=PL_LNAME) cl_name, cl_space
      INTEGER il_time, il_usrtag, il_err, il_err2
      INTEGER il_day, il_month, il_year, il_hour, il_min, il_sec
      INTEGER il_tstart, il_tend, il_putstart, il_putend
C
      LOGICAL entered
      SAVE entered
      DATA entered / .TRUE. /
C
      REAL (KIND=8),SAVE :: QQQM1A(ip_lonm1,ip_latm1,ip_nivm1,1)
      REAL (KIND=8),SAVE :: QQQM1B(ip_lonm1,ip_latm1,ip_nivm1,1)
C
C       Ajout de l'increment
C      
      REAL inscoef
      REAL,SAVE :: dQQQM1CUM(ip_lonm1,ip_latm1,ip_nivm1,ip_nassim)=0.
      REAL :: dQQQM1a(ip_lonm1,ip_latm1,ip_nivm1,ip_nassim)
C
C       Variance d'erreur d'ebauche
C
      REAL, DIMENSION(:,:,:,:), ALLOCATABLE :: rda_sigmab
      INTEGER :: il_sigbck
      INTEGER :: ibi,ibj,ibk,ibl
      REAL :: rl_norminc, rl_norminca
C
      NPROCS=1
      MYID=0
      MASTER = 0
C
      nbjour= (/31,28,31,30,31,30,31,31,30,31,30,31/)
C
ccccc
c     _______________________________________
c    |                                       |
c    | 1. PALM GET INITIALIZED PARAMETERS    |
c    |_______________________________________|
c
ccccc
c
c     1.1. PALM_Get times and date
c
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_tstart'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_tstart,il_err)
c
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_tend'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_tend,il_err)
c
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_putstart'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $    il_putstart,il_err)
c
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_putend'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_putend,il_err)
c
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'id_iter'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,iterassim,
     $             il_err)
c
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'id_assimtime'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,assimtime,il_err)
c
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'id_pastassimtime'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     +             pastassimtime,il_err)
c
      cl_space = 'one_real'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'rd_daynum0'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,daynum0,il_err)
C
      cl_space = 'date'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'ida_date0'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,idate0,il_err)
c
      cl_space = 'date'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'ida_date00'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,idate00,il_err)
c
c     1.2. PALM_Get logicals
c
      cl_space = 'one_logical'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'ld_out'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,LOUT,il_err)
c
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_NCDF'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,NCDF,il_err)
C
C*    Activate output for the first time step
c
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'il_NOUT'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,NOUT,il_err)
C
C* LOAD NetCDF for the initial time
C
      PSOLM1A(:,:)=0.
      PSOLM1B(:,:)=0.
      QQQM1A(:,:,:,:) = 0.
      QQQM1B(:,:,:,:) = 0.
      TGRIM1A(:,:,:) = 0.
      TGRIM1B(:,:,:) = 0.
      CALL datencd(daynum0,idate0,NAMDOM1,
     $    nbjour,cl_file,idate1)
      CALL loadncd(cl_file,TRANAM1(1),PSOLM1B,QQQM1B,TGRIM1B)
C
C
C** MAIN LOOP
C
C
      SELAPS = int (REAL(il_tstart) - daynum0)
      daynum = REAL(il_tstart) - DT
C
      DO WHILE ((SELAPS+DT) .le. il_tend)
        daynum = daynum+DT
        SELAPS=int(daynum-daynum0)
        DELAPS=SELAPS/86400
        HELAPS=mod(SELAPS,86400)/3600
        MELAPS=mod(SELAPS,3600)/60
        WRITE(PL_OUT,'(A14,I1.1,A5,I2.2,A5,I2.2,A21)')
     $      '>> TIMESTEP : ',
     $      DELAPS,' (d) ',
     $      HELAPS,' (h) ',
     $      MELAPS,' (min) FROM BEGINNING'
        CALL flush(PL_OUT)
        WRITE(*,'(A14,I1.1,A5,I2.2,A5,I2.2,A21)') 
     $      '>> TIMESTEP : ',
     $      DELAPS,' (d) ',
     $      HELAPS,' (h) ',
     $      MELAPS,' (min) FROM BEGINNING'
C
        dQQQM1a = 0.
C
C* Compute interpolation coefficients between NetCDF times
C
        NSECCY=mod(int(daynum),NSCY)
        NITER=NSECCY/(NSCY/NITERT)
        CINTB=REAL(NITER)/REAL(NITERT)
        CINTA=1.-CINTB
C
        IF (NSECCY.EQ.0.AND. int(SELAPS).ge.il_tend) THEN
            CINTA=0.
            CINTB=1.
        END IF
C
C* Update NetCDF fields: transfer B to A and load new B
C
        IF (NSECCY.EQ.0 .AND. int(SELAPS).lt.il_tend) THEN
            PSOLM1A = PSOLM1B
            QQQM1A = QQQM1B
            TGRIM1A = TGRIM1B
C
            CALL datencd(daynum+REAL(NSCY),idate0,NAMDOM1,
     $          nbjour,cl_file,idate1)
            CALL loadncd(cl_file,TRANAM1(1),PSOLM1B,QQQM1B,TGRIM1B)
C
        ENDIF
C
        IF (int(daynum).EQ.pastassimtime) THEN
C
C** Reception of the assimilation increment and its application as
C** model state correction
C
            cl_space = 'controlled_species'
            il_time = int(daynum)
            il_usrtag = PL_NO_TAG
            cl_name = 'rda_dxa'
            CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     +          dQQQM1a,il_err)
C
C* IF an analysis increment has been produced, update cumulative incr.
C
            IF (il_ERR.eq.0) THEN
                WRITE(6,*) ' > Updating increment at ',il_time
                WRITE(PL_OUT,*) ' > Updating increment at ',il_time
                rl_norminc=0.
                DO ibi=1,ip_lonm1
                  DO ibj=1,ip_latm1
                    DO ibk=1,ip_nivm1
                      DO ibl=1,ip_nassim
                        rl_norminc=rl_norminc+
     $                      dQQQM1CUM(ibi,ibj,ibk,ibl)*
     $                      dQQQM1CUM(ibi,ibj,ibk,ibl)
                      END DO
                    END DO
                  END DO
                END DO
C
                dQQQM1CUM = dQQQM1CUM + dQQQM1a
C
                rl_norminc=0.
                rl_norminca=0.
                DO ibi=1,ip_lonm1
                  DO ibj=1,ip_latm1
                    DO ibk=1,ip_nivm1
                      DO ibl=1,ip_nassim
                        rl_norminc=rl_norminc+
     $                      dQQQM1CUM(ibi,ibj,ibk,ibl)*
     $                      dQQQM1CUM(ibi,ibj,ibk,ibl)
                        rl_norminca=rl_norminca+
     $                      dQQQM1a(ibi,ibj,ibk,ibl)*
     $                      dQQQM1a(ibi,ibj,ibk,ibl)
                      END DO
                    END DO
                  END DO
                END DO
C
            END IF
C
        ENDIF
C
C** Time interpolation and analysis update
C
        PSOLM1 = CINTA*PSOLM1A + CINTB*PSOLM1B
        QQQM1 = (CINTA*QQQM1A + CINTB*QQQM1B)+dQQQM1CUM
        DO ibl=1,ip_nassim
          DO ibk=1,ip_nivm1
            DO ibj=1,ip_latm1
              DO ibi=1,ip_lonm1
                IF (QQQM1(ibi,ibj,ibk,ibl).lt.
     $              REAL(ip_enforced_ana_min)*1.e-9) THEN
                    QQQM1(ibi,ibj,ibk,ibl)=
     $                  REAL(ip_enforced_ana_min)*1.e-9
                    IF (ip_iprint.gt.0)
     $                  WRITE(PL_OUT,*) '+ Correcting value',
     $                  ' to enforced minimum for point ',
     $                  ibi, ibi, ibk
                END IF
              END DO
            END DO
          END DO
        END DO
        TGRIM1 = (CINTA*TGRIM1A + CINTB*TGRIM1B)
C
        IF (myid.eq.master .and. mod(SELAPS,NCDF).eq.0.) THEN
C
#ifdef NIV_60
            do ilon=1,ip_lonm1
              DO ilat=1,ip_latm1
                xphfm1(ilon,ilat,1)=ALM1(1)
     $              +BLM1(1)*psolm1(ilon,ilat)
                xpam1(ilon,ilat,1)=0.5*xphfm1(ilon,ilat,1)
              END DO
            enddo
            
            do iniv=2,ip_nivm1
              do ilon=1,ip_lonm1
                DO ilat=1,ip_latm1
                  XPAPLUS=ALM1(iniv)+BLM1(iniv)*
     $                psolm1(ilon,ilat)
                  xphfm1(ilon,ilat,iniv)=XPAPLUS
                  XPAMOIN=ALM1(iniv-1)
     $                +BLM1(iniv-1)*psolm1(ilon,ilat)
                  xpam1(ilon,ilat,iniv)=0.5*(XPAPLUS+XPAMOIN)
                END DO
              enddo
            enddo
#else
            do ilon=1,ip_lonm1
              DO ilat=1,ip_latm1
                xpam1(ilon,ilat,1)=
     $              EXP(ALOG(ALM1(1)+BLM1(1)*
     $              psolm1(ilon,ilat))-1.)
                xphfm1(ilon,ilat,1)=ALM1(1)
     $              +BLM1(1)*psolm1(ilon,ilat)
              END DO
            enddo
c
            do iniv=2,ip_nivm1
              do ilon=1,ip_lonm1
                DO ilat=1,ip_latm1
                  XPAPLUS=ALM1(iniv)+BLM1(iniv)*
     $                psolm1(ilon,ilat)
                  xphfm1(ilon,ilat,iniv)=XPAPLUS
                  XPAMOIN=ALM1(iniv-1)+BLM1(iniv-1)
     $                *psolm1(ilon,ilat)
                  xpam1(ilon,ilat,iniv)=EXP(
     $                (XPAPLUS*ALOG(XPAPLUS)-XPAMOIN*ALOG(XPAMOIN))
     $                /(XPAPLUS-XPAMOIN) - 1. )
                END DO 
              enddo
            enddo
c
#endif
C
            call OUTNETCDF_light(daynum,idate0,idate00,iterassim)
        END IF

        
C
        IF (myid.eq.master .and. int(daynum).EQ.assimtime) THEN
C
C** Send the background error variance estimate
C
            ALLOCATE( rda_sigmab(ip_lonm1,ip_latm1,ip_nivm1,ip_nassim))
C     
            cl_space = 'one_integer'
            il_time = assimtime 
            il_usrtag = PL_NO_TAG
            cl_name = 'il_sigmab'
            CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     +          il_sigbck,il_err)
C        
            WRITE(*,'(A2)') '++'
            WRITE(*,'(A24,A14)') '++ COMPUTING BACKGROUND ',
     $          'ERROR VARIANCE'
            WRITE(*,'(A26,I3,A1)') 
     $          ' + USING A % STD. ERR. OF ', il_sigbck,'%'
            WRITE(*,'(A2)') '++'
            WRITE(PL_OUT,'(A24,A14)') '++ COMPUTING BACKGROUND ',
     $          'ERROR VARIANCE'
            WRITE(PL_OUT,'(A26,I3,A1)') 
     $          ' + USING A % STD. ERR. OF ', il_sigbck,'%'
C
            DO ib = 1, ip_nassim
              rda_sigmab(:,:,:,ib) =
     $            QQQM1(:,:,:,iga_assim_idx(ib)) * REAL(il_sigbck)/100.
            END DO
C     
            cl_space = 'controlled_species'
            il_time = assimtime
            il_usrtag = PL_NO_TAG
            cl_name = 'rda_sigmab'
            CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,
     +          rda_sigmab,il_err) 
C
            DEALLOCATE( rda_sigmab )
C
        ENDIF
C
c
ccccc
c     _______________________________________________
c    |                                              |
c    | 8. PALM PUT CHEMISTRY SPECIES OVER THE GRID  |
c    |______________________________________________|
c
ccccc
C      
        IF (int(daynum) .ge. il_putstart .and.
     $      int(daynum) .le. il_putend) THEN
            
            IF (mod(int(daynum),int(DT0)).EQ.0.and.myid.eq.master) THEN
C
C* Use the allocatable rda_sigmab array as a send buffer
C
                ALLOCATE(rda_sigmab(ip_lonm1,ip_latm1,ip_nivm1,
     $              ip_nassim))
C        
                DO ib = 1, ip_nassim
                  rda_sigmab(:,:,:,ib) =
     $                QQQM1(:,:,:,iga_assim_idx(ib))
                END DO
                
C
                IF (ip_iprint .EQ. 1) THEN
                    WRITE(*,*) 'Sending model state for ',int(daynum)
                    WRITE(PL_OUT,*) 'Sending model state for ',
     $                  int(daynum)
                    CALL FLUSH(PL_OUT)
                END IF
                cl_space = 'controlled_species'
                il_time = int(daynum)
                il_usrtag = PL_NO_TAG
                cl_name = 'rda_x'
                CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,
     +              rda_sigmab,il_err)
C
                DEALLOCATE (rda_sigmab)
C
                cl_space = 'ground'
                il_time = int(daynum)
                il_usrtag = PL_NO_TAG
                cl_name = 'rda_psolm1'
                CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,
     $              PSOLM1,il_err)
C      
            ENDIF 
C
            IF (ip_fenetre .EQ. ip_correctime .AND. 
     +          int(daynum) .EQ. (assimtime - ip_obs_slot).and.
     +          myid.eq.master) THEN
C
C** Send the background error variance estimate
C
                ALLOCATE(rda_sigmab(ip_lonm1,ip_latm1,ip_nivm1,
     $              ip_nassim))
C     
                cl_space = 'one_integer'
                il_time = assimtime 
                il_usrtag = PL_NO_TAG
                cl_name = 'il_sigmab'
                CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     +              il_sigbck,il_err)
C        
                DO ib = 1, ip_nassim
                  rda_sigmab(:,:,:,ib) =
     $                QQQM1(:,:,:,iga_assim_idx(ib))
     $                * REAL(il_sigbck)/100.
                END DO
C     
                cl_space = 'controlled_species'
                il_time = assimtime 
                il_usrtag = PL_NO_TAG
                cl_name = 'rda_sigmab'
                CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,
     +              rda_sigmab,il_err)
C
                DEALLOCATE( rda_sigmab )
            ENDIF
C
        ENDIF        
C
C* END of main time loop
      END DO      
C
      END
C
      SUBROUTINE datencd(daynum,idate0,NAMDOM,nbjour,clf,idate1)
C
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
      REAL (KIND=8) :: daynum
      INTEGER , DIMENSION(5) :: idate0,idate1
      CHARACTER (LEN=6) :: NAMDOM
      INTEGER , DIMENSION(12) :: nbjour
      CHARACTER (LEN=22) :: clf
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~      
      INTEGER :: NBDAY,NBHOUR
C
      NBDAY=int((daynum)/86400.)
      NBHOUR=int((daynum-real(NBDAY)*86400.)/3600.)
C
      idate1(5)=0
      idate1(4)=idate0(4)+NBHOUR
      idate1(3)=idate0(3)+NBDAY
      idate1(2)=idate0(2)
      idate1(1)=idate0(1)
C
      IF (idate1(4).GE.24) THEN 
          idate1(4)=idate1(4)-24
          idate1(3)=idate1(3)+1
      ENDIF 
C
      IF (mod(idate0(1),4).EQ.0) THEN
          nbjour(2)=29          ! 29 days in february
      ENDIF 
C
      IF (idate1(3).GT.nbjour(idate1(2))) THEN 
          idate1(3)=idate1(3)-nbjour(idate1(2))
          idate1(2)=idate1(2)+1
      ENDIF 
C
      IF (idate1(2).GT.12) THEN 
          idate1(1)=idate1(1)+1
          idate1(2)=1
      ENDIF 
C
      WRITE(clf,fmt='(''HM'',A6,''+'',I4.4,I2.2,I2.2,I2.2,''.nc'')')
     $        NAMDOM,idate1(1),idate1(2),idate1(3),idate1(4)
C
      RETURN
      END
C
      SUBROUTINE loadncd(cd_file,cd_seek,rda_psolm,rda_species,rda_temp)
C
      USE palmlib
      USE paradi_mod
      USE netcdf
C
      IMPLICIT NONE
C
      CHARACTER(LEN=28) :: cd_file
      CHARACTER(LEN=15) :: cd_seek
      REAL(KIND=8) :: rda_psolm(ip_lonm1,ip_latm1)
      REAL(KIND=8) :: rda_species(ip_lonm1,ip_latm1,ip_nivm1,1)
      REAL(KIND=8) :: rda_temp(ip_lonm1,ip_latm1,ip_nivm1)
C
      CHARACTER(LEN=28) :: cl_seek
      INTEGER :: il_status, il_ncid,il_varid, il_ERR
      REAL*4 :: rla_psolm(ip_lonm1,ip_latm1)
      REAL*4 :: rla_species(ip_lonm1,ip_latm1,ip_nivm1)
      REAL*4 :: rla_temp(ip_lonm1,ip_latm1,ip_nivm1)
C
      il_status=NF90_OPEN(TRIM(cd_file),NF90_NOWRITE,il_ncid)
      IF (il_STATUS .ne. NF90_NOERR) THEN
          WRITE(*,*) 'Error opening ',trim(cd_file)
          WRITE(PL_OUT,*) 'Error opening ',trim(cd_file)
          CALL PALM_Abort(il_err)
      ENDIF
      WRITE(*,*) '>> OPENING NetCDF FILE ',trim(cd_file)
      WRITE(PL_OUT,*) '>> OPENING NetCDF FILE ',trim(cd_file)
C
      cl_seek="air_pressure_at_surface"
      il_status=NF90_INQ_VARID(il_ncid,trim(cl_seek),il_varid)
      IF (il_status .eq. NF90_NOERR) THEN
          il_status=NF90_GET_VAR(il_ncid,il_varid,rla_psolm)
          rda_psolm(:,:)=DBLE(rla_psolm(:,:))
      ELSE
          WRITE(*,*) 'Variable ',TRIM(cl_seek),
     $        ' not in FILE ',trim(cd_file)
          WRITE(PL_OUT,*) 'Variable ',TRIM(cl_seek),
     $        ' not in file ',trim(cd_file)
          CALL PALM_Abort(il_err)
      ENDIF
C
      cl_seek="air_temperature"
      il_status=NF90_INQ_VARID(il_ncid,trim(cl_seek),il_varid)
      IF (il_status .eq. NF90_NOERR) THEN
          il_status=NF90_GET_VAR(il_ncid,il_varid,rla_temp)
          rda_temp(:,:,:)=DBLE(rla_temp(:,:,:))
      ELSE
          WRITE(*,*) 'Variable ',TRIM(cl_seek),
     $        ' not in FILE ',trim(cd_file)
          WRITE(PL_OUT,*) 'Variable ',TRIM(cl_seek),
     $        ' not in file ',trim(cd_file)
          CALL PALM_Abort(il_err)
      ENDIF
C
      il_status=NF90_INQ_VARID(il_ncid,cd_seek,il_varid)
      IF (il_status .eq. NF90_NOERR) THEN
          il_status=NF90_GET_VAR(il_ncid,il_varid,rla_species)
          rda_species(:,:,:,1)=DBLE(rla_species(:,:,:))
      ELSE
          WRITE(*,*) 'Variable ',TRIM(cd_seek),
     $        ' not in file ',trim(cd_file)
          WRITE(PL_OUT,*) 'Variable ',TRIM(cd_seek),
     $        ' not in file ',trim(cd_file)
          CALL PALM_Abort(il_err)
      ENDIF
C
      il_status=NF90_CLOSE(il_ncid)
C
      RETURN
C      
      END
C
      subroutine outnetcdf_light( daynum, idate0 ,idate00, id_iter)
C
CCCCC
C
C
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 09/06/2005
C
C   Type      : Service
C   Purpose   : Write NetCDF output files
C   Called by : 
C
CCC
C
C  Original : A. Piacentini, 04/2005
C
C  Modified :
C
CCCCC
C
	USE PARADI_MOD
	USE EXPE_MOD
	USE GRIDS_MOD
	USE MOCF_MOD
	USE MRAT_MOD
	USE FORMT_MOD
	USE PHOTO_MOD
	USE OUTNETCDF_MOD
	USE SVPHOT_MOD
        USE CSTES_MOD
C
	IMPLICIT NONE
C
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~

	INTEGER, intent(in) :: id_iter
	REAL (KIND=8) :: daynum
	INTEGER , DIMENSION(5) :: idate0,idate00
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C		
C
#ifdef NIV_60
#define CPP_ESP 'O_3'
#else
#define CPP_ESP 'O_x'
#endif
C
      INCLUDE 'netcdf.inc'
C
      INTEGER :: IL_FD,IL_NBNIV,IL_NC_AHYB,IL_NC_BHYB
      INTEGER :: IL_NC_TEMP,IL_NC_XWIND,IL_NC_YWIND,IL_NC_WWIND
      INTEGER :: NBDAY,NBHOUR
      INTEGER :: ityptr,itrunc,idgl,idlon,iflev
      INTEGER :: ib_i, ib_j, ib_k, ib, ib_dom
      INTEGER :: il_dom, il_lat, il_lon
      LOGICAL :: llgard
      CHARACTER(len=4) :: cdpref
      CHARACTER(len=26) :: cln1
      CHARACTER(len=6) :: cl_name
      INTEGER , ALLOCATABLE :: IDATE(:),idate1(:),nbjour(:)
C
C  NETCDF DECLARATION
C    
      INTEGER :: il_ncfile, il_dlon, il_dlat, il_dniv, il_diter
      INTEGER :: il_flon, il_flat, il_fniv, il_fiter
      INTEGER :: il_done, il_fone, il_fdob
      INTEGER :: ila_ncdim(1)
      INTEGER :: ila_ncdims(4), ila_ncstart(4), ila_nccount(4)
      INTEGER :: il_nc_psol
      INTEGER, DIMENSION(:), ALLOCATABLE :: ila_nc_field
      INTEGER, DIMENSION(:), ALLOCATABLE :: ila_nc_ctot
      INTEGER, DIMENSION(:), ALLOCATABLE :: ila_field_pos
      INTEGER, DIMENSION(:), ALLOCATABLE :: ila_field_kd
      INTEGER :: il_outsp
      INTEGER :: il_err
      REAL (KIND=8), DIMENSION(:,:,:), ALLOCATABLE :: rla_coltot
      REAL (KIND=4), DIMENSION(:,:), ALLOCATABLE ::rla_coltot2d
      REAL (KIND=4), DIMENSION(:), ALLOCATABLE :: rla_dim_lo
      REAL (KIND=4), DIMENSION(:), ALLOCATABLE :: rla_dim_la
      REAL (KIND=4), DIMENSION(nivm1) :: rla_dim_ni
      REAL (KIND=4), DIMENSION(:,:,:), ALLOCATABLE :: rla_field
      REAL (KIND=4), DIMENSION(:,:), ALLOCATABLE :: rla_psolml
      REAL (KIND=4), DIMENSION(0:nivm1) :: rla_alm1,rla_blm1
      REAL (KIND=4) :: rl_daynum
      CHARACTER(LEN=26), DIMENSION(:), ALLOCATABLE :: cla_fieldname
      CHARACTER(LEN=48) :: cl_fld_stdname
      CHARACTER(LEN=26) :: cl_fieldunit
      CHARACTER(LEN=48) :: cldat, cltori, cl_source
C
	ALLOCATE (idate(5)) ; idate(:) = 0
	ALLOCATE (idate1(5)) ; idate1(:) = 0
	ALLOCATE (NBJOUR(12))
C
	nbjour =(/31,28,31,30,31,30,31,31,30,31,30,31/)

C
C 1. CHOOSE TIME UNIT ACCORDING TO INITIAL FILE(S)
C
      if (knshort .le. 0) THEN
          knshort = 1
C          species(1) = CPP_ESP
          species(1) = TRANAM1(1)
      end if
C
      ALLOCATE (ila_field_pos(knshort)) ; ila_field_pos(:) = 0
      ALLOCATE (ila_field_kd(knshort)) ; ila_field_kd(:) = 0
      ALLOCATE (cla_fieldname(knshort))
      cla_fieldname(:) = "123456789 123456789 123456"
C
      il_outsp = 0
      DO ib = 1,knshort
        il_fd = 0
        DO ib_i = 1, ntra1
          IF (species(ib) .eq. TRANAM1(ib_i)) THEN
              il_outsp = il_outsp+1
              ila_field_kd(il_outsp) = 1
              ila_field_pos(il_outsp) = ib_i
              cla_fieldname(il_outsp) = species(ib)
              il_fd = 1
              go to 181
          END IF
        END DO
        DO ib_i = 1, nsls1
          IF (species(ib) .eq. SLSNAM1(ib_i)) THEN
              il_outsp = il_outsp+1
              ila_field_kd(il_outsp) = 2
              ila_field_pos(il_outsp) = ib_i
              cla_fieldname(il_outsp) = species(ib)
              il_fd = 1
              go to 181
          END IF
        END DO
 181    CONTINUE
        IF (il_fd .eq. 0) THEN
            WRITE(6,*) 'Component ',trim(species(ib)),
     $          ' not existent in this scheme'
        END IF
      END DO
C
      write(cldat,fmt=
     $    '(''seconds since '',
     $    I4.4,"-",I2.2,"-",I2.2," ",I2.2,":",I2.2,":00")')
     $      idate00(1),idate00(2),idate00(3),idate00(4),idate00(5)
      write(cltori,fmt=
     $    '(I4.4,"-",I2.2,"-",I2.2," ",I2.2,":",I2.2,":00")')
     $      idate00(1),idate00(2),idate00(3),idate00(4),idate00(5)
C
C 1.1 DETERMINE OUTPUT DATE
C
      NBDAY=int(daynum/86400.)
      NBHOUR=int((daynum-real(NBDAY)*86400.)/3600.)
C
      idate1(5)=0
      idate1(4)=idate0(4)+NBHOUR
      idate1(3)=idate0(3)+NBDAY
      idate1(2)=idate0(2)
      idate1(1)=idate0(1)
      IF (idate1(4).GE.24) THEN
          idate1(4)=idate1(4)-24
          idate1(3)=idate1(3)+1
      ENDIF
      IF (mod(idate0(1),4).EQ.0) THEN
          nbjour(2)=29
      ENDIF
      IF (idate1(3).GT.nbjour(idate1(2))) THEN
          idate1(3)=idate1(3)-nbjour(idate1(2))
          idate1(2)=idate1(2)+1
      ENDIF
      IF (idate1(2).GT.12) THEN
          idate1(1)=idate1(1)+1
          idate1(2)=1
      ENDIF
      IF (id_iter.gt.0 .and. idate1(4).eq.0
     $    .and. int(daynum).gt.0) THEN
          idate1(3)=idate1(3)-1
          idate1(4)=24
          IF (idate1(3).eq.0) THEN
              idate1(2) = idate1(2)-1
              IF (idate1(2) .eq. 0) THEN
                  idate1(2) = 12
                  idate1(1) = idate1(1)-1
              ENDIF
              idate1(3)=nbjour(idate1(2))
          ENDIF
      ENDIF
C
C SPAN THE DOMAINS
C
C
      il_lon = lonm1
      ALLOCATE (rla_dim_lo(lonm1))
      DO ib_i = 1, lonm1
        rla_dim_lo(ib_i) = REAL(rlonm1(ib_i)*180./3.141592653589793)
      END DO
C
      il_lat = latm1
      ALLOCATE (rla_dim_la(latm1))
      DO ib_j = 1, latm1
        rla_dim_la(ib_j) = REAL(rlatm1(ib_j)*180./3.141592653589793)
      END DO
C
      cl_name = NAMDOM1
C
      ALLOCATE (rla_field(il_lon,il_lat,nivm1)) ; rla_field(:,:,:) = 0.0
      ALLOCATE (rla_psolml(il_lon,il_lat)) ; rla_psolml(:,:) = 0.0
C
      DO ib_k = 1,KNLEVELS
        rla_dim_ni(ib_k) = REAL(iga_level(ib_k))
      END DO
C
CCC WRITE HISTORY IN NETCDF FILE
C
      IF (id_iter .ge. 0) THEN
       WRITE(cln1,
     + fmt='(''HM'',A6,''+'',I4.4,I2.2,I2.2,I2.2,''+I'',I2.2,''.nc'')')
     + cl_name,idate1(1),idate1(2),idate1(3),idate1(4),id_iter
      ELSE
       WRITE(cln1,
     + fmt='(''HM'',A6,''+'',I4.4,I2.2,I2.2,I2.2,''.nc'')')
     + cl_name,idate1(1),idate1(2),idate1(3),idate1(4)
      END IF
C
C** OPEN NETCDF FILE
C
      il_err = NF_CREATE(cln1, NF_CLOBBER, il_ncfile)
      PRINT *, '>> Writing NetCDF output file ',TRIM(cln1)
      PRINT *, '   On processor ',MYID
C
C** DEFINE DIMENSIONS
C
      il_nbniv = KNLEVELS
      il_err = NF_DEF_DIM(il_ncfile, 'time', NF_UNLIMITED, il_diter)
      il_err = NF_DEF_DIM(il_ncfile, 'lev', il_nbniv, il_dniv)
      il_err = NF_DEF_DIM(il_ncfile, 'lat', il_lat, il_dlat)
      il_err = NF_DEF_DIM(il_ncfile, 'lon', il_lon, il_dlon)
      IF (KTOTC .ne. 0) THEN
          il_err = NF_DEF_DIM(il_ncfile, 'one', 1, il_done)
      END IF
C
C** DEFINE DIMENSION VARIABLES
C
      ila_ncdim(1) = il_diter
      il_err = NF_DEF_VAR(il_ncfile, 'time', NF_REAL, 1, ila_ncdim,
     $    il_fiter)
C
      ila_ncdim(1) = il_dniv
      il_err = NF_DEF_VAR(il_ncfile, 'lev', NF_REAL, 1, ila_ncdim,
     $    il_fniv)
C
      ila_ncdim(1) = il_dlat
      il_err = NF_DEF_VAR(il_ncfile, 'lat', NF_REAL, 1, ila_ncdim,
     $    il_flat)
C
      ila_ncdim(1) = il_dlon
      il_err = NF_DEF_VAR(il_ncfile, 'lon', NF_REAL, 1, ila_ncdim,
     $    il_flon)
C
      IF (KTOTC .ne. 0) THEN
          ila_ncdim(1) = il_done
          il_err = NF_DEF_VAR(il_ncfile, 'one', NF_REAL, 1, 
     $        ila_ncdim, il_fone)
          il_err = NF_DEF_VAR(il_ncfile, 'Dobson', NF_REAL, 1, 
     $         ila_ncdim, il_fdob) 
          il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fdob,
     $        'long_name', 49,
     $        'Dobson constant for total column units conversion')
      END IF
C
C**  DEFINE ATTRIBUTES
C
      il_ERR = NF_PUT_ATT_TEXT(il_ncfile, NF_GLOBAL, 'Conventions',
     $    6, 'CF-1.0')
      cl_source='MOCAGE with chemical scheme '//trim(CHEMSCHEME) 
      il_ERR = NF_PUT_ATT_TEXT(il_ncfile, NF_GLOBAL, 'source',
     $   len_trim(cl_source), trim(cl_source))
      il_ERR = NF_PUT_ATT_TEXT(il_ncfile, NF_GLOBAL, 'domain',
     $   len_trim(cl_name),cl_name)
      il_ERR = NF_PUT_ATT_INT(il_ncfile, NF_GLOBAL, 'levels',
     $   NF_INT,1,nivm1)
      il_ERR = NF_PUT_ATT_TEXT(il_ncfile, NF_GLOBAL, 'institution',
     $   16, 'CNRM and CERFACS')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_flon,  'standard_name',
     $    9, 'longitude')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_flat,  'standard_name',
     $    8, 'latitude')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fniv,  'standard_name',
     $    43, 'atmosphere_hybrid_sigma_pressure_coordinate')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fniv,  'positive',
     $    4, 'down')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fniv,  'formula_terms'
     $   , 60,
     $   'ap: a_hybr_coord b: b_hybr_coord ps: air_pressure_at_surface')

      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fiter, 'standard_name', 9,
     $    'Time axis')
C
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_flon, 'units', 12,
     $    'degrees_east')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_flat, 'units',13,
     $    'degrees_north')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fniv, 'units', 3, 'lev')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fiter,'axis',  1, 't')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fiter,'units',
     $    LEN_TRIM(cldat), cldat)
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fiter,'time_origin',
     $    LEN_TRIM(cltori), cltori)
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_fiter,'calendar', 6,
     $    'noleap')
C
C
C** DEFINE PRESSURE COORDINATE VARIABLES AND ATTRIBUTES
C
C   SURFACE PRESSURE (2D)
C
      ila_ncdims(1) = il_dlon
      ila_ncdims(2) = il_dlat
      ila_ncdims(3) = il_diter
C
      il_err = NF_DEF_VAR(il_ncfile, 'air_pressure_at_surface', 
     $    NF_REAL,3, ila_ncdims, il_nc_psol)
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_psol, 'standard_name',
     $    23, 'air_pressure_at_surface')
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_psol, 'units',
     $    2, 'Pa')
C
C  HYBRID COORDINATE COEFFICIENTS (2*1D)
C
      ila_ncdims(1) = il_dniv
C
      il_err = NF_DEF_VAR(il_ncfile, 'a_hybr_coord', 
     $    NF_REAL,1, ila_ncdims, il_nc_ahyb)
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_ahyb, 'long_name',
     $    34, 'a coefficient of hybrid coordinate')
C
      il_err = NF_DEF_VAR(il_ncfile, 'b_hybr_coord', 
     $    NF_REAL,1, ila_ncdims, il_nc_bhyb)
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_bhyb, 'long_name',
     $    34, 'b coefficient of hybrid coordinate')
C
C** TEMPERATURE (3D)
C
      ila_ncdims(1) = il_dlon
      ila_ncdims(2) = il_dlat
      ila_ncdims(3) = il_dniv
      ila_ncdims(4) = il_diter

      il_err = NF_DEF_VAR(il_ncfile, 'air_temperature', NF_REAL,4,
     $    ila_ncdims, il_nc_temp)
      il_err = NF_PUT_ATT_TEXT(il_ncfile, il_nc_temp, 'units',
     $    1, 'K')

C
C** DEFINE FIELD VARIABLES AND ATTRIBUTES
C
      ALLOCATE (ila_nc_field(il_outsp)) ; ila_nc_field(:) = 0
      IF (KTOTC .ne. 0) THEN
          ALLOCATE (ila_nc_ctot(il_outsp)) ; ila_nc_field(:) = 0
          ALLOCATE (rla_coltot(il_lon,il_lat,nivm1))
                    rla_coltot(:,:,:) = 0.0
          ALLOCATE (rla_coltot2d(il_lon,il_lat));rla_coltot2d(:,:) = 0.0
      ENDIF
C
      do ib = 1, il_outsp
        ila_ncdims(1) = il_dlon
        ila_ncdims(2) = il_dlat
        ila_ncdims(3) = il_dniv
        ila_ncdims(4) = il_diter
C
        il_err = NF_DEF_VAR(il_ncfile, cla_fieldname(ib), NF_REAL,4,
     $      ila_ncdims, ila_nc_field(ib))
C
        cl_fld_stdname="mole_fraction_of_"//cla_fieldname(ib)
        il_err = NF_PUT_ATT_TEXT(il_ncfile, ila_nc_field(ib), 
     $      'standard_name' ,LEN_TRIM(cl_fld_stdname),
     $      TRIM(cl_fld_stdname))
C
        cl_fieldunit = 'ppv'
        il_err = NF_PUT_ATT_TEXT(il_ncfile, ila_nc_field(ib), 'units',
     $      LEN_TRIM(cl_fieldunit), cl_fieldunit)
C
        IF (KTOTC .ne. 0) THEN
            ila_ncdims(1) = il_dlon
            ila_ncdims(2) = il_dlat
            ila_ncdims(3) = il_diter
            cl_fld_stdname=TRIM(cla_fieldname(ib))//"_tc"
            il_err = NF_DEF_VAR(il_ncfile, TRIM(cl_fld_stdname),
     $          NF_REAL,3,ila_ncdims, ila_nc_ctot(ib))
            cl_fld_stdname="Total column of "//cla_fieldname(ib)
            il_err = NF_PUT_ATT_TEXT(il_ncfile, ila_nc_ctot(ib),
     $          'long_name', LEN_TRIM(cl_fld_stdname),
     $          TRIM(cl_fld_stdname))
            cl_fieldunit = 'molec m-2' 
            il_err = NF_PUT_ATT_TEXT(il_ncfile, ila_nc_ctot(ib),
     $          'units', LEN_TRIM(cl_fieldunit), cl_fieldunit)
        END IF
C
      END DO

C
C** END OF DEFINE MODE
C
      il_err= NF_ENDDEF(il_ncfile)
C
C** WRITE DATA TO FILE
C
      il_err = NF_PUT_VAR_REAL(il_ncfile,il_flon,rla_dim_lo)
C
      il_err = NF_PUT_VAR_REAL(il_ncfile,il_flat,rla_dim_la)
C
      il_err = NF_PUT_VAR_REAL(il_ncfile,il_fniv,
     $    rla_dim_ni(1:il_nbniv))
C  
      ila_ncstart(1) = 1
      ila_nccount(1) = 1
      rl_daynum=REAL(daynum)
      il_err = NF_PUT_VARA_REAL(il_ncfile,il_fiter,ila_ncstart,
     $    ila_nccount,rl_daynum)
C
      IF (KTOTC .ne. 0) THEN
          ila_ncstart(1) = 1
          ila_nccount(1) = 1
          rl_daynum=1.
          il_err = NF_PUT_VARA_REAL(il_ncfile,il_fone,ila_ncstart,
     $        ila_nccount,rl_daynum)
          rl_daynum=2.685e20
          il_err = NF_PUT_VARA_REAL(il_ncfile,il_fdob,ila_ncstart,
     $        ila_nccount,rl_daynum)
      END IF          
C
C output of the surface pressure      
C
      ila_ncstart(1) = 1
      ila_ncstart(2) = 1
      ila_ncstart(3) = 1
      ila_nccount(1) = il_lon
      ila_nccount(2) = il_lat
      ila_nccount(3) = 1
	rla_psolml=REAL(psolm1)	
      il_err = NF_PUT_VARA_REAL(il_ncfile,il_nc_psol, ila_ncstart,
     $    ila_nccount,rla_psolml)
C
C output of the temperature   
C
      ila_ncstart(1) = 1
      ila_ncstart(2) = 1
      ila_ncstart(3) = 1
      ila_ncstart(4) = 1
      ila_nccount(1) = il_lon
      ila_nccount(2) = il_lat
      ila_nccount(3) = il_nbniv
      ila_nccount(4) = 1
C      
      DO ib_k = 1, il_nbniv
        rla_field(:,:,:)=
     $      REAL(TGRIM1(:,:,:))
      END DO

      il_err = NF_PUT_VARA_REAL(il_ncfile,il_nc_temp,
     $    ila_ncstart, ila_nccount,rla_field)

C
C output of the pressure coordinate coefficients    
C
      ila_ncstart(1) = 1
      ila_nccount(1) = il_nbniv
      DO ib_k = 1, KNLEVELS
	rla_alm1(ib_k)=REAL(alm1(iga_level(ib_k)))
        rla_blm1(ib_k)=REAL(blm1(iga_level(ib_k)))
      END DO
C
      il_err = NF_PUT_VARA_REAL(il_ncfile,il_nc_ahyb, ila_ncstart,
     $    ila_nccount,rla_alm1(1:KNLEVELS))
C
      il_err = NF_PUT_VARA_REAL(il_ncfile,il_nc_bhyb, ila_ncstart,
     $    ila_nccount,rla_blm1(1:KNLEVELS))
C
C output of the species
C
      DO ib =1, il_outsp
        ila_ncstart(1) = 1
        ila_ncstart(2) = 1
        ila_ncstart(3) = 1
        ila_ncstart(4) = 1
        ila_nccount(1) = il_lon
        ila_nccount(2) = il_lat
        ila_nccount(3) = il_nbniv
        ila_nccount(4) = 1
        IF (ila_field_kd(ib) .eq. 1) THEN
            DO ib_k = 1, il_nbniv
              rla_field(:,:,ib_k)=
     $            REAL(QQQM1(:,:,iga_level(ib_k),
     $            ila_field_pos(ib)))
            END DO
        ELSE
            DO ib_k = 1, il_nbniv
              rla_field(:,:,ib_k)=
     $            REAL(HHHM1(:,:,iga_level(ib_k),
     $            ila_field_pos(ib)))
            END DO
        ENDIF

        il_err = NF_PUT_VARA_REAL(il_ncfile,ila_nc_field(ib),
     $      ila_ncstart, ila_nccount,rla_field)
        IF (KTOTC .ne. 0) THEN
            IF (ila_field_kd(ib) .eq. 1) THEN
                CALL sumvert (il_lon,il_lat,NIVM1, XPAM1,
     $              XPHFM1,QQQM1(:,:,:,ila_field_pos(ib)),
     $              rla_coltot)
            ELSE
                CALL sumvert (il_lon,il_lat,NIVM1, XPAM1,
     $              XPHFM1,HHHM1(:,:,:,ila_field_pos(ib)),
     $              rla_coltot)
            ENDIF
            rla_coltot2d=REAL(rla_coltot(:,:,NIVM1))
            ila_ncstart(1) = 1
            ila_ncstart(2) = 1
            ila_ncstart(3) = 1
            ila_nccount(1) = il_lon
            ila_nccount(2) = il_lat
            ila_nccount(3) = 1
            il_err = NF_PUT_VARA_REAL(il_ncfile,ila_nc_ctot(ib),
     $          ila_ncstart, ila_nccount,rla_coltot2d)
        END IF           
      END DO
C
C** CLOSE THE NETCDF FILE
C
      il_err = NF_CLOSE(il_ncfile)
C
      DEALLOCATE(rla_dim_lo)
      DEALLOCATE(rla_dim_la)
      DEALLOCATE(rla_field)
      DEALLOCATE(rla_psolml)
      DEALLOCATE(ila_nc_field)
      IF (KTOTC .ne. 0) THEN
          DEALLOCATE (ila_nc_ctot)
          DEALLOCATE (rla_coltot)
          DEALLOCATE (rla_coltot2d)
      ENDIF
C
      DEALLOCATE(ila_field_kd)
      DEALLOCATE(ila_field_pos)
      DEALLOCATE(cla_fieldname)
C
	DEALLOCATE (idate)
	DEALLOCATE (idate1)
	DEALLOCATE (nbjour)
C
C
      RETURN
      END
C      
