SUBROUTINE pl_trigger_unit(cd_unit)
!
!****  pl_trigger_unit - List of units and their identifier
!
!     Purpose:
!     --------
!       User subroutine to describe the unit list.
!       This subroutine is called by the PALM library to active
!       a new unit.
!
!***  Interface:
!     ----------
!         CALL pl_trigger_unit(cd_unit)
!
!      Input:
!      ------
!         cd_unit: Unit identifier
!
!     Externals:
!     ----------
!       Modules:
!       --------
!          palmlib, pl_def_err
!
!     History:
!     -------
!     file automaticaly generated by PREPALM, don't edit this file
!*----------------------------------------------------------------
!
!** 1. DECLARATION
!
!** 1.1 Modules
!
!
  USE palmlib
  USE pl_def_err
  

  IMPLICIT NONE

  CHARACTER(LEN=PL_LNAME) :: cd_unit

  
  cd_unit = ADJUSTL(cd_unit)
  SELECT CASE(cd_unit)
  CASE('Branch_one')
      CALL Branch_one
  CASE('Branch_obs')
      CALL Branch_obs
  CASE('MODEL')
      CALL MODEL
  CASE('MODEL_fcst')
      CALL MODEL
  CASE('INV_R')
      CALL INV_R
  CASE('WRTOBS')
      CALL WRTDAT
  CASE('Bkwd_ADJ')
      CALL backward_propagation
  CASE('Fwd_TL')
      CALL propagate_incr
  CASE('increment_to_model')
      CALL gauss2model
  CASE('p_sigma_to_gauss')
      CALL model2gauss
  CASE('model2gauss_ini')
      CALL model2gauss
  CASE('model2gauss')
      CALL model2gauss
  CASE('INIT')
      CALL INIT
  CASE('INIT_OBS')
      CALL OBSINIT
  CASE('sqrtB')
      CALL sqrtB
  CASE('sqrtB_fcst')
      CALL sqrtB
  CASE('sqrtB_T')
      CALL sqrtB_adj
  CASE('INITv')
      CALL INITv
  CASE('INISTATS')
      CALL INISTATS
  CASE('plot_inc')
      CALL plot_inc
  CASE('LECTOBS')
      CALL LECTDAT
  CASE('H')
      CALL H_NOV
  CASE('H_LT')
      CALL H_NOV_LT
  CASE('Hstar')
      CALL H_NOV_ADJ
  CASE DEFAULT
	WRITE(PL_OUT,*) 'Error: Unit Not found!'
        call flush(PL_OUT)
  END SELECT
  
END SUBROUTINE pl_trigger_unit
!
!*=========================================================================
!
SUBROUTINE pl_trigger_brch(cd_branch)
!
!****  pl_trigger_brch - List of branches and their identifier
!
!     Purpose:
!     --------
!       User subroutine to describe the branch list.
!       This subroutine is called by the PALM library to active
!       a new branch.
!
!***  Interface:
!     ----------
!           CALL pl_trigger_branch(cd_branch)
!
!      Input:
!      ------
!         cd_unit: branch identifier
!
!
!     Externals:
!     ----------
!       Modules:
!       --------
!          palmlib, pl_def_err
!
!     History:
!     -------
!     file automaticaly generated by PREPALM, don't edit this file
!*----------------------------------------------------------------
!
!** 1. DECLARATION
!
!** 1.1 Modules
!
  USE palmlib
  USE pl_def_err

  IMPLICIT NONE

  CHARACTER(LEN=PL_LNAME) :: cd_branch

  cd_branch = ADJUSTL(cd_branch)

  SELECT CASE(cd_branch)
  CASE('Branch_one')
      CALL Branch_one
  CASE('Branch_obs')
      CALL Branch_obs
  CASE DEFAULT
  WRITE(PL_OUT,*) 'Error: Branch Not found!'
  call flush(PL_OUT)

  END SELECT

END SUBROUTINE pl_trigger_brch
!
!*=================================================================
!

SUBROUTINE pl_descriptor_trigger(cd_function_name, id_action, id_nproc, &
   id_arank, ida_shape, id_size, ida_descr)
!
!****  pl_descriptor_trigger
!
!     Purpose:
!     --------
!       User subroutine for calling a descriptor function.
!       These functions allow to describe the distribution of an object on
!       a unit.  
!       It must have in input : 
!         id_action        : contains PL_GIVE_SIZE if the user function must
!                            return the size of the descriptor vector, 
!                            PL_GIVE_DESCR if the user function must return the
!                            filled descriptor vector
!         id_nproc         : number of processor of the unit on which the 
!                            object is distributed
!         id_arank         : rank of the global array
!         ida_shape        : shape of the global array
!       in output :
!         id_size : the size of the descriptor vector ida_descr
!         ida_descr : the descriptor vector 
!       For the custom descriptor, the descriptor vector must contains :
!       For each processor :
!          - number of blocs
!          - shape of the local array
!          For each bloc :
!             - shape of the bloc
!             - global start of the bloc
!             - local start of the bloc
!          End for
!       End for  
!       
!
!***  Interface:
!     ----------
!         CALL pl_descriptor_trigger(cd_function_name, id_action, id_nproc, &
!   id_arank, ida_shape, id_size, ida_descr)
!
!      Inputs:
!      -------
!         cd_function_name : name of the function to call
!         id_action        : contains PL_GIVE_SIZE if the user function must
!                            return the size of the descriptor vector, 
!                            PL_GIVE_DESCR if the user function must return the
!                            filled descriptor vector
!         id_nproc         : number of processor of the unit on which the 
!                            object is distributed
!         id_arank         : rank of the global array
!         ida_shape        : shape of the global array
!
!      Outputs:
!      --------
!         id_size : the size of the descriptor vector ida_descr
!         ida_descr : the descriptor vector 
!
!

!     Externals:
!     ----------
!       Modules:
!       --------
!          palmlib, pl_def_err
!
!     History:
!     -------
!     file automaticaly generated by PREPALM, don't edit this file
!*----------------------------------------------------------------
!
!** 1. DECLARATION
!
!** 1.1 Modules
!
  USE palmlib
!  
  IMPLICIT NONE
!

  CHARACTER(LEN=PL_LNAME)       :: cd_function_name
  INTEGER                       :: id_nproc, id_arank, il_err
  INTEGER, DIMENSION(id_arank)  :: ida_shape
  INTEGER                       :: id_action, id_size
  INTEGER, DIMENSION(id_size)   :: ida_descr
!
!*----------------------------------------------------------------
!
  SELECT CASE(cd_function_name)

  CASE DEFAULT
         WRITE(PL_OUT,*) 'Error: Function Not found!'
         call flush(PL_OUT)
         CALL PALM_ABORT(il_err)

  END SELECT

END SUBROUTINE pl_descriptor_trigger
SUBROUTINE pl_algebra_trigger(cd_routine_name, sda_arguments, id_alg_index, &
   id_place)
!
!**** pl_algebra_trigger
!
!     Purpose:
!       call the palm routine envelopping a given algebra routine
!
!
!** + Declarations
!
!** ++ Include files and modules
!
  USE palmlib  !*I contains some parameters (PL_LNAME)
  USE palmdef  !*I contains some parameters and global variables (cg_msg)
  USE pl_def_err !*I for the treatment of the errors
  USE pl_algebra
!
!** ++ Local variables 
!
  IMPLICIT NONE
!
  !*V cp_sub : a parameter for tracking the subroutine
  CHARACTER(LEN=PL_SLGTH), PARAMETER :: cp_sub = 'pl_algebra_trigger' 
!
  !** number of argument of the algebra routine
  INTEGER :: il_nb_arg
!

  INTEGER :: A1(*)
  INTEGER :: A2(*)
  INTEGER :: A3(*)
  INTEGER :: A4(*)
  INTEGER :: A5(*)
  INTEGER :: A6(*)
  INTEGER :: A7(*)
  INTEGER :: A8(*)
  INTEGER :: A9(*)
  INTEGER :: A10(*)
  INTEGER :: A11(*)
  INTEGER :: A12(*)
  INTEGER :: A13(*)
  INTEGER :: A14(*)
  INTEGER :: A15(*)
  INTEGER :: A16(*)
  INTEGER :: A17(*)
  INTEGER :: A18(*)
  POINTER (P1,A1)
  POINTER (P2,A2)
  POINTER (P3,A3)
  POINTER (P4,A4)
  POINTER (P5,A5)
  POINTER (P6,A6)
  POINTER (P7,A7)
  POINTER (P8,A8)
  POINTER (P9,A9)
  POINTER (P10,A10)
  POINTER (P11,A11)
  POINTER (P12,A12)
  POINTER (P13,A13)
  POINTER (P14,A14)
  POINTER (P15,A15)
  POINTER (P16,A16)
  POINTER (P17,A17)
  POINTER (P18,A18)
!
!
!** ++ Dummy variables 
!
  !*V sda_arguments : array containing the arguments of the algebra routine
  !*V                 but converted in a vector of integers format  
  TYPE(argument_t), DIMENSION(*) :: sda_arguments
!
  !*V cd_routine_name : the name of the palm routine envelopping the algebra 
  !*V                   routine we want to call 
  CHARACTER(LEN=PL_LNAME) :: cd_routine_name  
!
  !*V id_alg_index : index of the algebra routine in the algebra table
  INTEGER :: id_alg_index
!
  INTEGER :: id_place, il_err
!
!*------------------------------------------------------------------------
!
  il_nb_arg = sga_alg_table(id_alg_index)%il_nb_arguments
!
  !** Set the pointers of the arguments of the algebra routine
  !** It's a little bit ugly but indispensable because the mslaves uses CRAY 
  !** pointers
  IF (id_place.eq.PL_BUFF) THEN

      IF (il_nb_arg.ge.1) CALL pl_set_P(sda_arguments(1),P1)  
      IF (il_nb_arg.ge.2) CALL pl_set_P(sda_arguments(2),P2)  
      IF (il_nb_arg.ge.3) CALL pl_set_P(sda_arguments(3),P3)  
      IF (il_nb_arg.ge.4) CALL pl_set_P(sda_arguments(4),P4)  
      IF (il_nb_arg.ge.5) CALL pl_set_P(sda_arguments(5),P5)  
      IF (il_nb_arg.ge.6) CALL pl_set_P(sda_arguments(6),P6)  
      IF (il_nb_arg.ge.7) CALL pl_set_P(sda_arguments(7),P7)  
      IF (il_nb_arg.ge.8) CALL pl_set_P(sda_arguments(8),P8)  
      IF (il_nb_arg.ge.9) CALL pl_set_P(sda_arguments(9),P9)  
      IF (il_nb_arg.ge.10) CALL pl_set_P(sda_arguments(10),P10)  
      IF (il_nb_arg.ge.11) CALL pl_set_P(sda_arguments(11),P11)  
      IF (il_nb_arg.ge.12) CALL pl_set_P(sda_arguments(12),P12)  
      IF (il_nb_arg.ge.13) CALL pl_set_P(sda_arguments(13),P13)  
      IF (il_nb_arg.ge.14) CALL pl_set_P(sda_arguments(14),P14)  
      IF (il_nb_arg.ge.15) CALL pl_set_P(sda_arguments(15),P15)  
      IF (il_nb_arg.ge.16) CALL pl_set_P(sda_arguments(16),P16)  
      IF (il_nb_arg.ge.17) CALL pl_set_P(sda_arguments(17),P17)  
      IF (il_nb_arg.ge.18) CALL pl_set_P(sda_arguments(18),P18)  
  ELSE
      IF (il_nb_arg.ge.1) P1 = LOC(sda_arguments(1)%ila_value(1)) 
      IF (il_nb_arg.ge.2) P2 = LOC(sda_arguments(2)%ila_value(1)) 
      IF (il_nb_arg.ge.3) P3 = LOC(sda_arguments(3)%ila_value(1)) 
      IF (il_nb_arg.ge.4) P4 = LOC(sda_arguments(4)%ila_value(1)) 
      IF (il_nb_arg.ge.5) P5 = LOC(sda_arguments(5)%ila_value(1)) 
      IF (il_nb_arg.ge.6) P6 = LOC(sda_arguments(6)%ila_value(1)) 
      IF (il_nb_arg.ge.7) P7 = LOC(sda_arguments(7)%ila_value(1)) 
      IF (il_nb_arg.ge.8) P8 = LOC(sda_arguments(8)%ila_value(1)) 
      IF (il_nb_arg.ge.9) P9 = LOC(sda_arguments(9)%ila_value(1)) 
      IF (il_nb_arg.ge.10) P10 = LOC(sda_arguments(10)%ila_value(1)) 
      IF (il_nb_arg.ge.11) P11 = LOC(sda_arguments(11)%ila_value(1)) 
      IF (il_nb_arg.ge.12) P12 = LOC(sda_arguments(12)%ila_value(1)) 
      IF (il_nb_arg.ge.13) P13 = LOC(sda_arguments(13)%ila_value(1)) 
      IF (il_nb_arg.ge.14) P14 = LOC(sda_arguments(14)%ila_value(1)) 
      IF (il_nb_arg.ge.15) P15 = LOC(sda_arguments(15)%ila_value(1)) 
      IF (il_nb_arg.ge.16) P16 = LOC(sda_arguments(16)%ila_value(1)) 
      IF (il_nb_arg.ge.17) P17 = LOC(sda_arguments(17)%ila_value(1)) 
      IF (il_nb_arg.ge.18) P18 = LOC(sda_arguments(18)%ila_value(1)) 
  END IF
!
!
  SELECT CASE (cd_routine_name)
!

   CASE ('PL_SAXPY')
       CALL PL_SAXPY(A1,A2,A3,A4,A5,A6, id_alg_index)
!
   CASE ('PL_SDOT')
       CALL PL_SDOT(A1,A2,A3,A4,A5,A6,A7, id_alg_index)
!
   CASE ('PL_SLBFGS')
       CALL PL_SLBFGS(A1,A2,A3,A4,A5,A6,A7,A8,A9,A10,A11,A12,A13,A14,A15,A16,A17, id_alg_index)
!
  CASE default
!
     WRITE(PL_OUT,*) 'unknown algebra routine name ',cd_routine_name
     CALL PALM_ABORT(il_err)
!
  END SELECT
!
END SUBROUTINE pl_algebra_trigger
! 
!
!
! algebra triggers or dummy trigger
!
!
SUBROUTINE pl_saxpy(N, alpha, X, incx, Y, incy, id_alg_index)
  USE palmlib  !*I contains some parameters (PL_LNAME)
  USE pl_algebra
  IMPLICIT NONE
  INTEGER, INTENT(IN)                                :: N, incx, incy 
  REAL, INTENT(IN)                  :: alpha
  REAL, DIMENSION(*), INTENT(IN)    :: X
  REAL, DIMENSION(*), INTENT(INOUT) :: Y
  !*V id_alg_index : index of the algebra routine in the algebra table
  INTEGER :: id_alg_index
  !*V cp_sub : a parameter for tracking the subroutine
  CHARACTER(LEN=PL_SLGTH), PARAMETER :: cp_sub = 'pl_daxpy' 
  CHARACTER(LEN=PL_LNAME) :: cl_name
  CALL saxpy(N, alpha, X, incx, Y, incy)
  cl_name = 'All'
  CALL pl_put_obj(cl_name, .true., id_alg_index)
END SUBROUTINE pl_saxpy

SUBROUTINE pl_sdot(N, alpha, X, incx, Y, incy, dot, id_alg_index)
  USE palmlib  !*I contains some parameters (PL_LNAME)
  USE pl_algebra
  USE pl_def_err  !*I for the treatment of errors
  IMPLICIT NONE
  INTEGER, INTENT(IN)                                :: N, incx, incy 
  REAL, INTENT(IN)                  :: alpha
  REAL, DIMENSION(*), INTENT(IN)    :: X
  REAL, DIMENSION(*), INTENT(INOUT) :: Y
  REAL                              :: dot
  !*V id_alg_index : index of the algebra routine in the algebra table
  INTEGER :: id_alg_index
  !*V cp_sub : a parameter for tracking the subroutine
  CHARACTER(LEN=PL_SLGTH), PARAMETER :: cp_sub = 'pl_sdot' 
  CHARACTER(LEN=PL_LNAME) :: cl_name
  REAL   :: rf_sdot_doit
  dot = rf_sdot_doit(N, alpha, X, incx, Y, incy)
  cl_name = 'All'
  CALL pl_put_obj(cl_name, .true., id_alg_index)
END SUBROUTINE pl_sdot

REAL FUNCTION rf_sdot_doit(N, alpha, X, incx, Y, incy)
  IMPLICIT NONE
  INTEGER, INTENT(IN)                                :: N, incx, incy 
  REAL, INTENT(IN)                  :: alpha
  REAL, DIMENSION(*), INTENT(IN)    :: X
  REAL, DIMENSION(*), INTENT(INOUT) :: Y
  !*V cp_sub : a parameter for tracking the subroutine
  REAL   :: sdot
  rf_sdot_doit = alpha * sdot(N, X, incx, Y, incy)
END FUNCTION rf_sdot_doit

SUBROUTINE pl_slbfgs(id_n, id_m, dda_x, dd_f, dda_g, ld_diagco, dda_diag, &
   id_print1, id_print2, dd_eps, dd_xtol, dd_gtol, id_flag, id_nbmaxeval, &
   dda_y, dda_result, ld_finish, id_alg_index)
  USE palmlib  !*I contains some parameters (PL_LNAME)
  USE pl_algebra
  USE pl_def_err  !*I for the treatment of errors
  IMPLICIT NONE
  INTEGER                                   :: id_n, id_m, id_flag, &
     id_print1, id_print2, id_nbmaxeval
  REAL(KIND=KIND(1.0)), DIMENSION(id_n)    :: dda_x, dda_g, dda_result, &
     dda_diag, dda_y
  REAL(KIND=KIND(1.0))                     :: dd_eps, dd_f, dd_xtol, dd_gtol
  LOGICAL :: ld_diagco
  !*V id_alg_index : index of the algebra routine in the algebra table
  INTEGER :: id_alg_index
  LOGICAL :: ld_finish
  !*V cp_sub : a parameter for tracking the subroutine
  CHARACTER(LEN=PL_SLGTH), PARAMETER :: cp_sub = 'pl_slbfgs' 
  CHARACTER(LEN=PL_LNAME) :: cl_name
  !*V dla_w : a working array for lbfgs 
  !*V dla_x : contains the iterate (we must store it in save to avoid the user
  !*V         to put it at each call) 
  !*V dla_diag : contains the diag Hk0 (we must store it in save to avoid the 
  !*V            user to put it at each call) 
  !*V dla_g, dla_f : contains f and g (we must store them in save to avoid the 
  !*V            user to put them at each call) 
  REAL(KIND=KIND(1.0)), DIMENSION(:), ALLOCATABLE, SAVE :: dla_w, dla_x, &
     dla_diag, dla_g, dl_f
  !*V for computing the number of evaluations (and testing if it is superior 
  !*V to this allowed by the user.
  INTEGER, SAVE :: il_nb_eval
  !*V array containing the frequency and the level of printing (shared in to 
  !*V integer in PrePALM because we can't set an array in PrePALM)
  INTEGER, DIMENSION(2) :: ila_print
  INTEGER :: il_iter
  INTEGER :: il_err
  !*V contains the number of the output file for lbfgs and the number of 
  !*V iterations and evaluations
  INTEGER :: ig_mp, ig_lp
  REAL(KIND=KIND(1.0)) :: dg_gtol, dg_stpmin, dg_stpmax
  COMMON /LB3/ig_mp, ig_lp, dg_gtol, dg_stpmin, dg_stpmax
  !** set the accuracy of the line search routine MCSRCH
  dg_gtol = dd_gtol
  ila_print = (/ id_print1, id_print2 /)
  !** it is the first iteration, allocate and initialize arrays
  IF (id_flag.eq.0) THEN 
      ALLOCATE(dla_w(id_n*(2*id_m+1)+2*id_m), dla_x(id_n), dla_diag(id_n), &
	 dl_f(1), dla_g(id_n), stat=il_err)
      IF(il_err.ne.0) THEN
	  cg_msg = 'Error allocating dla_w, dla_x, dla_diag, dl_f or dla_g'
	  CALL pl_err_severe(cp_sub, ip_noalloc, il_err)
      ENDIF
      dla_w = 0.d0
      dla_diag = 0.d0 
      dla_x = dda_x
      il_nb_eval = 1
      !** output file of lbfgs
      IF (id_print1.ge.0) THEN
	  ig_mp = 300478
	  ig_lp = 300478
	  OPEN(ig_mp, file = 'lbfgs.out', position='append')
	  CALL pl_lb1(id_flag, id_print1, id_print2, dd_eps, dd_xtol,  &
	     ld_diagco, id_nbmaxeval, il_nb_eval, 0, id_n, id_m,       &
	     dda_x, dd_f, dda_g, .false.)
      END IF
  ENDIF
  !** if the user has put the diagonal matrix Hk0
  IF ( (ld_diagco).AND.( (id_flag.eq.0).OR.(id_flag.eq.2) ) ) THEN
      dla_diag = dda_diag
  END IF
  !** if the user has put the value of the function and of the gradient
  IF (id_flag.le.1) THEN
      dl_f(1) = dd_f
      dla_g = dda_g
  END IF
  IF (ld_finish) THEN
      !** the user has asked lbfgs to stop
      IF (id_flag.eq.0) THEN 
	  il_iter = 0
      ELSE
	  il_iter = -1
      END IF
      CALL pl_lb1(id_flag, id_print1, id_print2, dd_eps, dd_xtol,  &
	 ld_diagco, id_nbmaxeval, il_nb_eval, il_iter, id_n, id_m, &
	 dda_x, dd_f, dda_g, ld_finish)
      id_flag = 0
  ELSE
      CALL SLBFGS(id_n, id_m, dla_x, dl_f, dla_g, ld_diagco, dla_diag, &
	 ila_print, dd_eps, dd_xtol, dla_w, id_flag)
  END IF
  !** test of termination in case of too much evaluations
  IF (id_flag.eq.1) THEN
      il_nb_eval = il_nb_eval + 1
      IF (il_nb_eval.gt.id_nbmaxeval) THEN 
	  id_flag = 3
	  CALL pl_lb1(id_flag, id_print1, id_print2, dd_eps, dd_xtol,  &
	     ld_diagco, id_nbmaxeval, il_nb_eval-1, -1, id_n, id_m, &
	     dla_x, dl_f, dla_g, .true.)
      END IF
  END IF
  cl_name = 'All'
  CALL pl_get_at_next_step(cl_name, .true., id_alg_index)
  CALL pl_put_obj(cl_name, .false., id_alg_index)
  cl_name = 'iflag'
  CALL pl_put_obj(cl_name, .true., id_alg_index) !** put iflag

  SELECT CASE(id_flag)
  CASE (1)
      !** the user must provide f(x) and g(x)
      cl_name = 'x'
      CALL pl_put_obj(cl_name, .true., id_alg_index) !** put x
      !** don't get x at the next iteration
      cl_name = 'x'
      CALL pl_get_at_next_step(cl_name, .false., id_alg_index) 
      !** don't get dda_diag at the next iteration
      cl_name = 'diag'
      CALL pl_get_at_next_step(cl_name, .false., id_alg_index) 
      dda_x = dla_x
  CASE (2)
      !** the user must provide dda_diag 
      cl_name = 'y'
      CALL pl_put_obj(cl_name, .true., id_alg_index) !** put y
      !** don't get x at the next iteration
      cl_name = 'x'
      CALL pl_get_at_next_step(cl_name, .false., id_alg_index) 
      !** don't get f at the next iteration
      cl_name = 'f'
      CALL pl_get_at_next_step(cl_name, .false., id_alg_index) 
      !** don't get g at the next iteration
      cl_name = 'g'
      CALL pl_get_at_next_step(cl_name, .false., id_alg_index) 
      dda_y = dla_x 
  CASE default
      !** the minimization is finished
      IF (id_print1.ge.0) THEN
	  WRITE(ig_mp,*)
	  WRITE(ig_mp,*)
	  CLOSE(ig_mp)
      END IF
      cl_name = 'result'
      CALL pl_put_obj(cl_name, .true., id_alg_index) !** put result
      dda_result = dla_x
      il_nb_eval = 0
      DEALLOCATE(dla_x, dla_diag, dla_w, dl_f, dla_g, stat=il_err)
      IF(il_err.ne.0) THEN
	  cg_msg = 'error deallocating dla_x, dla_diag, dla_w, dl_f or dla_g'
	  CALL pl_err_severe(cp_sub, ip_nodeall, il_err)
      ENDIF
  END SELECT
END SUBROUTINE pl_slbfgs




SUBROUTINE pl_lb1(id_flag, id_print1, id_print2, dd_eps, dd_xtol, ld_diagco, &
   id_nbmaxeval, id_nb_eval, id_iter, id_n, id_m, dda_x, dd_f, dda_g,        &
   ld_finish)
  USE palmlib  !*I contains some parameters (PL_LNAME)
  USE pl_algebra
  USE pl_def_err  !*I for the treatment of errors
  IMPLICIT NONE
  INTEGER                                   :: id_n, id_flag, id_nbmaxeval, &
     id_print1, id_print2, id_nb_eval, id_iter, id_m
  REAL(KIND=KIND(1.0)), DIMENSION(id_n)    :: dda_x, dda_g
  REAL(KIND=KIND(1.0))                     :: dd_eps, dd_f, dd_xtol
  !*V contains the number of the output file for lbfgs and the number of 
  !*V iterations and evaluations
  INTEGER :: ig_mp, ig_lp
  REAL(KIND=KIND(1.0)) :: dg_gtol, dg_stpmin, dg_stpmax
  !*V a logical to tell lbfgs to stop if the stopping criterion has been 
  !*V fulfilled
  LOGICAL :: ld_finish, ld_diagco
  REAL(KIND=KIND(1.0))   :: sdot
  !*V cp_sub : a parameter for tracking the subroutine
  CHARACTER(LEN=PL_SLGTH), PARAMETER :: cp_sub = 'pl_lb1' 
  INTEGER :: il_err
  COMMON /LB3/ig_mp, ig_lp, dg_gtol, dg_stpmin, dg_stpmax
  !** no output, the level of printing is too small
  IF (id_print1.lt.0) RETURN

  IF ((id_iter.eq.0).and.(id_flag.eq.0).and.(.not.ld_finish) ) THEN
      !** it is the beginning of lbfgs, write some usefull informations in the
      !** output file

      WRITE(ig_mp,10)
      WRITE(ig_mp,20) dd_eps 
      WRITE(ig_mp,30) dd_xtol
      WRITE(ig_mp,40) ld_diagco
      WRITE(ig_mp,50) id_nbmaxeval
      
  END IF

  !** the minimisation is finished
  IF (ld_finish) THEN

      SELECT CASE(id_flag)

      CASE(0,1,2)
	  !** the user has asked lbfgs to stop
	  CALL LB1((/id_print1, id_print2/), id_iter, id_nb_eval,            &
	     SQRT(sdot(id_n, dda_g, 1, dda_g, 1)), id_n, id_m, dda_x, dd_f,  &
	     dda_g, 0.d0, .false.)	  
	  WRITE(ig_mp,*)
	  WRITE(ig_mp,60)
	  WRITE(ig_mp,70)      
	  
      CASE(3)
	  !** the max number of evaluations has been reached 
	  CALL LB1((/id_print1, id_print2/), -1, id_nb_eval,                 &
	     SQRT(sdot(id_n, dda_g, 1, dda_g, 1)), id_n, id_m, dda_x, dd_f,  &
	     dda_g, 0.d0, .false.)	  
	  WRITE(ig_mp,*)
	  WRITE(ig_mp,80) id_nbmaxeval
	  WRITE(ig_mp,90)      

      CASE default
	  WRITE(cg_msg,*) 'Invalid flag ',id_flag
	  CALL pl_err_severe(cp_sub, ip_generic, il_err)

      END SELECT

  END IF
 10   FORMAT('*************************************************')
 20   FORMAT('Tolerance for the stopping criterion : ',D10.3)
 30   FORMAT('Estimate of the machine precision : ',D10.3)
 40   FORMAT('Diagonal matrix Hk0 provided : ',L1)
 50   FORMAT('Maximum number of evaluations : ',I5)
 60   FORMAT('LBFGS TERMINATED ON USER DEFINED STOPPING CRITERION (FINISH = .TRUE.)')
 70  FORMAT('IFLAG = 0')
 80  FORMAT('LBFGS EXCEEDED NUMBER OF EVALUATIONS ',I5)
 90  FORMAT('IFLAG = 3')
END SUBROUTINE pl_lb1

 
!
!
!
!
!=========================================================================
!
!
!
!
SUBROUTINE pl_set_P(sd_argument,P)
!
!**** pl_set_P
!
!     Purpose:
!       Make P point on the adress of an algebra argument (described by its 
!       handle in the buffer/mailbuff or by the variable containing the 
!       argument)
!       Called by pl_algebra_trigger
!
!     Interface:
!       call pl_set_P(sd_argument,P)
!       Input : 
!         sd_argument : structure containing either the handle of the argument either its value
!       Ouput :
!         P : CRAY pointer to the argument 
!
!***  Method:
!       If there is a handle, call the dmem_lock subroutine, otherwise make a 
!       loc to make P point on sd_argument%ila_value
!
!     Externals:
!
!       pl_get_at_next_step
!
!     History:
!     --------
!      Version    Programmer      Date         Description
!      ---------------------------------------------------
!      1.0        SB            2001/05/11     Created
!*------------------------------------------------------------------------
!
!** + Declarations
!
!** ++ Include files and modules
!
  USE palmlib  !*I contains some parameters (PL_LNAME)
  USE palmdef  !*I contains some parameters and global variables (cg_msg)
  USE pl_def_err !*I for the treatment of the errors
  USE pl_algebra
!
!** ++ Local variables 
!
  IMPLICIT NONE
  INTEGER :: A(*), il_err
!
!** ++ Dummy variables 
!
!
  pointer (P,A)
  TYPE(argument_t) :: sd_argument
!
!*------------------------------------------------------------------------
!
!
  IF (sd_argument%il_hdle.le.0) THEN

      P = loc(sd_argument%ila_value(1))

  ELSE

      CALL fdmem_lock(sd_argument%il_hdle,P,il_err)

  END IF
!
!
END SUBROUTINE pl_set_P
!
!
!
!
!=========================================================================
!
!
!
!
SUBROUTINE pl_algebra_trigger_init(cd_routine_name, id_alg_index)
!
!**** pl_algebra_trigger_init
!
!     Purpose:
!       Initialize the communication context of an algebra routine \
!       (establish what sent inputs will be received for the first call \
!        of the algebra routine
!        Called by pl_fill_alg_table
!
!     Interface:
!       call pl_algebra_trigger_init(cd_routine_name, id_alg_index)
!       input : 
!egin{itemize} 
!item          cd_routine_name : name of the palm routine envelopping the \
!                                 algebra routine we want to run
!item          id_alg_index : index of the algebra unit in the algebra table
!end{itemize} 
!
!***  Method:
!       Select the name of the palm routine envelopping the algebra routine \
!       and depending on it, set sla_sent_inputs()%ll_communicate and 
!       sla_outputs to .false. or .true. thanks to pl_get_at_next_step and
!       pl_put_obj
!
!     Externals:
!
!       pl_get_at_next_step
!
!     History:
!     --------
!      Version    Programmer      Date         Description
!      ---------------------------------------------------
!      1.0        SB            2000/10/21     Created
!*------------------------------------------------------------------------
!
!** + Declarations
!
!** ++ Include files and modules
!
  USE palmlib  !*I contains some parameters (PL_LNAME)
  USE palmdef  !*I contains some parameters and global variables (cg_msg)
  USE pl_def_err !*I for the treatment of the errors
  USE pl_algebra
!
!** ++ Local variables 
!
  IMPLICIT NONE
!
  !*V cp_sub : a parameter for tracking the subroutine
  CHARACTER(LEN=PL_SLGTH), PARAMETER :: cp_sub = 'pl_init_algebra_trigger' 
  CHARACTER(LEN=PL_LNAME) :: cl_name
!
!** ++ Dummy variables 
!
!
  !*V cd_routine_name : the name of the palm routine envelopping the algebra 
  !*V                   routine we want to call 
  CHARACTER(LEN=PL_LNAME) :: cd_routine_name  
!
  !*V id_alg_index : index of the algebra routine in the algebra table
  INTEGER :: id_alg_index,il_err
!
!*------------------------------------------------------------------------
!
  SELECT CASE (cd_routine_name)
!
  CASE ('PL_SAXPY')
!
      cl_name = 'All'
      CALL pl_get_at_next_step(cl_name,.true., id_alg_index)
!
  CASE ('PL_SDOT')
!
      cl_name = 'All'
      CALL pl_get_at_next_step(cl_name,.true., id_alg_index)
!
  CASE ('PL_SLBFGS')
!
      cl_name = 'All'
      CALL pl_get_at_next_step(cl_name,.true., id_alg_index)
!
  CASE default
!
      WRITE(PL_OUT,*) 'unknown algebra routine name ',cd_routine_name
      CALL PALM_ABORT(il_err)
!
  END SELECT
!
END SUBROUTINE pl_algebra_trigger_init 
