C
C**** palm_user_param - Declaration of user parameters
C
C     Purpose:
C     --------
C     To provide the mnemonic keys for user subroutine call
C
C     Interface:
C     ---------
C       INCLUDE "palm_user_param.h"
C
C***
C     History:
C     --------
C       Version     Programmer      Date        Description
C       -------     ----------      ----        -----------
C       1.0         Andrea          2001/08/08  Created
C*----------------------------------------------------------------------
C
C**  1. DECLARATIONS
C
C
C**  1.1  Dimension
C
C   ip_lonm1:  ! Number of model grid longitudes 
       INTEGER ip_lonm1
       PARAMETER (ip_lonm1 = 180)
C   ip_latm1:  ! Number of model grid latitudes 
       INTEGER ip_latm1
       PARAMETER (ip_latm1 = 90)
C   ip_nivm1:  ! Number of model grid vertical levels 
       INTEGER ip_nivm1
       PARAMETER (ip_nivm1 = 60)
C   ip_lonc1:  ! Number of control grid longitudes 
       INTEGER ip_lonc1
       PARAMETER (ip_lonc1 = ip_lonm1)
C   ip_latc1:  ! Number of control grid latitudes 
       INTEGER ip_latc1
       PARAMETER (ip_latc1 = ip_latm1)
C   ip_gauss_modelgrid:  ! Model grid type: 0 if regular, 1 if Gauss (e.g. MOCAGE T42) 
       INTEGER ip_gauss_modelgrid
       PARAMETER (ip_gauss_modelgrid = 0)
C   ip_ntra1:  ! Number of transported species 
       INTEGER ip_ntra1
       PARAMETER (ip_ntra1 = 1)
C   ip_nsls1:  ! Number of short-lived species 
       INTEGER ip_nsls1
       PARAMETER (ip_nsls1 = 0)
C   ip_nassim:  ! Number of assimilated transported species 
       INTEGER ip_nassim
       PARAMETER (ip_nassim = 1)
C   ip_ninstmax:  ! Max number of instruments for one species 
       INTEGER ip_ninstmax
       PARAMETER (ip_ninstmax = 1)
C   ip_ninstmax_avk:  ! Max number of instruments with avg. kernel (AVKERN, INTQTY) 
       INTEGER ip_ninstmax_avk
       PARAMETER (ip_ninstmax_avk = 0)
C   ip_nobsmax:  ! Maximun number of observation each day 
       INTEGER ip_nobsmax
       PARAMETER (ip_nobsmax = 2000)
C   ip_nobstspmax:  ! Maximun number of observation each slot 
       INTEGER ip_nobstspmax
       PARAMETER (ip_nobstspmax = 200)
C   ip_nobsperinst_noavk_slot:  ! Maximun nb of obs in a slot per instr. without av. kern. 
       INTEGER ip_nobsperinst_noavk_slot
       PARAMETER (ip_nobsperinst_noavk_slot = 200)
C   ip_nobsperinst_avk_slot:  ! Maximun nb of obs in a slot per instr. with av. kern. 
       INTEGER ip_nobsperinst_avk_slot
       PARAMETER (ip_nobsperinst_avk_slot = 0)
C   ip_nbnivpresmax:  ! Maximum number of pressure levels per profile 
       INTEGER ip_nbnivpresmax
       PARAMETER (ip_nbnivpresmax = 18)
C   ip_nbniv_avk_in:  ! Maximum number of levels before application of avg. kern. 
       INTEGER ip_nbniv_avk_in
       PARAMETER (ip_nbniv_avk_in = 7)
C   ip_nbniv_avk_out:  ! Maximum number of levels after application of avg. kern. 
       INTEGER ip_nbniv_avk_out
       PARAMETER (ip_nbniv_avk_out = 7)
C   ip_tc_as_prof:  ! Tot col treatment: 0 as scalar, 1 as shifted profiles 
       INTEGER ip_tc_as_prof
       PARAMETER (ip_tc_as_prof = 0)
C   ip_nbconstmax:  ! Maximum number of constituants in obs 
       INTEGER ip_nbconstmax
       PARAMETER (ip_nbconstmax = 1)
C   ip_latobsmin:  ! Observations north of this latitude are discarded 
       INTEGER ip_latobsmin
       PARAMETER (ip_latobsmin = -90)
C   ip_latobsmax:  ! Observations south of this latitude are discarded 
       INTEGER ip_latobsmax
       PARAMETER (ip_latobsmax = 90)
C   ip_lonobsmin:  ! Observation west of this lon. are discarded. Use [-180,180] period 
       INTEGER ip_lonobsmin
       PARAMETER (ip_lonobsmin = -180)
C   ip_lonobsmax:  ! Observation east of this lon. are discarded. Use [-180,180] period 
       INTEGER ip_lonobsmax
       PARAMETER (ip_lonobsmax = 180)
C   ip_presobsmin:  ! Observations over this pressure (Pa) are discarded 
       INTEGER ip_presobsmin
       PARAMETER (ip_presobsmin = 0)
C   ip_presobsmax:  ! Observations under this pressure (Pa) are discarded 
       INTEGER ip_presobsmax
       PARAMETER (ip_presobsmax = 150000)
C   ip_enforced_obs_min:  ! Min enforced observation value  ** in ppb ** 
       INTEGER ip_enforced_obs_min
       PARAMETER (ip_enforced_obs_min = 0)
C   ip_enforced_ana_min:  ! Min enforced analysis field value  ** in ppb ** 
       INTEGER ip_enforced_ana_min
       PARAMETER (ip_enforced_ana_min = 0)
C   ip_scrip:  ! Grid remapping: 1 for SCRIP, 0 for SPHEREPACK 
       INTEGER ip_scrip
       PARAMETER (ip_scrip = 1)
C   ip_nbfenet:  ! Number of assimilation windows 
       INTEGER ip_nbfenet
       PARAMETER (ip_nbfenet = 8)
C   ip_fenetre:  ! Size (in sec.) of each assmilation window 
       INTEGER ip_fenetre
       PARAMETER (ip_fenetre = 3*3600)
C   ip_obs_slot:  ! Observ. time slot i.e. frequency of model puts (secs.) 
       INTEGER ip_obs_slot
       PARAMETER (ip_obs_slot = 1*3600)
C   ip_correctime:  ! Time where the correction is added 
       INTEGER ip_correctime
       PARAMETER (ip_correctime = ip_fenetre * 0/3)
C   ip_insincr:  ! If ip_insincr = 1, the increment is added linearly over the assimilation windows 
       INTEGER ip_insincr
       PARAMETER (ip_insincr = 0)
C   ip_niter:  ! Max number of iterations of the minimizer 
       INTEGER ip_niter
       PARAMETER (ip_niter = 100)
C   ip_omf:  ! ip_omf = 1 to output obs. minus forecast diags 
       INTEGER ip_omf
       PARAMETER (ip_omf = 1)
C   ip_oma:  ! ip_oma = 1 to output obs. minus analysis diags 
       INTEGER ip_oma
       PARAMETER (ip_oma = 1)
C   ip_omd:  ! ip_omd = 1 to output obs. minus dry run spanning the whole assim. exp. diags 
       INTEGER ip_omd
       PARAMETER (ip_omd = 1)
C   ip_restart_on_dsk:  ! if = 0 keep old i.c. in palm, if =1 store it on disk 
       INTEGER ip_restart_on_dsk
       PARAMETER (ip_restart_on_dsk = 0)
C   ip_iprint:  ! Toggle diagnostic output 
       INTEGER ip_iprint
       PARAMETER (ip_iprint = 0)
C   ip_loader_verbosity:  ! Obs loader verbosity : 0 = silent, 1 = rejection info, 2 complete info 
       INTEGER ip_loader_verbosity
       PARAMETER (ip_loader_verbosity = 0)
C   ip_assimsize:  ! Size (in sec.) of global assmilation window 
       INTEGER ip_assimsize
       PARAMETER (ip_assimsize = ip_nbfenet*ip_fenetre)
C   ip_lono1:  ! Number of observable grid longitudes 
       INTEGER ip_lono1
       PARAMETER (ip_lono1 = ip_lonc1)
C   ip_lato1:  ! Number of observable grid latitudes 
       INTEGER ip_lato1
       PARAMETER (ip_lato1 = ip_latc1)
C   ip_dimx:  ! Vector size of transported species 
       INTEGER ip_dimx
       PARAMETER (ip_dimx = ip_lonc1*ip_latc1*ip_nivm1*ip_nassim)
C   ip_dimobs:  ! Dimension of observation space 
       INTEGER ip_dimobs
       PARAMETER (ip_dimobs = ip_nobstspmax*ip_nbnivpresmax*ip_nbconstma
     $ x)

C
C*======================================================================
C

