#!/bin/ksh
#
#=============================#
# Others files used by MOCAGE #
#=============================#
#
echo
echo '====================='
echo '>>> Script getauxi.sh'
echo '====================='
#
if [ $CRUN -eq 1 ] ; then
PATH_CSTE=$MOCAGE_ROOT/MOCAGE_CONSTANTS

#####
#
# Fichiers parametres et MPI
#
if [ $MOCFG = T42 ] ; then
    echo "Copy paradi_${CHEMSCHEME}CLMT42 from $DIRSOURCES"
    echo
    cp $DIRSOURCES/paradi_${CHEMSCHEME}CLMT42 ./namelist_param
elif [ $MOCFG = T21 ] ; then
    echo "Copy paradi_${CHEMSCHEME}CLMT21 from $DIRSOURCES"
    echo
    cp $DIRSOURCES/paradi_${CHEMSCHEME}CLMT21 ./namelist_param
else
    echo "Copy paradi_${CHEMSCHEME}${NDOM}DOM from $DIRSOURCES"
    echo
    cp $DIRSOURCES/paradi_${CHEMSCHEME}${NDOM}DOM ./namelist_param
fi
#####
#
# CHEMSCHEME definition and files
#
FILE_CHEMSCHEME=$MOCAGE_ROOT/MOCAGE/${CHEMSCHEME}/${CHEMSCHEME}_DEF
if test -s $FILE_CHEMSCHEME ; then
 NTRA1=`grep NTRA1= $FILE_CHEMSCHEME`
 NTRA1=`expr $NTRA1 | cut -c7-`
 NSLS1=`grep NSLS1= $FILE_CHEMSCHEME`
 NSLS1=`expr $NSLS1 | cut -c7-`
 NK=`grep NK= $FILE_CHEMSCHEME`
 NK=`expr $NK | cut -c4-`
 NHET=`grep NHET= $FILE_CHEMSCHEME`
 NHET=`expr $NHET | cut -c6-`
 echo '>> '$CHEMSCHEME' IS A VALID CHEMICAL SCHEME!'
 echo ' > NUMBER OF TRANSPORTED SPECIES : '$NTRA1
 echo ' > NUMBER OF TRANSIENTS AND SPECIES THAT ARE NOT TRANSPORTED : '$NSLS1
 echo ' > NUMBER OF PHOTO- AND THERMOCHEMICAL REACTIONS : '$NK
 echo ' > NUMBER OF HETEROGENEOUS CHEMISTRY REACTIONS : '$NHET
else
 echo '>> NOT A VALID CHEMSCHEME : ABORT!'
 echo
 exit
fi
#
if [ $CNETCDF != 1 ] ; then
#
if [ $CHEMSCHEME = CARIOLLE ] ; then
 # Cariolle chemistry
 echo ">> RETRIEVING LINEARISED OZONE COEFFICIENTS FROM : $PATH_CSTE/chm/"
 RESOL=90x47
 for DOM in $LISTDOM
 do
   cp $PATH_CSTE/chm/CARIO3PARAM_v2.8_"$DOM"L"$NIV" CARIOZO
   cp $PATH_CSTE/chm/CARIOCOPARAM_v1.2_"$DOM"L"$NIV" CARIOCO
   cp $PATH_CSTE/chm/CARIOHNO3PARAM_v1.2_"$DOM"L"$NIV" CARIOHNO3
#   cp $PATH_CSTE/chm/CARIOZO CARIOZO
 done
 echo "Retrieve ${CHEMSCHEME}_SURF from $PATH_CSTE/chm"
 cp $PATH_CSTE/chm/${CHEMSCHEME}"_SURF" CHEMSCHEME_SURF
else
 echo "Retrieve ${CHEMSCHEME}_SURF from $PATH_CSTE/chm"
 cp $PATH_CSTE/chm/${CHEMSCHEME}"_SURF" CHEMSCHEME_SURF
fi
#
if [ $MOCFG = T42 ] ; then
 echo ">> RETRIEVING Gauss T42 Grid definition : $PATH_CSTE/gauss/"
 cp $PATH_CSTE/gauss/GaussT42 Gauss
fi
if [ $MOCFG = T21 ] ; then
 echo ">> RETRIEVING Gauss T21 Grid definition : $PATH_CSTE/gauss/"
 cp $PATH_CSTE/gauss/GaussT21 Gauss
fi
echo
#####
#
# Photolyses
#
MM0=`expr $DATEIN | cut -c5-6`
if [ $CHEMSCHEME != TRACERS0 ] || [ $CHEMSCHEME != TRACERCO ] || [ $CHEMSCHEME != AEROSOLS ] || [ $CHEMSCHEME != CARIOLLE ] ; then
# echo ">> RETRIEVING MASTER PHOTOLYSES TABLE (jdata115) FROM : $PATH_CSTE/clim/"
# cp $PATH_CSTE/clim/Jdata${MM0}.bin jdata115
 if [ $STATION_INPUT != DELAGE ] ; then
  echo ">> RETRIEVING MASTER PHOTOLYSES TABLE (jdata115) FROM : $PATH_JDATA"
  cp $PATH_JDATA/Jdata${MM0}.bin jdata115
  echo
 else
  echo ">> RETRIEVING MASTER PHOTOLYSES TABLE (jdata115) ON DELAGE FROM : $PATH_JDATA"
  ftget $PATH_JDATA/Jdata${MM0}.bin jdata115
 fi
fi
#####
#
# Top boundaries
#
if [ $STATION = tori ] ; then
 echo '>> RETRIEVING TOP BOUNDARY CLIMATOLOGIES FROM : '$MUTILS
 MPREV=`expr $MM0 - 1`
 if [ $MPREV -le 9 ] ; then
  MMPREV=0$MPREV
 else
  MMPREV=$MPREV
 fi
 if [ $MPREV -eq 0 ] ; then
  MMPREV=12
 fi
 MNEXT=`expr $MM0 + 1`
 if [ $MNEXT -le 9 ] ; then
  MMNEXT=0$MNEXT
 else
  MMNEXT=$MNEXT
 fi
 if [ $MNEXT -eq 13 ] ; then
  MMNEXT=01
 fi
 if [ $MNEXT -le 9 ] ; then
  MMNEXT=0$MNEXT
 fi
 cp $MUTILS/clim/CHEMA1995_19941998$MMPREV .
 cp $MUTILS/clim/CHEMA1995_19941998$MM0 .
 cp $MUTILS/clim/CHEMA1995_19941998$MMNEXT .
 cp $MUTILS/emi/EMISNIV_MOIS_${MMPREV}_* .
 cp $MUTILS/emi/EMISNIV_MOIS_${MM0}_* .
 cp $MUTILS/emi/EMISNIV_MOIS_${MMNEXT}_* .
 #
 #
 # RESPATIALISATION DES EMISSIONS (TEMPORAIRE)
 #
 cp $MUTILS/emi/HIGHRES.tar.gz .
 gunzip HIGHRES.tar.gz
 tar -xvf HIGHRES.tar
 rm -rf HIGHRES.tar
 echo '>> FICHIERS POUR RESPATIALISATION DES EMISSIONS :'
 ls -lrt *WEEK
 ls -lrt PROP.URBANIS*
else
 if [ $NIV -eq 47 ] ; then
  echo '>> RETRIEVING TOP BOUNDARY CLIMATOLOGIES (topbd) FROM : '$PATH_CSTE/clim/
  cp $PATH_CSTE/clim/topbd topbd
  echo
 fi
fi
fi
fi
