#!/bin/ksh
#
# Function timesec (the main script follows)
# Purpose   : Return seconds elapsed between two dates
# Interface  nsec=`timesec $DATEIN $DATEOUT`
#
timesec(){
DATESTART=$1
DATEEND=$2
YY0=`expr $DATESTART | cut -c1-4`
MM0=`expr $DATESTART | cut -c5-6`
JJ0=`expr $DATESTART | cut -c7-8`
HH0=`expr $DATESTART | cut -c9-10`
YY1=`expr $DATEEND | cut -c1-4`
MM1=`expr $DATEEND | cut -c5-6`
JJ1=`expr $DATEEND | cut -c7-8`
HH1=`expr $DATEEND | cut -c9-10`
set -A tabbis 0 31 28 31 30 31 30 31 31 30 31 30 31

TEST=FALSE

if [ $YY0 -eq $YY1 ] ; then
   (( YY1+=1 ))
   TEST=TRUE
fi

if [ $YY0 -gt $YY1 ] ; then
   echo -1
   exit
fi

idateconvtot=0

YY=`expr $YY0 + 1`
while [ $YY -lt $YY1 ] ; do
    if [ `expr $YY % 4` -eq 0 ] ; then
	let idateconvtot+=31622400
    else
	let idateconvtot+=31536000
    fi
    (( YY+= 1 ))
done

if [ `expr $YY0 % 4` -eq 0 ] ; then
    tabbis[2]=29
fi

MM=$MM0
while [ $MM -le 12 ] ; do
    monthsecs=`expr ${tabbis[$MM]} \* 86400`
    let idateconvtot+=$monthsecs
    (( MM+=1 ))
done

if [ `expr $YY0 % 4` -eq 0 ] ; then
    tabbis[2]=28
fi

if [ `expr $YY1 % 4` -eq 0 ] ; then
    tabbis[2]=29
fi

MME=`expr $MM1 - 1`
MM=1
while [ $MM -le $MME ] ; do
    monthsecs=`expr ${tabbis[$MM]} \* 86400`
    let idateconvtot+=$monthsecs
    (( MM+=1 ))
done

if [ `expr $YY1 % 4` -eq 0 ] ; then
    tabbis[2]=28
fi

ddiff=`expr $JJ1 - $JJ0`
ddiff=`expr $ddiff \* 86400`

hdiff=`expr $HH1 - $HH0`
hdiff=`expr $hdiff \* 3600`

#mdiff=`expr $MN1 - $MN0`
#mdiff=`expr $mdiff \* 60`

let idateconvtot+=$ddiff
let idateconvtot+=$hdiff
#let idateconvtot+=$mdiff

if [ $TEST = TRUE ] ; then

    if [ $MM1 -le 2 ] ; then

	if [ `expr $YY0 % 4` -eq 0 ] ; then
	   (( idateconvtot-=31622400 ))
	else
	   (( idateconvtot-=31536000 ))
	fi
	
    else

	if [ `expr $YY1 % 4` -eq 0 ] ; then
	   (( idateconvtot-=31622400 ))
	else
	   (( idateconvtot-=31536000 ))
	fi
	
    fi

    (( YY1-=1 ))
    TEST=FALSE

fi

if [ $idateconvtot -lt 0 ] ; then
    idateconvtot=-1
fi

echo $idateconvtot
}

#
# END OF TIMESEC FUNCTION
#
# BEGIN OF THE SCRIPT
#
# ARGUMENTS :
# getdates_AP.sh DATEIN DATEOUT nscy nout [NDAYSPLIT]
#  DATEIN : starting date in yyyymmddhh
#  DATEOUT: end date in yyyymmddhh
#  nscy   : forcing file frequency in hours
#  nout   : task (cf. later)
#  NDAYSPLIT: only for nout=5 number of days in a single job
#
#
# nout = 0 : prepare dates for FM files
#            (compute or retrieve forcings ; called by jobMCG)
# nout = 1 : prepare dates for SM files
#            (compute or retrieve forcings ; called by jobMCG)
# nout = 2 : prepare dates for HM files
#            (no model run, retrieve history files ; called by jobMCG)
# nout = 3 : prepare dates for HM files
#            (compute statistics, time profiles ; called by jobHM)
# nout = 4 : prepare dates for SM files
#            (compute statistics, time profiles ; called by jobSM)
# nout = 5 : prepare dates for automatic re-start
#            (called by jobMCG)
# nout = 6 : prepare dates for FM files for mode "FORC1"
#            (called by jobMCG)
# nout = 7 : prepare dates for SM files (4 consecutive days ;
#            called by sumocage)
#


DATEIN=$1
DATEOUT=$2
nscy=$3
nout=$4
NDAYSPLIT=$5

if [ $nscy -le 0 ] ; then
  echo "NOT A VALID CYCLE LENGTH : ABORT!"
  exit
fi
nsec=`timesec $DATEIN $DATEOUT`

DATESINV=FALSE
offset=0

if [ $nsec -lt 0 ] ; then
    DATESINV=TRUE
    nsec=`timesec $DATEOUT $DATEIN`
    offset=`expr $nsec \/ 3600`
    offset=`expr $offset % $nscy`
    let offset-=$nscy
    offset=`expr $offset % $nscy`
    DATETMP=$DATEOUT
    DATEOUT=$DATEIN
    DATEIN=$DATETMP
fi

YY0=`expr $DATEIN | cut -c1-4`
MM0=`expr $DATEIN | cut -c5-6`
JJ0=`expr $DATEIN | cut -c7-8`
HH0=`expr $DATEIN | cut -c9-10`
YY1=`expr $DATEOUT | cut -c1-4`
MM1=`expr $DATEOUT | cut -c5-6`
JJ1=`expr $DATEOUT | cut -c7-8`
HH1=`expr $DATEOUT | cut -c9-10`

set -A tabbis 0 31 28 31 30 31 30 31 31 30 31 30 31

if [ `expr $YY0 % 4` -eq 0 ] ; then
    tabbis[2]=28
fi

if [ $nout -eq 7 ] ; then
    nout=1
    HH0=12
    HH1=12
    JJ1=`expr $JJ0 + 3`
    if [ $JJ1 -gt ${tabbis[$MM0]} ] ; then
	let JJ1-=${tabbis[$MM0]}
	MM1=`expr $MM0 + 1`
	if [ $MM1 -gt 12 ] ; then
	    MM1=1
	    (( YY1+=1 )) 
	fi
    fi
fi

if [ $nscy -ge 672 ] ; then
    echo "MORE THAN A MONTH BETWEEN TWO FILES : ABORT!"
    exit
fi
    
if [[ $nout -eq 0 || $nout -eq 1 || $nout -eq 6 ]] ; then
#    if [[ $nscy -ne 3 && $nscy -ne 6 ]] ; then
#	echo "NOT A VALID CYCLE LENGTH : ABORT!"
#	exit
#    fi
    a=b
fi

if [ $nout -eq 4 ] ; then
    if [ `expr $nscy % 24` -ne 0 ] ; then
	echo "NOT A VALID CYCLE LENGTH : ABORT!"
	exit	
    fi
    HH0=0
fi

if [[ $nout -eq 0 || $nout -eq 1 || $nout -eq 6 ]] ; then
    ihour=`expr $HH0 \/ $nscy`
    ihour=`expr $ihour \* $nscy`
    ihour=`expr $ihour - $nscy`
fi

if [[ $nout -eq 2 || $nout -eq 3 || $nout -eq 4 || $nout -eq 5 ]] ; then
ihour=$HH0
fi

iday=$JJ0
imonth=$MM0
iyear=$YY0

idayold=0
idayold2=$iday

if [[ $nout -eq 3 && $DATESINV != TRUE  ]] ; then
    echo "$iyear""$imonth""$iday""$ihour"
fi

if  [[ $nout -eq 2 && $DATESINV = TRUE ]] ; then
    echo "$iyear""$imonth""$iday""$ihour"
fi

if [[ $nout -eq 2 || $nout -eq 3 ]] ; then
    let ihour+=$offset
fi

if  [ $nout -eq 4 ] ; then
          echo "$iyear""$imonth""$iday"
fi

if [ $nout -eq 5 ] ; then
    echo "$iyear""$imonth""$iday""$ihour"
    nscy=`expr $NDAYSPLIT \* 24`
fi

idate0=$DATEIN
idate1=$DATEOUT
idateloop="0000000000"

while [ $nsec -gt 0 ] ; do

    if [ $nout -eq 6 ] ; then
	nsec=`timesec $idate0 $idateloop`
	if [ $nsec -ge 172800 ] ; then
	    nscy=6
	fi
    fi
    
    imin=0
    let ihour+=$nscy
    
    if [ $ihour -ge 24 ] ; then
	thour=`expr $ihour \/ 24`
	let iday+=$thour
	ihour=`expr $ihour % 24`
    fi
    
    if [ $iday -gt ${tabbis[$imonth]} ] ; then
	let iday-=${tabbis[$imonth]}
    (( imonth+=1 )) 
    fi
    
    if [ $imonth -gt 12 ] ; then
	(( iyear+=1 ))
	imonth=1
    fi
    
    if [[ $nout -eq 0 || $nout -eq 6 ]] ; then
	if [ $imonth -le 9 ] ; then
	    imonth="0"`expr $imonth \* 1`
	fi
	if [ $iday -le 9 ] ; then
	    iday="0"`expr $iday \* 1`
	fi
	if [ $ihour -le 9 ] ; then
	    ihour="0"`expr $ihour \* 1`
	fi
	echo "$iyear""$imonth""$iday""$ihour"
    fi
    
    if [ $nout -eq 1 ] ; then
	if [[ $iday -ne $idayold && $ihour -ne 0 ]] ; then
	    if [ $imonth -le 9 ] ; then
		imonth="0"`expr $imonth \* 1`
	    fi
	    if [ $iday -le 9 ] ; then
		iday="0"`expr $iday \* 1`
	    fi
	    echo "$iyear""$imonth""$iday"
	    idayold=$iday
	fi
    fi
    
    if [ $imonth -le 9 ] ; then
	imonth="0"`expr $imonth \* 1`
    fi
    if [ $iday -le 9 ] ; then
	iday="0"`expr $iday \* 1`
    fi
    if [ $ihour -le 9 ] ; then
	ihour="0"`expr $ihour \* 1`
    fi
    idateloop="$iyear""$imonth""$iday""$ihour"
    nsec=`timesec $idateloop $idate1`
    
    if [ $nsec -gt 0 ] ; then
	if [ $nout -eq 2 ] ; then
	    if [[ $nsec -ne 0 || $DATESINV != TRUE ]] ; then
		if [ $imonth -le 9 ] ; then
		    imonth="0"`expr $imonth \* 1`
		fi
		if [ $iday -le 9 ] ; then
		    iday="0"`expr $iday \* 1`
		fi
		if [ $ihour -le 9 ] ; then
		    ihour="0"`expr $ihour \* 1`
		fi
		echo "$iyear""$imonth""$iday""$ihour"
	    fi
	fi
    fi

    if [ $nsec -gt 0 ] ; then
	if [ $nout -eq 5 ] ; then
	    if [ $imonth -le 9 ] ; then
		imonth="0"`expr $imonth \* 1`
	    fi
	    if [ $iday -le 9 ] ; then
		iday="0"`expr $iday \* 1`
	    fi
	    if [ $ihour -le 9 ] ; then
		ihour="0"`expr $ihour \* 1`
	    fi
	    echo "$iyear""$imonth""$iday""$ihour"
	    echo "$iyear""$imonth""$iday""$ihour"
	fi
    fi

    if [ $nsec -gt 0 ] ; then
	if [ $nout -eq 4 ] ; then
	    if [ $iday -ne $idayold2 ] ; then
		if [ $imonth -le 9 ] ; then
		    imonth="0"`expr $imonth \* 1`
		fi
		if [ $iday -le 9 ] ; then
		    iday="0"`expr $iday \* 1`
		fi
		echo "$iyear""$imonth""$iday"
		idayold2=$iday
	    fi
	fi
    fi
    
    if [ $nout -eq 3 ] ; then
	if [ $imonth -le 9 ] ; then
	    imonth="0"`expr $imonth \* 1`
	fi
	if [ $iday -le 9 ] ; then
	    iday="0"`expr $iday \* 1`
	fi
	if [ $ihour -le 9 ] ; then
	    ihour="0"`expr $ihour \* 1`
	fi
	echo "$iyear""$imonth""$iday""$ihour"
    fi

done

if [ $nout -eq 5 ] ; then
    echo $DATEOUT
fi

if [ $nsec -ne 0 ] ; then
    if [[ $nout -eq 2 && $DATESINV != TRUE ]] ; then
	echo $DATEOUT
    fi
fi
