#!/bin/ksh
#
#============#
# Run MOCAGE #
#============#
#
echo
echo '========================'
echo '>>> Script run_mocage.sh'
echo '========================'
#
#
if [ $CRUN -eq 1 ] ; then
 if [[ $STATION_INPUT = DELAGE && $STATION = tori ]] ; then
  mv ${FTDIR}/$JOBID/* $PATH_RUN
 fi
 cd $PATH_RUN
 cp namelist_param paradi_"$CHEMSCHEME$NDOM"DOM+$DATEIN+$DATEOUT
 ls -l
 echo
 echo "DATEIN="$DATEIN
 echo "DATEOUT="$DATEOUT
#
if [ $STATION = tori ] ; then
 #
 # Fabrication de la condition a la limite sup
 # (Utile en 47 Niveaux)
 #
 if [ $CNETCDF != 1 ] ; then
  if [ $NIV -eq 47 ] ; then
$MUTILS/exe/MKTOPBD << EOF
$DATEIN
$DATEOUT
EOF
  fi
 fi
 #
 # Si on commence une simulation depuis la clim
 # Fabrication d'un fichier de depart coherent
 # Initializations
 #
 if [ $NSTART -eq 0 ] ; then
  $MUTILS/exe/INIT
 fi
 #
 export F_FMTBUF=300000
 #export F_FMTBUF=131072
 export F_SETBUF6=0
 export F_SETBUF0=0
 export F_ERRCNT=1
 #export F_PROGINF=detail
 export F_RECLUNIT=word
 export F_SETBUF=32768
 export F_FTRACE=FMT2
fi
#
echo
echo '######################################################'
echo '#   __  __   _____   _____   _____   _____   _____   #'
echo '#  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|  #'
echo '#  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|  #'
echo '#  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|  #'
echo '#                                                    #'
echo '###| Version 1.0 | Meteo-France | CNRM/GMGEC/ERAM |###'
echo
if [ $CASSIM = 0 ] ; then
  echo "  PERIOD OF JOB : from $DATEIN to $DATEOUT "
  echo '  EXPERIMENT : DIRECT RUN'
  echo "               Schema $CHEMSCHEME"
  echo "               Niveaux $NIV"
  echo "  RESULTS STORED in:"
  echo "         $PATH_OUTPUT"
else
  echo "  PERIOD OF JOB : from $DATEIN to $DATEOUT "
  echo '  EXPERIMENT : 3DFGAT'
  echo "               Schema $CHEMSCHEME"
  echo "               Niveaux $NIV"
  echo "               Observations $TYPEOBS"
  echo "  RESULTS STORED in:"
  echo "         $PATH_OUTPUT"
fi
echo
echo '######################################################'
echo '#                                                    #'
echo '###| Version Maquette ADOMOCA | CNRM-CERFACS & al |###'
echo '#                                                    #'
echo '######################################################'
echo

# Separate dates and other parameters in the namelist file
cp $PATH_WORK/namelist .
cp namelist namelist+$DATEIN+$DATEOUT
set -A nb_lines `wc -l namelist` 
nb_lines_tail=`expr ${nb_lines[0]} - 8`
tail -$nb_lines_tail namelist > namelist_tail

# Split the period by 1 day
$PATH_TOOLS/getdates_AP.sh $DATEIN $DATEOUT $NHCY 5 1 > outdates_mocage

set -A DATES `cat outdates_mocage`
rm outdates_mocage
NTOTDAYS=`expr ${#DATES[*]} / 2`
n1=0
n2=1

while test ${n2} -le ${#DATES[*]}
do
#
YY0=`expr ${DATES[$n1]} | cut -c1-4`
MM0=`expr ${DATES[$n1]} | cut -c5-6`
JJ0=`expr ${DATES[$n1]} | cut -c7-8`
HH0=`expr ${DATES[$n1]} | cut -c9-10`
YY1=`expr ${DATES[$n2]} | cut -c1-4`
MM1=`expr ${DATES[$n2]} | cut -c5-6`
JJ1=`expr ${DATES[$n2]} | cut -c7-8`
HH1=`expr ${DATES[$n2]} | cut -c9-10`
#
rm namelist
echo $YY0 > tempo
echo $MM0 >> tempo
echo $JJ0 >> tempo
echo $HH0 >> tempo
echo $YY1 >> tempo
echo $MM1 >> tempo
echo $JJ1 >> tempo
echo $HH1 >> tempo
cat tempo namelist_tail > namelist
rm tempo
#
NUMDAY=`expr $n1 / 2 + 1`
echo
echo '>> DAY : '$NUMDAY' / '$NTOTDAYS
echo '>> RUN MOCAGE BETWEEN '${DATES[$n1]}' AND '${DATES[$n2]}
#

if [ $CNETCDF != 1 ] ; then
    for DOM in $LISTDOM
      do
      inifile=HM"$DOM"+"$YY0""$MM0""$JJ0""$HH0"
      if [ ! -s $inifile ] ; then
	  echo "MISSING INITIAL CONDITION FILE $inifile"
	  echo "STOPPING ITERATIONS"
	  exit 17    
      fi
    done
fi


if [ $STATION = tori ] ; then
 echo "Running on tori (NEC station at Meteo-France)"
 echo
 if [ $CASSIM = 0 ] ; then
  if [ $TOTAL -ne 1 ] ; then
   export OMP_NUM_THREADS=4
   time ./MOCAGE
  else
   totalview MOCAGE
  fi
 else
  \rm -f palm_driver.out
  \rm -f palm_log.out
  \rm -f lbfgs.out
  \rm -f palm_proc_00[1-9].out
  export F_ERRCNT=1
  export F_RECLUNIT=word
  export F_SETBUF=32768
  export OMP_NUM_THREADS=2
  if [ $NUMDAY -le 9 ] ; then
      NSPLIT="0"$NUMDAY
  else
      NSPLIT=$NUMDAY
  fi
  export MPIRUN_SPLIT="day$NSPLIT"
  ~mrpm631/public/mpi/xmpirun -xtr -np 3 ./MOCAGE
 fi
fi
#
if [ $STATION = aerosv1 ] ; then
 echo "Running on aerosv1 (PC station at LA)"
 echo
 if [ $CASSIM = 0 ] ; then
  ./MOCAGE
 else
  lamboot
  mpirun -np 3 MOCAGE
  lamhalt
 fi
fi
#
if [ $STATION = CERFACS ] ; then
    if [ $HOST = stratus.cerfacs.fr ] ; then
	echo Running on CERFACS quadricore stratus
	if [ $CASSIM = 0 ] ; then
	    export NCPUS=4
	    ./MOCAGE
	else
	    export NCPUS=2
	    mpirun -np 3 MOCAGE
	fi
    else
	echo Running on CERFACS PC
	if [ $CASSIM = 0 ] ; then
	    export NCPUS=2
	    ./MOCAGE
	else
	    lamboot
#	    $PATH_JOBS/memst > ~/run_mem.dat &
#	    mempid=$!
	    mpirun -np 3 MOCAGE
#	    kill -9 $mempid
	    lamhalt
	fi
    fi
fi
#
if [ $STATION = CRAY ] ; then
mpiexec -n 3 $XD1LAUNCHER ./MOCAGE
fi
#
cp namelist namelist_day$NUMDAY
#
DATEI=${DATES[$n1]}
DATEO=${DATES[$n2]}

#cp namelist_param paradi_"$CHEMSCHEME$NDOM"DOM+$DATEI+$DATEO

if [ $CASSIM != 0 ] ; then
    cp  assimilated_species assimilated_species+$DATEI+$DATEO 
    cp  MOCAGE.ppl MOCAGE+$DATEI+$DATEO.ppl 
    LIST=`ls palm_driver.out lbfgs.out palm_proc_00[1-9].out palm_log.out`
    for i in $LIST; do
	tgt=`echo $i | sed "s/\(.*\)\.\([^\.]*\)$/\1+$DATEI+$DATEO.\2/"`   
	mv $i $tgt
    done
fi

n1=`expr $n1 + 2`
n2=`expr $n2 + 2`
#

done

rm namelist_tail

ls -l

# Write results in the exp_description.log file
###############################################

rm -f exp_description.log

if [ $CASSIM = 0 ] ; then
  cat > exp_description.log << EOF

######################################################
#   __  __   _____   _____   _____   _____   _____   #
#  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|  #
#  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|  #
#  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|  #
#                                                    #
######################################################

  END OF JOB : $DATEIN TO $DATEOUT
  EXPERIMENT : DIRECT RUN
               Scheme $CHEMSCHEME
               $NIV levels
  RESULTS STORED in:
         $PATH_OUTPUT

######################################################
#                                                    #
###| Version Maquette ADOMOCA | CNRM-CERFACS & al |###
#                                                    #
######################################################

Parameters:

Forcing reading frequency (NHCY): $NHCY
Number of domains (NDOM): $NDOM
Heterogeneous chemistry (SOLID): $SOLID
Surfaces processes (SURF): $SURF
Parametrized transport (PRMTRSP): $PRMTRSP
Clouds (CLOUD): $CLOUD
EOF
else
  cat > exp_description.log << EOF

######################################################
#   __  __   _____   _____   _____   _____   _____   #
#  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|  #
#  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|  #
#  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|  #
#                                                    #
######################################################

  END OF JOB : Starting at $DATEIN
  EXPERIMENT : 3DFGAT
               Scheme $CHEMSCHEME
               $NIV levels
               Observations: $TYPEOBS
  RESULTS STORED in:
         $PATH_OUTPUT

######################################################
#                                                    #
###| Version Maquette ADOMOCA | CNRM-CERFACS & al |###
#                                                    #
######################################################

Parameters:

Forcing reading frequency (NHCY): $NHCY
Number of domains (NDOM): $NDOM
Heterogeneous chemistry (SOLID): $SOLID
Surfaces processes (SURF): $SURF
Parametrized transport (PRMTRSP): $PRMTRSP
Clouds (CLOUD): $CLOUD
EOF
fi

if [ $OBS = .TRUE. ] ; then
 for i in HOBS*; do
  echo "Obs file $i" >> exp_description.log
  echo "Containing" >> exp_description.log
  types=`head -1 $i | awk '{print $2}'`
  lasttypeline=`expr $types + 1`
  head -$lasttypeline $i | tail -$types >> exp_description.log
  echo ' ' >> exp_description.log
 done
fi
if [ $CASSIM = 1 ] ; then
 for i in HDAT*; do
  echo "Assimilated data file $i" >> exp_description.log
  echo "Containing" >> exp_description.log
  types=`head -1 $i | awk '{print $2}'`
  lasttypeline=`expr $types + 1`
  head -$lasttypeline $i | tail -$types >> exp_description.log
  echo ' ' >> exp_description.log
 done
fi

if [[ $STATION_OUTPUT = DELAGE && $STATION = tori ]] ; then
 cd $FTDIR/$JOBID
 mv $PATH_RUN/HM* .
 mv $PATH_RUN/paradi* .
 mv $PATH_RUN/namelist+$DATEIN+$DATEOUT .
 mv $PATH_RUN/*.log .
 mv $PATH_RUN/MOCAGE .
 if [ $OBS = .TRUE. ] ; then
  mv $PATH_RUN/HDIAG* .
 fi
 if [ $CASSIM = 1 ] ; then
  mv $PATH_RUN/*.ppl .
  mv $PATH_RUN/*.out .
  mv $PATH_RUN/assim* .
  mv $PATH_RUN/DX*.nc .
  mv $PATH_RUN/HSTAT*.nc .
  mv $PATH_RUN/HDOM* .
 fi
fi


if [ $CSMAIL -eq 1 ] ; then
echo '>> SENDING eMAIL TO : '$USRMEL

if [ $CASSIM = 0 ] ; then
cat > imel << EOF
Subject: MOCAGE V3 DIRECT RUN $CHEMSCHEME $NIV niv a fini

######################################################
#   __  __   _____   _____   _____   _____   _____   #
#  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|  #
#  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|  #
#  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|  #
#                                                    #
######################################################

  PERIOD OF JOB : from $DATEIN to $DATEOUT
  EXPERIMENT    : DIRECT RUN
                  Scheme $CHEMSCHEME
                  $NIV levels
  RESULTS STORED in:
         $PATH_OUTPUT

######################################################
#                                                    #
###| Version Maquette ADOMOCA | CNRM-CERFACS & al |###
#                                                    #
######################################################

EOF
else
cat > imel << EOF
Subject: MOCAGE V3 3DFGAT $CHEMSCHEME $NIV niv a fini

######################################################
#   __  __   _____   _____   _____   _____   _____   #
#  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|  #
#  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|  #
#  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|  #
#                                                    #
######################################################

  PERIOD OF JOB : from $DATEIN to $DATEOUT
  EXPERIMENT    : 3DFGAT
                  Schema $CHEMSCHEME
                  Niveaux $NIV
                  Observations $TYPEOBS
  RESULTS STORED in:
         $PATH_OUTPUT

######################################################
#                                                    #
###| Version Maquette ADOMOCA | CNRM-CERFACS & al |###
#                                                    #
######################################################

EOF
fi

  if [ `hostname` = majorana ] ; then
    if [ $CASSIM = 0 ] ; then
        /home/andrea/bin/sendEmail -f "Majorana RUN <andrea@cerfacs.fr>" -t "$USRMEL" -u "Mocage $CHEMSCHEME $NIV Niv DRY RUN Ended" -s "smtp.fastwebnet.it" < imel
    else
        /home/andrea/bin/sendEmail -f "Majorana RUN <andrea@cerfacs.fr>" -t "$USRMEL" -u "Mocage $CHEMSCHEME $NIV Niv 3DFGAT with $TYPEOBS Ended" -s "smtp.fastwebnet.it" < imel
    fi
  fi

  if [ $STATION = tori ] ; then
    cat imel | /usr/sbin/sendmail $USRMEL
  fi

fi

fi

